(*******************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which
typically will have the same name as this file except ending in
".nb" instead of ".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent
to using the Evaluate Initialization Cells menu command in the front
end.

DO NOT EDIT THIS FILE.  This entire file is regenerated
automatically each time the parent Notebook file is saved in the
Mathematica front end.  Any changes you make to this file will be
overwritten.
***********************************************************************)

BeginPackage["Lautrup`NDSolveBV`","Utilities`FilterOptions`"];
ClearAttributes[NDSolveBV,{Protected,ReadProtected}];

(* Messages *)
NDSolveBV::usage="Numerical differential equation boundary value solver.\n
    NDSolveBV[eqs,vars,intv,init,opts] where\n
    \teqs\tcontains differential equations and boundary values with variables for the unknown ones\n
    \tvars\tcontains the variable names";
 
NDSolveBV::lengtherror="The number of equations, boundary conditions, and variables must be the same (`1`)";\

NDSolveBV::intervalerror="Interval must be of the form {x,a,c,b} with c between a and b";\


JoinSolutions::usage="Option for NDSolveBV for solution as a single interpolating function";\


Begin["`Private`"];

(* Auxiliary functions *)
NumericListQ[x_]:=And@@Map[NumericQ,x];

(* Main module *)
NDSolveBV[eqs_,vars_,intv_,init_,opts___]:=
    Module[{x,a,c,b,n,ivars,aBV,bBV,odes,cvals,solac,solbc,sol,roots,args,
        varsac,varsbc,fun,sub,int,optsNDSolveBV,optsNDSolve,optsFindRoot,
        optsFunctionInterpolation}, 
      
      (* check inputs *)
      If[Length[intv]\[NotEqual] 4,Message[NDSolveBV::intervalerror]];
      x=intv[[1]];
      a=intv[[2]];
      c=intv[[3]];
      b=intv[[4]];
      If[\[Not](a\[LessEqual]c\[LessEqual]b||
              a\[GreaterEqual]c\[GreaterEqual]b),
        Message[NDSolveBV::intervalerror]];
      n=Length[vars];
      If[Length[eqs]\[NotEqual]3n||Length[init]\[NotEqual]n,
        Message[NDSolveBV::lengtherror,n]];
      
      (* check options *)
      optsNDSolve = FilterOptions[NDSolve,opts];
      optsFindRoot=FilterOptions[FindRoot,opts];
      optsNDSolveBV=FilterOptions[NDSolveBV,opts,Options[NDSolveBV]];
      optsFunctionInterpolation=FilterOptions[FunctionInterpolation,opts];
      
      (* construct boundary conditions *)
      aBV=Cases[eqs,_[a]\[Equal]_];
      bBV=Cases[eqs,_[b]\[Equal]_];
      odes=Complement[eqs,aBV,bBV];
      
      (* determine roots *)
      ivars=Map[First[#]&,init];
      solac[args_?NumericListQ]:=
        Apply[NDSolve,{odes\[Union]aBV,vars,{x,a,c},
                optsNDSolve}/.MapThread[#1\[Rule]#2&,{ivars,args}]]//First;
      solbc[args_?NumericListQ]:=
        Apply[NDSolve,{odes\[Union]bBV,vars,{x,c,b},
                optsNDSolve}/.MapThread[#1\[Rule]#2&,{ivars,args}]]//First;
      cvals=Map[#[c]&,vars];
      sol[args_?NumericListQ]:=(cvals/.solac[args])-(cvals/.solbc[args]);
      roots=Apply[FindRoot,{sol[ivars],init,optsFindRoot}];
      
      (* do integral *)
      args=ivars/.roots;
      varsac=solac[args];
      varsbc=solbc[args];
      
      (* construct output *)
      (*fun[f_,x_]:=If[x<c,f[x]/.varsac//Evaluate,f[x]/.varsbc//Evaluate];
        sub=Map[#[x_]->fun[#,x]&,vars];*)
      fun[f_]:=
        Evaluate[If[#<c,f[#]/.varsac//Evaluate,f[#]/.varsbc//Evaluate]]&;
      sub=Map[#->fun[#]&,vars];
      int[f_]:=
        Apply[FunctionInterpolation,{f[x]/.sub,{x,a,b},
            optsFunctionInterpolation}];
      If[JoinSolutions/.{optsNDSolveBV}, sub=Map[#\[Rule]int[#]&,vars]];
      {sub,roots}
      ];
Options[NDSolveBV]={JoinSolutions\[Rule]False};
End[];
SetAttributes[NDSolveBV,{Protected,ReadProtected}];
EndPackage[];





























