(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     19148,        719]*)
(*NotebookOutlinePosition[     19820,        742]*)
(*  CellTagsIndexPosition[     19776,        738]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["3. Gravity", "Title"],

Cell[BoxData[
    \(<< Lautrup`\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Earth data", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Radii", "Section"],

Cell[BoxData[
    \(a = \(EarthRadius // N\) // ToUnit[Kilo\ Meter]\)], "Input"],

Cell[BoxData[
    \(a\_1 = EarthCoreRadius // N\)], "Input"],

Cell[BoxData[
    \(a\_2 = a - a\_1 // ToUnit[Kilo\ Meter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Masses", "Section"],

Cell[BoxData[
    \(M = EarthMass // N\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(M\_1 = EarthCoreMass // N\)], "Input"],

Cell[BoxData[
    \(M\_2 = \(\(M\)\(-\)\(M\_1\)\(\ \ \ \ \)\)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Volumes", "Section"],

Cell[BoxData[
    \(V = \(\(4  \[Pi]\)\/3\) a\^3 // ToSI\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(V\_\(\(1\)\(\ \)\) = \(\(4  \[Pi]\)\/3\) a\_1\^3 // ToSI\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(V\_2 = V - V\_1\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Densities", "Section"],

Cell[BoxData[
    \(\[Rho]\_1 = M\_1\/V\_1 // ToSI\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\[Rho]\_2 = M\_2\/V\_2 // ToSI\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\[Rho]\_0 = M\/V // ToSI\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gravity at core boundary", "Section"],

Cell[BoxData[{
    \(\(G = GravitationalConstant // N;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity // N;\)\)}], "Input"],

Cell[BoxData[
    \(g\_1 = G  M\_1\/a\_1\^2 // ToSI\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(g\_1\/g\_0\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Core gravity at surface", "Section"],

Cell[BoxData[
    \(g\_10 = G  M\_1\/a\^2 // ToSI\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gravity plots", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Fetch parameters", "Section"],

Cell[BoxData[{
    \(\(a = 6371;\)\), "\n", 
    \(\(a\_1 = 3485;\)\), "\n", 
    \(\(\[Rho]\_1 = 10.9;\)\), "\n", 
    \(\(\[Rho]\_2 = 4.5;\)\), "\[IndentingNewLine]", 
    \(\(G = GravitationalConstant // NToSI;\)\), "\[IndentingNewLine]", 
    \(\(M = EarthMass // NToSI;\)\), "\[IndentingNewLine]", 
    \(\(M\_1 = EarthCoreMass // NToSI;\)\)}], "Input"],

Cell[BoxData[{
    \(\(rmax = 8000;\)\), "\n", 
    \(\(rmin\  = \ 1;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Earth data (from CRC96)", "Section"],

Cell[BoxData[
    \(\(depth = {0, 0.01`, 3, 3.01`, 21, 21.01`, 41, 61, 81, 101, 121, 171, 
          221, 271, 321, 371, 571, 871, 1171, 1471, 1771, 2071, 2371, 2671, 
          2886, 2886.01`, 2971, 3371, 3671, 4071, 4471, 4871, 5156, 5156.01`, 
          5371, 5771, 6071, 6371};\)\)], "Input"],

Cell[BoxData[
    \(\(density = {0.`, 1.02`, 1.02`, 2.8`, 2.8`, 3.49`, 3.51`, 3.52`, 3.48`, 
          3.44`, 3.4`, 3.37`, 3.34`, 3.37`, 3.47`, 3.59`, 3.95`, 4.54`, 
          4.67`, 4.81`, 4.96`, 5.12`, 5.31`, 5.45`, 5.53`, 9.96`, 10.09`, 
          10.63`, 11.`, 11.36`, 11.69`, 11.99`, 12.12`, 12.3`, 12.48`, 
          12.52`, 12.53`, 12.58`};\)\)], "Input"],

Cell[BoxData[
    \(\(pressure = {0, 0, 3, 3, 5, 5, 12, 19, 26, 33, 39, 56, 73, 89, 106, 
          124, 199, 328, 466, 607, 752, 903, 1061, 1227, 1352, 1352, 1442, 
          1858, 2154, 2520, 2844, 3116, 3281, 3281, 3385, 3529, 3592, 
          3617};\)\)], "Input"],

Cell[BoxData[
    \(\(gravity = {981, 981, 982, 982, 983, 983, 983, 984, 984, 984, 985, 
          987, 989, 991, 993, 994, 999, 997, 992, 991, 994, 1002, 1017, 1042, 
          1069, 1069, 1050, 953, 874, 760, 641, 517, 427, 427, 355, 218, 122, 
          0};\)\)], "Input"],

Cell[BoxData[
    \(\(radius = Last[depth] - depth;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Density plot", "Section"],

Cell[BoxData[
    \(\(fig0a = 
        ListPlot[Thread[List[radius, density]], PlotJoined \[Rule] True, 
          PlotStyle -> {Text["\<Core\>", {2000, 13}], 
              Text["\<Mantle\>", {5000, 7}]}, 
          AxesLabel -> {"\<r[km]\>", \
\*"\"\<\[Rho][g/\!\(cm\^3\)]\>\""}];\)\)], "Input"],

Cell[BoxData[
    \(densitymodel[r_] := 
      If[r < a\_1, \[Rho]\_1, If[r < a, \[Rho]\_2, 0]]\)], "Input"],

Cell[BoxData[
    \(\(fig0b = 
        Plot[densitymodel[r\ ], {r, 0, rmax}, 
          PlotStyle -> Dashing[{0.01}]];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig0a, 
          fig0b] // \[IndentingNewLine]Figure["\<gravity0\>"];\)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gravity plot", "Section"],

Cell[BoxData[
    \(\(fig1a = 
        ListPlot[Thread[List[radius, gravity]], PlotJoined \[Rule] True, 
          AxesLabel -> {"\<r[km]\>", \*"\"\<-g(r)[cm/\!\(s\^2\)]\>\""}, 
          PlotStyle -> {Text["\<Core\>", {1800, 800}], 
              Text["\<Mantle\>", {5000, 
                  1100}], \n\t\t\tText["\<Outside\>", {7200, 
                  1100}]}];\)\)], "Input"],

Cell[BoxData[{
    \(\(massmodel[r_] = 
        If[r < a\_1, \(4\/3\) \[Pi]\ \(r\^3\) \(10\^12\) \[Rho]\_1, 
          If[r < a, 
            M\_1 + \(4\/3\) \[Pi]\ \((r\^3 - a\_1\^3)\) \(10\^12\) \[Rho]\_2, 
            M]];\)\), "\n", 
    \(\(gravitymodel[r_] = \(-\ 10\^\(-4\)\) G 
           massmodel[r]\/r\^\(\(\ \)\(2\)\);\)\)}], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\(\(fig1b = 
        Plot[\(-gravitymodel[r]\), {r, rmin, rmax}, 
          PlotStyle -> {Dashing[{0.01}]}];\)\(\n\)
    \)\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\(fig1c = 
        Plot[\(10\^\(-4\)\) G\ M\_1\/r\^2, {r, a\_1, rmax}, 
          PlotStyle -> {Dashing[{0.005}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig4d = 
        Graphics[{Dashing[{0.02}], Line[{{3485, 0}, {3485, 1100}}], 
            Line[{{6371, 0}, {6371, 1100}}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig1a, fig1b, fig1c, 
          fig4d] // \[IndentingNewLine]Figure["\<gravity1\>"];\)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Potential plot", "Section"],

Cell[BoxData[
    \(\(gravityfit = 
        Interpolation[Thread[List[radius, \(-gravity\)]], 
          InterpolationOrder -> 1];\)\)], "Input"],

Cell[BoxData[
    \(potentialfit[
        r_] := \(-10\^\(-9\)\) G\ M\/a + \(10\^\(-5\)\) 
          NIntegrate[gravityfit[s], {s, r, a}, PrecisionGoal -> 1]\)], "Input"],

Cell[BoxData[
    \(\(fig4a = 
        Plot[\(-potentialfit[r]\), {r, 0, 
            a}, \n\t\tPlotStyle -> {Text["\<Core\>", {1800, 80}], 
              Text["\<Mantle\>", {5000, 
                  60}], \n\t\t\tText["\<Outside\>", {7500, 
                  40}]}];\)\)], "Input"],

Cell[BoxData[
    \(potentialmodel[r_] := 
      If[r < a, \(10\^\(-5\)\) 
            NIntegrate[gravitymodel[s], {s, r, a}, 
              PrecisionGoal -> 4] - \(10\^\(-9\)\) G\ M\/a, \(-10\^\(-9\)\) 
          G\ M\/r]\)], "Input"],

Cell[BoxData[
    \(\(fig4b = 
        Plot[\(-potentialmodel[r]\), {r, 0, rmax}, \t
          PlotStyle -> {Dashing[{0.01}]}];\)\)], "Input"],

Cell[BoxData[
    \(corepotential[r_] := \(-10\^\(\(-9\)\(\ \)\)\) 
        G\ If[r < 
            a\_1, \(-\(2\/3\)\) \[Pi]\ \((r\^\(\(\ \)\(2\)\) - 
                  a\_1\^2)\) \(\[Rho]\_1\) 10\^12 + M\_1\/a\_1, 
          M\_1\/r]\)], "Input"],

Cell[BoxData[
    \(\(fig4c = 
        Plot[\(-corepotential[r]\), {r, 0, rmax}, 
          PlotStyle -> {Dashing[{0.005}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(\(\n\)\(fig4d = 
        Graphics[{Dashing[{0.02}], Line[{{3485, 0}, {3485, 110}}], 
            Line[{{6371, 0}, {6371, 110}}]}];\)\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig4a, fig4b, fig4c, fig4d, PlotRange -> {0, 120}, 
          AxesLabel -> {"\<r[km]\>", \*"\"\<-\[CapitalPhi](r)[(km/s\!\(\()\^2\
\)\)]\>\""}] // \[IndentingNewLine]Figure["\<gravity4\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ratio of gravity to electricity in hydrogen", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Fetch parameters", "Section"],

Cell[BoxData[{
    \(\(m = ElectronMass // N;\)\), "\[IndentingNewLine]", 
    \(\(M = ProtonMass // N;\)\), "\[IndentingNewLine]", 
    \(\(\[Epsilon]\_0 = VacuumPermittivity // N;\)\), "\[IndentingNewLine]", 
    \(\(e = ElectronCharge // N;\)\), "\[IndentingNewLine]", 
    \(\(a = BohrRadius // N;\)\), "\[IndentingNewLine]", 
    \(\(G = GravitationalConstant // N;\)\)}], "Input"],

Cell[BoxData[
    \(F\_g = G\ \(m\ M\)\/a\^2\)], "Input"],

Cell[BoxData[
    \(F\_e = 
      e\^2\/\(4\ \[Pi]\ \(\[Epsilon]\_0\) a\^2\) // ToUnit[Newton]\)], "Input"],

Cell[BoxData[
    \(F\_g\/F\_e\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Earth-Moon field", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Parameters", "Section"],

Cell["Masses in arbitrary units", "Text"],

Cell[BoxData[{
    \(\(M = EarthMass\ 10\^\(-22\) // NToSI;\)\), "\[IndentingNewLine]", 
    \(\(m = MoonMass\ 10\^\(-22\) // NToSI;\)\)}], "Input",
  InitializationCell->False],

Cell["Distances in units of 1000 kilometer ", "Text",
  InitializationCell->False],

Cell[BoxData[{
    \(\(R = EarthRadius\ 10\^\(-6\) // NToSI;\)\ \ \  (*\ 
      1000\ kilometers\ *) \), "\[IndentingNewLine]", 
    \(\(r = MoonRadius\ 10\^\(-6\) // NToSI;\)\), "\n", 
    \(\(d = MoonOrbitRadius\ 10\^\(-6\) // NToSI;\)\)}], "Input",
  InitializationCell->False],

Cell["Frame dimensions in units of 1000 kilometer", "Text"],

Cell[BoxData[{
    \(\(x1 = \(-200. \);\)\), "\[IndentingNewLine]", 
    \(\(x2 = 600. ;\)\), "\n", 
    \(\(y1 = \(-300. \);\)\), "\[IndentingNewLine]", 
    \(\(y2 = 300. ;\)\)}], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Potential contours", "Section"],

Cell[BoxData[
    \(\(\[CapitalPhi][x_, y_] = \(-\ \(M\/\@\(x\^2 + y\^2\)\)\) - \ 
          m\/\@\(\((x - d)\)\^2 + y\^2\);\)\)], "Input",
  InitializationCell->False],

Cell[BoxData[{
    \(\(gx[x_, y_] = \(-\[PartialD]\_x \[CapitalPhi][x, y]\);\)\), "\n", 
    \(\(gy[x_, y_] = \(-\[PartialD]\_y \[CapitalPhi][x, y]\);\)\)}], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\(\(\[CapitalPhi]0 = \[CapitalPhi][x, 0] /. 
        FindRoot[gx[x, 0], {x, 300}]\n
    \(d\[CapitalPhi] = \ 0.2;\)\)\(\t\t\)\)\)], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\(fig2a = 
        ContourPlot[\[CapitalPhi][x, y], {x, x1, x2}, {y, y1, y2}, 
          PlotPoints -> 200, ContourShading -> False, 
          AspectRatio -> Automatic, ContourStyle -> {Dashing[{0.005}]}, 
          Contours -> 
            Table[\[CapitalPhi]0 + p, {p, \(-2\) d\[CapitalPhi], 
                3  d\[CapitalPhi], d\[CapitalPhi]}]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Field lines", "Section"],

Cell["Parametric solution", "Text"],

Cell[BoxData[{
    \(\(t1 = 0;\)\), "\[IndentingNewLine]", 
    \(\(t2 = 150000;\)\)}], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(sol[x0_, 
        y0_] := {x[t], y[t]} /. 
          NDSolve[{\(x'\)[t] == \(-gx[x[t], y[t]]\), \(y'\)[
                  t] == \(-gy[x[t], y[t]]\), x[0] == x0, y[0] == y0}, {x, 
              y}, {t, t1, t2}] // First\)], "Input",
  InitializationCell->False],

Cell["\<\
Start field lines equi-angular on a sphere concentric with  earth\
\>", "Text"],

Cell[BoxData[{
    \(\(i0 = 100;\)\), "\n", 
    \(\(scale = 10;\)\), "\n", 
    \(\(tblearth = 
        Table[sol[scale\ R\ Cos[\(2  \[Pi]\ i\)\/i0], 
            scale\ R\ Sin[\(2  \[Pi]\ i\)\/i0]], {i, i0}];\)\)}], "Input",
  InitializationCell->False],

Cell["\<\
Do the same for the moon with the correct ratio of field strengths\
\>", "Text"],

Cell[BoxData[{
    \(\(ratio = \(-\(gx[\(-scale\)\ R, 0]\/gx[d + scale\ r, 
                0]\)\);\)\), "\n", 
    \(\(j0 = Round[i0\/ratio];\)\), "\n", 
    \(\(tblmoon = 
        Table[sol[d + scale\ r\ \ Cos[\(2  \[Pi]\ j\)\/j0], 
            scale\ r\ Sin[\(2  \[Pi]\ j\)\/j0]], {j, j0}];\)\), "\n", 
    \(\(lines = Join[tblearth, tblmoon];\)\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  InitializationCell->False],

Cell[BoxData[
    \(\(\(fig2b = 
        ParametricPlot[Evaluate[lines], {t, t1, t2}, Compiled -> False, 
          PlotRange -> {{x1, x2}, {y1, y2}}, Frame -> True, 
          AspectRatio \[Rule] Automatic];\)\(\n\)
    \)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Final figure", "Section"],

Cell[BoxData[
    \(\(Show[fig2a, fig2b, 
          PlotRange -> {{x1, x2}, {y1, 
                y2}}] // \[IndentingNewLine]Figure["\<gravity2\>"];\)\)], \
"Input",
  InitializationCell->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Schwarzchild radii", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[{
    \(\(G = GravitationalConstant // N;\)\), "\[IndentingNewLine]", 
    \(\(c = SpeedOfLight // N;\)\)}], "Input"],

Cell[BoxData[
    \(r\_Earth = 2\ G\ EarthMass\/c\^2 // ToUnit[Millimeter]\)], "Input"],

Cell[BoxData[
    \(r\_Sun = 2  G\ SunMass\/c\^2 // ToUnit[Kilometer]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Escape velocities", "Title",
  InitializationCell->False],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[
    \(\(G = GravitationalConstant;\)\)], "Input"],

Cell[CellGroupData[{

Cell["Earth", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\(\(\@\(2  G\ EarthMass\/EarthRadius\) // ToUnit[Kilometer]\) // 
      N\)\(\n\)
    \)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sattellite speed", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    FractionBox[
      StyleBox["%",
        "Subsection"], \(\@2\)]], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mars", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\(\n\)\(\(\@\(2  G\ MarsMass\/MarsRadius\) // ToUnit[Kilometer]\) // 
      N\)\)\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moon", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\@\(2  G\ MoonMass\/MoonRadius\) // ToUnit[Kilo\ Meter]\) // 
      N\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sun", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\@\(2  G\ SunMass\/SunRadius\) // ToUnit[Kilo\ Meter]\) // 
      N\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Earth orbit", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\@\(2  G\ SunMass\/EarthOrbitRadius\) // ToUnit[Kilometer]\) // 
      N\)], "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moon orbit", "Subsection",
  InitializationCell->False],

Cell[BoxData[
    \(\(\@\(2  G\ EarthMass\/MoonOrbitRadius\) // ToUnit[Kilometer]\) // 
      N\)], "Input",
  InitializationCell->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["3.2 First cosmic velocity", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(a = EarthRadius;\)\)}], "Input"],

Cell[BoxData[
    \(\(r = a;\)\)], "Input"],

Cell[BoxData[
    \(v = \@\(\(g\_0\) a\^2\/r\) // ToUnit[Kilometer]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.3 Geostationary orbit", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(a = EarthRadius;\)\)}], "Input"],

Cell[BoxData[{
    \(\(T = Day*364\/365;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega] = \(2  \[Pi]\)\/T;\)\)}], "Input"],

Cell[BoxData[
    \(r = \((\(\(g\_0\) a\^2\)\/\[CapitalOmega]\^2)\)\^\(1/3\) // 
        ToUnit[Kilometer]\)], "Input"],

Cell[BoxData[
    \(r\/a // ToSI\)], "Input"],

Cell[BoxData[
    \(h = r - a // ToUnit[Kilometer]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4 Space elevator", "Section"],

Cell[TextData[{
  "Calculate k=",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\/\[Rho]\)]]
}], "Text"],

Cell[BoxData[
    \(k = 
      h \((\(g\_0\) 
                a\/\(a + h\) - \(\[CapitalOmega]\^2\) \((a + \(1\/2\) 
                      h)\))\) // ToSI\)], "Input"],

Cell["For beryllium", "Text"],

Cell[BoxData[{
    \(\(\[Sigma] = 1.4\ Giga\ Pascal;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho] = 8.23\ Gram\/Centimeter\^3;\)\)}], "Input"],

Cell[BoxData[
    \(\[Sigma]\/\[Rho] // ToSI\)], "Input"],

Cell["Needed factor", "Text"],

Cell[BoxData[
    \(k/%\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.6 Fall to center", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(a = EarthRadius;\)\)}], "Input"],

Cell[BoxData[
    \(v\_0 = \@\(\(g\_0\) a\) // ToUnit[Kilometer]\)], "Input"],

Cell[BoxData[
    \(t\_0 = \(\[Pi]\ a\)\/\(2  v\_0\) // ToSI\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{747, 903},
WindowMargins->{{12, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 27, 0, 95, "Title"],
Cell[1806, 55, 72, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1915, 62, 27, 0, 95, "Title"],
Cell[1945, 64, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2021, 69, 24, 0, 73, "Section"],
Cell[2048, 71, 80, 1, 30, "Input"],
Cell[2131, 74, 60, 1, 30, "Input"],
Cell[2194, 77, 71, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2302, 83, 25, 0, 73, "Section"],
Cell[2330, 85, 80, 2, 30, "Input",
  InitializationCell->False],
Cell[2413, 89, 58, 1, 30, "Input"],
Cell[2474, 92, 103, 2, 30, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[2614, 99, 26, 0, 73, "Section"],
Cell[2643, 101, 98, 2, 42, "Input",
  InitializationCell->False],
Cell[2744, 105, 118, 2, 42, "Input",
  InitializationCell->False],
Cell[2865, 109, 77, 2, 30, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[2979, 116, 28, 0, 73, "Section"],
Cell[3010, 118, 92, 2, 44, "Input",
  InitializationCell->False],
Cell[3105, 122, 92, 2, 44, "Input",
  InitializationCell->False],
Cell[3200, 126, 86, 2, 42, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[3323, 133, 43, 0, 73, "Section"],
Cell[3369, 135, 128, 2, 50, "Input"],
Cell[3500, 139, 93, 2, 44, "Input",
  InitializationCell->False],
Cell[3596, 143, 43, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3676, 149, 42, 0, 73, "Section"],
Cell[3721, 151, 62, 1, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3832, 158, 30, 0, 95, "Title"],
Cell[3865, 160, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[3941, 165, 35, 0, 73, "Section"],
Cell[3979, 167, 358, 7, 150, "Input"],
Cell[4340, 176, 85, 2, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4462, 183, 42, 0, 73, "Section"],
Cell[4507, 185, 296, 4, 70, "Input"],
Cell[4806, 191, 362, 5, 90, "Input"],
Cell[5171, 198, 268, 4, 70, "Input"],
Cell[5442, 204, 275, 4, 70, "Input"],
Cell[5720, 210, 66, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5823, 216, 31, 0, 73, "Section"],
Cell[5857, 218, 297, 6, 72, "Input"],
Cell[6157, 226, 108, 2, 30, "Input"],
Cell[6268, 230, 133, 3, 30, "Input"],
Cell[6404, 235, 141, 3, 50, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[6582, 243, 31, 0, 73, "Section"],
Cell[6616, 245, 379, 7, 93, "Input"],
Cell[6998, 254, 375, 8, 76, "Input",
  InitializationCell->False],
Cell[7376, 264, 185, 5, 50, "Input",
  InitializationCell->False],
Cell[7564, 271, 150, 3, 42, "Input"],
Cell[7717, 276, 161, 3, 50, "Input"],
Cell[7881, 281, 155, 3, 50, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[8073, 289, 33, 0, 73, "Section"],
Cell[8109, 291, 145, 3, 30, "Input"],
Cell[8257, 296, 170, 3, 42, "Input"],
Cell[8430, 301, 282, 6, 70, "Input"],
Cell[8715, 309, 235, 5, 63, "Input"],
Cell[8953, 316, 142, 3, 30, "Input"],
Cell[9098, 321, 247, 5, 44, "Input"],
Cell[9348, 328, 140, 3, 30, "Input"],
Cell[9491, 333, 169, 3, 70, "Input"],
Cell[9663, 338, 231, 3, 72, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9943, 347, 60, 0, 150, "Title"],
Cell[10006, 349, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[10082, 354, 35, 0, 73, "Section"],
Cell[10120, 356, 386, 6, 130, "Input"],
Cell[10509, 364, 57, 1, 42, "Input"],
Cell[10569, 367, 107, 2, 47, "Input"],
Cell[10679, 371, 43, 1, 45, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10771, 378, 33, 0, 95, "Title"],
Cell[10807, 380, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[10883, 385, 29, 0, 73, "Section"],
Cell[10915, 387, 41, 0, 33, "Text"],
Cell[10959, 389, 177, 3, 50, "Input",
  InitializationCell->False],
Cell[11139, 394, 82, 1, 33, "Text",
  InitializationCell->False],
Cell[11224, 397, 280, 5, 70, "Input",
  InitializationCell->False],
Cell[11507, 404, 59, 0, 33, "Text"],
Cell[11569, 406, 219, 5, 90, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[11825, 416, 37, 0, 73, "Section"],
Cell[11865, 418, 168, 3, 49, "Input",
  InitializationCell->False],
Cell[12036, 423, 195, 3, 50, "Input",
  InitializationCell->False],
Cell[12234, 428, 189, 4, 50, "Input",
  InitializationCell->False],
Cell[12426, 434, 384, 7, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12847, 446, 30, 0, 73, "Section"],
Cell[12880, 448, 35, 0, 33, "Text"],
Cell[12918, 450, 125, 3, 50, "Input",
  InitializationCell->False],
Cell[13046, 455, 280, 6, 90, "Input",
  InitializationCell->False],
Cell[13329, 463, 89, 2, 33, "Text"],
Cell[13421, 467, 255, 6, 83, "Input",
  InitializationCell->False],
Cell[13679, 475, 90, 2, 33, "Text"],
Cell[13772, 479, 425, 9, 155, "Input",
  InitializationCell->False],
Cell[14200, 490, 265, 6, 70, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[14502, 501, 31, 0, 73, "Section"],
Cell[14536, 503, 195, 5, 50, "Input",
  InitializationCell->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[14780, 514, 35, 0, 95, "Title"],
Cell[14818, 516, 51, 1, 30, "Input"],
Cell[14872, 519, 130, 2, 50, "Input"],
Cell[15005, 523, 87, 1, 42, "Input"],
Cell[15095, 526, 82, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15214, 532, 63, 1, 95, "Title",
  InitializationCell->False],
Cell[15280, 535, 51, 1, 30, "Input"],
Cell[15334, 538, 63, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[15422, 543, 56, 1, 38, "Subsection",
  InitializationCell->False],
Cell[15481, 546, 152, 4, 73, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[15670, 555, 67, 1, 38, "Subsection",
  InitializationCell->False],
Cell[15740, 558, 122, 4, 50, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[15899, 567, 55, 1, 38, "Subsection",
  InitializationCell->False],
Cell[15957, 570, 145, 3, 73, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[16139, 578, 55, 1, 38, "Subsection",
  InitializationCell->False],
Cell[16197, 581, 133, 3, 51, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[16367, 589, 54, 1, 38, "Subsection",
  InitializationCell->False],
Cell[16424, 592, 131, 3, 51, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[16592, 600, 62, 1, 38, "Subsection",
  InitializationCell->False],
Cell[16657, 603, 136, 3, 51, "Input",
  InitializationCell->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[16830, 611, 61, 1, 38, "Subsection",
  InitializationCell->False],
Cell[16894, 614, 137, 3, 51, "Input",
  InitializationCell->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17080, 623, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[17130, 627, 44, 0, 73, "Section"],
Cell[17177, 629, 46, 1, 30, "Input"],
Cell[17226, 632, 108, 2, 50, "Input"],
Cell[17337, 636, 43, 1, 30, "Input"],
Cell[17383, 639, 80, 1, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17500, 645, 42, 0, 73, "Section"],
Cell[17545, 647, 46, 1, 30, "Input"],
Cell[17594, 650, 108, 2, 50, "Input"],
Cell[17705, 654, 128, 2, 76, "Input"],
Cell[17836, 658, 119, 2, 46, "Input"],
Cell[17958, 662, 45, 1, 40, "Input"],
Cell[18006, 665, 63, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18106, 671, 37, 0, 73, "Section"],
Cell[18146, 673, 107, 4, 38, "Text"],
Cell[18256, 679, 167, 4, 42, "Input"],
Cell[18426, 685, 29, 0, 33, "Text"],
Cell[18458, 687, 141, 2, 63, "Input"],
Cell[18602, 691, 57, 1, 42, "Input"],
Cell[18662, 694, 29, 0, 33, "Text"],
Cell[18694, 696, 36, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18767, 702, 37, 0, 73, "Section"],
Cell[18807, 704, 46, 1, 30, "Input"],
Cell[18856, 707, 108, 2, 50, "Input"],
Cell[18967, 711, 77, 1, 32, "Input"],
Cell[19047, 714, 73, 1, 42, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

