(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     11672,        461]*)
(*NotebookOutlinePosition[     12370,        485]*)
(*  CellTagsIndexPosition[     12326,        481]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["7. Hydrostatic shapes", "Title",
  InitializationCell->True],

Cell["<<Lautrup`", "Input",
  PageWidth->Infinity,
  InitializationCell->True,
  ShowSpecialCharacters->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Various calculations", "Title"],

Cell[CellGroupData[{

Cell["Newton's bucket", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[CapitalOmega] = 2  \[Pi]/Second;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 10\ Centimeter;\)\)}], "Input"],

Cell["Height at rim", "Text"],

Cell[BoxData[
    \(h = \(\[CapitalOmega]\^2\/\(2  g\_0\)\) a\^2 // 
        ToUnit[Centimeter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tidal range", "Section"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = EarthRadius // N;\)\), "\[IndentingNewLine]", 
    \(\(DD = MoonOrbitRadius // N;\)\), "\[IndentingNewLine]", 
    \(\(m\  = \ MoonMass // N;\)\), "\[IndentingNewLine]", 
    \(\(M\  = \ EarthMass // N;\)\)}], "Input"],

Cell["Tidal constant", "Text"],

Cell[BoxData[
    \(H\_0 = \(3\/2\) a \(\((a\/DD)\)\^3\) m\/M // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Center of mass of Earth-Moon system", "Section"],

Cell["Distance of CM from center of Earth", "Text"],

Cell[BoxData[
    \(d = DD\ m\/\(m + M\) // ToUnit[Kilo\ Meter]\)], "Input"],

Cell["Depth of CM", "Text"],

Cell[BoxData[
    \(a - d // ToUnit[Kilo\ Meter]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rotating Earth", "Title"],

Cell[CellGroupData[{

Cell["Centrifugal parameter q", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = EarthRadius // N;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity // N;\)\), "\n", 
    \(\(\(\[CapitalOmega] = \(2  \[Pi]\)\/\(24\ Hour\)\)\(;\)\(\ \)\( (*\ 
      what\ about\ siderial\ \(\(time\)\(?\)\)\ *) \)\)\)}], "Input"],

Cell["Ratio of centrifugal acceleration to standard gravity", "Text"],

Cell[BoxData[
    \(q = \(\(\[CapitalOmega]\^2\) a\)\/g\_0 // ToSI\)], "Input"],

Cell[BoxData[
    \(1\/q\)], "Input"],

Cell["Intuitive depth of polar \"valley\"", "Text"],

Cell[BoxData[
    \(q\ a // ToUnit[Kilometer]\)], "Input"],

Cell["Depth on a rigid Earth", "Text"],

Cell[BoxData[
    \(\(1\/2\) q\ a\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Standard integral leading to eq. (7-27)", "Section"],

Cell[BoxData[
    \(Integrate[\(1\/3 - u\^2\)\/\@\(r\^2 + s\^2 - 2  r\ s\ u\), {u, \(-1\), 
        1}, Assumptions \[Rule] r > s > 0]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot of gravity variation", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = EarthRadius;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(\[Rho]\_1 = 4.5;\)\), "\n", 
    \(\(\[Rho]\_0 = 5.5;\)\), "\n", 
    \(\(q = 1\/291;\)\)}], "Input"],

Cell["Deformation scale", "Text"],

Cell[BoxData[
    \(h\_0 = \(\(1\/\(\(2\)\(\ \)\)\) q\ a\)\/\(1 - \(3\/5\) \
\[Rho]\_1\/\[Rho]\_0\)\)], "Input"],

Cell["Normal gravity", "Text"],

Cell[BoxData[
    \(\(h[\[Theta]_] = \(h\_0\) \((Sin[\[Theta]]\^2 - 2\/3)\);\)\)], "Input"],

Cell[BoxData[
    \(\(g[\[Theta]_] = 
        1 - q\ Sin[\[Theta]]\^2 - 
          2\ \(h[\[Theta]]\/a\) \((1 - \(9\/10\) \
\[Rho]\_1\/\[Rho]\_0)\);\)\)], "Input"],

Cell["Plot as a function of latitude", "Text"],

Cell[BoxData[
    \(\(Plot[g[\(\(90 - d\)\/180\) \[Pi]], {d, 0, 90}, 
          AxesLabel \[Rule] {"\<\[Delta]\>", \*"\"\<g/\!\(g\_0\)\>\""}] \
// \[IndentingNewLine]Figure["\<shapes3\>", 
          ImageSize \[Rule] 200];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot of MacLaurin function", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[
    \(maclaurin[
        e_] = \(e\^\(-3\)\) \((\(\@\(1 - e\^2\)\) \((3 - 2  e\^2)\) 
              ArcSin[e] - 3  e \((1 - e\^2)\))\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[maclaurin[e], {e, 0, 1}, PlotRange \[Rule] {0, 0.25}, 
          AxesLabel \[Rule] {"\<e\>", \*"\"\<\!\(\[CapitalOmega]\^2\)/2\!\(\
\[Pi]G\[Rho]\_0\)\>\""}] // \[IndentingNewLine]Figure["\<shapes1\>"];\)\)], \
"Input"],

Cell[BoxData[
    \(FindMaximum[maclaurin[e], {e, 0.85, 0.8, 0.99}]\)], "Input"],

Cell[BoxData[
    \(Series[maclaurin[e], {e, 0, 7}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["7.2.1 Ultracentrifuge", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(\[CapitalOmega] = 
        60000\ \(2  \[Pi]\)\/\(60  Second\) // 
          N;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 10\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram\/Meter\^3;\)\)}], "Input"],

Cell[BoxData[
    \(g = a\ \[CapitalOmega]\^2 // ToSI\)], "Input"],

Cell[BoxData[
    \(g\/g\_0\)], "Input"],

Cell[BoxData[
    \(p = \(1\/2\) \(\[Rho]\_0\) \(\[CapitalOmega]\^2\) a\^2 // 
        ToUnit[Atmosphere]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.4.1 Olympic games", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(a = EarthRadius;\)\), "\n", 
    \(g\_0 = AccelerationDueToGravity; \), "\n", 
    \(\[Rho]\_1 = 4.5; \), "\n", 
    \(\(\[Rho]\_0 = 5.5;\)\), "\n", 
    \(\(\[Delta]\_1 = 19  \[Degree];\)\), "\n", 
    \(\(\[Delta]\_2 = 55  \[Degree];\)\), "\[IndentingNewLine]", 
    \(\(q = 1\/291;\)\)}], "Input"],

Cell[BoxData[{
    \(\(\[Theta]\_1 = 90  \[Degree] - \[Delta]\_1;\)\), "\n", 
    \(\(\[Theta]\_2 = 90  \[Degree] - \[Delta]\_2;\)\)}], "Input"],

Cell[BoxData[
    \(\(h\_0 = \(\(1\/\(\(2\)\(\ \)\)\) q\ a\)\/\(1 - \(3\/5\) \[Rho]\_1\/\
\[Rho]\_0\);\)\)], "Input"],

Cell[BoxData[
    \(\(h[\[Theta]_] = \(h\_0\) \((Sin[\[Theta]]\^2 - 2\/3)\);\)\)], "Input"],

Cell[BoxData[
    \(\(g[\[Theta]_] = 
        1 - q\ Sin[\[Theta]]\^2 - 
          2\ \(h[\[Theta]]\/a\) \((1 - \(9\/10\) \
\[Rho]\_1\/\[Rho]\_0)\);\)\)], "Input"],

Cell[BoxData[
    \(g[\[Theta]\_2]\/g[\[Theta]\_1]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["7.1 Rama", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 10\ Kilometer;\)\)}], "Input"],

Cell["Escape velocity", "Text"],

Cell[BoxData[
    \(v = \@\(\(g\_0\) a\) // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.2 PressureRise", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalDelta]p = 20\ Hecto\ Pascal;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ \ Kilogram/Meter\^3;\)\)}], "Input"],

Cell["Surface change", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]h = \(-\(\[CapitalDelta]p\/\(\(\[Rho]\_0\) g\_0\)\)\) // 
        ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.3 TidalAirPressure", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(H\_0 = 54\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1.2\ \ Kilogram/Meter\^3;\)\)}], "Input"],

Cell["Pressure change", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]p = \(\[Rho]\_0\) \(g\_0\) H\_0 // 
        ToUnit[Pascal]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.4 TidalBulge", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(H\_0 = 54\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(a = EarthRadius;\)\)}], "Input"],

Cell["Angle", "Text"],

Cell[BoxData[
    \(1\/Degree\ ArcCos[1\/\@3] // N\)], "Input"],

Cell["Volume", "Text"],

Cell[BoxData[
    \(V = \(\(\(4  \[Pi]\ a\^2\)\/\(9 \@ 3\)\) H\_0 // N\) // 
        ToUnit[Kilometer\^3]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.5 Geostationary", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = EarthRadius;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(D\_moon = MoonOrbitRadius;\)\), "\[IndentingNewLine]", 
    \(\(m\_moon = MoonMass;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega] = \(2  \[Pi]\)\/\(24\ Hour\);\)\), "\
\[IndentingNewLine]", 
    \(\(R\_moon = MoonRadius;\)\)}], "Input"],

Cell["Satellite orbit ratius", "Text"],

Cell[BoxData[
    \(D\_satt = \((\(\(g\_0\) a\^2\)\/\[CapitalOmega]\^2)\)\^\(1/3\) // 
        ToUnit[Kilometer]\)], "Input"],

Cell["Ratio of moon to sattelite distances", "Text"],

Cell[BoxData[
    \(D\_moon\/D\_satt // ToSI\)], "Input"],

Cell["Satellite mass", "Text"],

Cell[BoxData[
    \(m\_satt = \(m\_moon\) \((D\_satt\/D\_moon)\)\^3 // ToSI\)], "Input"],

Cell["Ratio", "Text"],

Cell[BoxData[
    \(m\_satt\/m\_moon\)], "Input"],

Cell["Apparent radius", "Text"],

Cell[BoxData[
    \(R\_satt = R\_moon\ D\_satt\/D\_moon\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.7 CentrifugalMoon", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = EarthRadius;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega] = \(2  \[Pi]\)\/\(28\ Day\);\)\)}], "Input"],

Cell["Change in tidal range", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]H\_0 = \(\(\[CapitalOmega]\^2\) a\^2\)\/\(2  g\_0\) // 
        ToSI\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 937}},
AutoGeneratedPackage->None,
WindowSize->{815, 903},
WindowMargins->{{143, Automatic}, {Automatic, -13}},
WindowFrame->"Normal"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 66, 1, 95, "Title",
  InitializationCell->True],
Cell[1845, 56, 110, 3, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1992, 64, 37, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[2054, 68, 34, 0, 73, "Section"],
Cell[2091, 70, 46, 1, 30, "Input"],
Cell[2140, 73, 26, 0, 33, "Text"],
Cell[2169, 75, 200, 3, 70, "Input"],
Cell[2372, 80, 29, 0, 33, "Text"],
Cell[2404, 82, 108, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2549, 89, 30, 0, 73, "Section"],
Cell[2582, 91, 26, 0, 33, "Text"],
Cell[2611, 93, 245, 4, 90, "Input"],
Cell[2859, 99, 30, 0, 33, "Text"],
Cell[2892, 101, 82, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3011, 107, 54, 0, 73, "Section"],
Cell[3068, 109, 51, 0, 33, "Text"],
Cell[3122, 111, 76, 1, 40, "Input"],
Cell[3201, 114, 27, 0, 33, "Text"],
Cell[3231, 116, 61, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3341, 123, 31, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[3397, 127, 42, 0, 73, "Section"],
Cell[3442, 129, 51, 1, 30, "Input"],
Cell[3496, 132, 26, 0, 33, "Text"],
Cell[3525, 134, 259, 4, 84, "Input"],
Cell[3787, 140, 69, 0, 33, "Text"],
Cell[3859, 142, 79, 1, 47, "Input"],
Cell[3941, 145, 37, 1, 44, "Input"],
Cell[3981, 148, 51, 0, 33, "Text"],
Cell[4035, 150, 58, 1, 30, "Input"],
Cell[4096, 153, 38, 0, 33, "Text"],
Cell[4137, 155, 46, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4220, 161, 58, 0, 73, "Section"],
Cell[4281, 163, 147, 2, 56, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4465, 170, 44, 0, 73, "Section"],
Cell[4512, 172, 46, 1, 30, "Input"],
Cell[4561, 175, 26, 0, 33, "Text"],
Cell[4590, 177, 214, 5, 123, "Input"],
Cell[4807, 184, 33, 0, 33, "Text"],
Cell[4843, 186, 112, 2, 63, "Input"],
Cell[4958, 190, 30, 0, 33, "Text"],
Cell[4991, 192, 91, 1, 42, "Input"],
Cell[5085, 195, 163, 4, 44, "Input"],
Cell[5251, 201, 46, 0, 33, "Text"],
Cell[5300, 203, 237, 4, 63, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5586, 213, 43, 0, 95, "Title"],
Cell[5632, 215, 51, 1, 30, "Input"],
Cell[5686, 218, 161, 3, 37, "Input"],
Cell[5850, 223, 245, 4, 51, "Input"],
Cell[6098, 229, 80, 1, 30, "Input"],
Cell[6181, 232, 64, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6282, 238, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[6332, 242, 40, 0, 73, "Section"],
Cell[6375, 244, 46, 1, 30, "Input"],
Cell[6424, 247, 320, 6, 117, "Input"],
Cell[6747, 255, 66, 1, 30, "Input"],
Cell[6816, 258, 40, 1, 42, "Input"],
Cell[6859, 261, 118, 2, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7014, 268, 38, 0, 73, "Section"],
Cell[7055, 270, 46, 1, 30, "Input"],
Cell[7104, 273, 323, 7, 163, "Input"],
Cell[7430, 282, 144, 2, 50, "Input"],
Cell[7577, 286, 117, 2, 63, "Input"],
Cell[7697, 290, 91, 1, 42, "Input"],
Cell[7791, 293, 163, 4, 44, "Input"],
Cell[7957, 299, 63, 1, 44, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8069, 306, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[8119, 310, 27, 0, 73, "Section"],
Cell[8149, 312, 46, 1, 30, "Input"],
Cell[8198, 315, 26, 0, 33, "Text"],
Cell[8227, 317, 127, 2, 50, "Input"],
Cell[8357, 321, 31, 0, 33, "Text"],
Cell[8391, 323, 61, 1, 32, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8489, 329, 35, 0, 73, "Section"],
Cell[8527, 331, 46, 1, 30, "Input"],
Cell[8576, 334, 26, 0, 33, "Text"],
Cell[8605, 336, 222, 3, 70, "Input"],
Cell[8830, 341, 30, 0, 33, "Text"],
Cell[8863, 343, 118, 2, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9018, 350, 39, 0, 73, "Section"],
Cell[9060, 352, 46, 1, 30, "Input"],
Cell[9109, 355, 26, 0, 33, "Text"],
Cell[9138, 357, 206, 3, 70, "Input"],
Cell[9347, 362, 31, 0, 33, "Text"],
Cell[9381, 364, 106, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9524, 371, 33, 0, 73, "Section"],
Cell[9560, 373, 46, 1, 30, "Input"],
Cell[9609, 376, 26, 0, 33, "Text"],
Cell[9638, 378, 115, 2, 50, "Input"],
Cell[9756, 382, 21, 0, 33, "Text"],
Cell[9780, 384, 63, 1, 48, "Input"],
Cell[9846, 387, 22, 0, 33, "Text"],
Cell[9871, 389, 118, 2, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10026, 396, 36, 0, 73, "Section"],
Cell[10065, 398, 46, 1, 30, "Input"],
Cell[10114, 401, 26, 0, 33, "Text"],
Cell[10143, 403, 390, 7, 145, "Input"],
Cell[10536, 412, 38, 0, 33, "Text"],
Cell[10577, 414, 125, 2, 46, "Input"],
Cell[10705, 418, 52, 0, 33, "Text"],
Cell[10760, 420, 57, 1, 44, "Input"],
Cell[10820, 423, 30, 0, 33, "Text"],
Cell[10853, 425, 88, 1, 45, "Input"],
Cell[10944, 428, 21, 0, 33, "Text"],
Cell[10968, 430, 49, 1, 42, "Input"],
Cell[11020, 433, 31, 0, 33, "Text"],
Cell[11054, 435, 68, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11159, 441, 38, 0, 73, "Section"],
Cell[11200, 443, 46, 1, 30, "Input"],
Cell[11249, 446, 26, 0, 33, "Text"],
Cell[11278, 448, 207, 3, 85, "Input"],
Cell[11488, 453, 37, 0, 33, "Text"],
Cell[11528, 455, 116, 2, 47, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

