(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     28404,       1036]*)
(*NotebookOutlinePosition[     29138,       1061]*)
(*  CellTagsIndexPosition[     29094,       1057]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["16. Nearly ideal flow", "Title"],

Cell[BoxData[{
    \(<< Lautrup`\), "\n", 
    \(<< Graphics`\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Speed of sound", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Speed of sound in air", "Section"],

Cell[BoxData[{
    \(\(\[Gamma] = 7\/5;\)\), "\[IndentingNewLine]", 
    \(\(R = \ MolarGasConstant;\)\), "\[IndentingNewLine]", 
    \(\(T\_0 = 293\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(M\_mol = AirMolarMass;\)\)}], "Input"],

Cell[BoxData[
    \(c\_0 = \(\@\(\[Gamma]\ \(R\ T\_0\)\/M\_mol\) // ToSI\) // N\)], "Input"],

Cell[BoxData[
    \(c\_0 // ToUnit[Kilometer/Hour]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Speed of sound in water", "Section"],

Cell[BoxData[{
    \(\(K\_0 = 23\ Kilo\ Bar;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = \(10\^3\) Kilogram/Meter\^3;\)\)}], "Input"],

Cell[BoxData[
    \(c = \(\@\(K\_0\/\[Rho]\_0\) // ToSI\) // N\)], "Input"],

Cell[BoxData[
    \(c // ToUnit[Kilometer/Hour]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["River bed", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Analytic solution", "Section"],

Cell[TextData[{
  "The basic equations are (16-26)\n",
  Cell[BoxData[{
      \(Q = U\ d\), "\n", 
      \(H = \(1\/2\) U\^2 + \(g\_0\) \((b + d)\)\)}], "Input"],
  "\nThe values of  Q and H are determined at",
  StyleBox[" x=-\[Infinity].", "Input"],
  "\n",
  Cell[BoxData[{
      \(Q = \(U\_0\) h\_0\), "\n", 
      \(H = \(1\/2\) U\_0\^2 + \(g\_0\) h\_0\)}], "Input"],
  "\nwhere ",
  Cell[BoxData[
      \(TraditionalForm\`\(U\_0\) 
        and\ h\_0\ are\ the\ incoming\ velocity\ and\ \ surface\ \(\(height\)\
\(.\)\)\)]],
  "\n\nWithout loss of generality we may take ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            
            FormBox[\(g\_0 = \(1\ and\ \ h\_0 = \(\(1\)\(\ \)\(and\)\(\ \
\)\(put\)\(\ \)\)\)\),
              "TraditionalForm"], "\[Beta]"}], 
          "=", \(1/U\_0\^2 = 1/\(\(Fr\^2\)\(.\)\)\)}], TraditionalForm]]],
  " \nThen the above equations reduce to the third order equation for d,\n\n\
",
  Cell[BoxData[
      \(\((1 - 2  \[Beta] \((b + d - 1)\))\) d\^2 \[Equal] 1\)], "Input"],
  "\n\nSolving this equation we obtain the surface height"
}], "Text"],

Cell[BoxData[
    \(h[b_, \[Beta]_] = 
      b + d /. Solve[\((1 - 2  \[Beta] \((b + d - 1)\))\) d\^2 \[Equal] 1, 
            d] // Simplify[#, \[Beta] > 0 && 0 < d < 1] &\)], "Input"],

Cell["One real and two complex solutions.", "Text"],

Cell[CellGroupData[{

Cell["Plot solutions", "Subsection"],

Cell[BoxData[
    \(\(b0 = 0.5;\)\)], "Input"],

Cell[BoxData[
    \(\(Fr = 1.6902;\)\)], "Input"],

Cell[BoxData[
    \(\(Fr = 0.3655;\)\)], "Input"],

Cell[BoxData[
    \(\[Beta] = 1\/Fr\^2\)], "Input"],

Cell["Plot real parts", "Text"],

Cell[BoxData[
    \(\(Plot[\(h[b, \[Beta]] // Re\) // Evaluate, {b, 0, b0}, 
        PlotRange \[Rule] All, 
        PlotStyle \[Rule] {Thickness[0.01], Dashing[{0.02, 0.02}], 
            Dashing[{0.005, 0.02}]}];\)\)], "Input"],

Cell["Plot imaginary parts", "Text"],

Cell[BoxData[
    \(\(Plot[\(h[b, \[Beta]] // Im\) // Evaluate, {b, 0, b0}, 
        PlotRange \[Rule] All, 
        PlotStyle \[Rule] {Thickness[0.01], Dashing[{0.02, 0.02}], 
            Dashing[{0.005, 0.02}]}];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coincidence of roots", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["\<\
All roots are real below the branch point, which is determined by the \
coincidence of the two complex roots.
The solution at the branch point determines both d and b,\
\>", "Text"],

Cell[BoxData[{
    \(\(d = \[Beta]\^\(\(-1\)/3\);\)\), "\[IndentingNewLine]", 
    \(\(b = 1 + 1\/\(2  \[Beta]\) - \(3\/2\) d;\)\)}], "Input"],

Cell[BoxData[
    \(\(Plot[b, {\[Beta], 0.01, 8}];\)\)], "Input"],

Cell["\<\
Find the critical Froude numbers for a convenient choice of  b\
\>", "Text"],

Cell[BoxData[
    \(\(b\_0 = 0.3;\)\)], "Input"],

Cell[BoxData[
    \(sol1 = FindRoot[b \[Equal] b\_0, {\[Beta], 2}]\)], "Input"],

Cell[BoxData[
    \(Fr\_1 = 1\/\@\[Beta] /. sol1\)], "Input"],

Cell[BoxData[
    \(sol2 = FindRoot[b \[Equal] b\_0, {\[Beta], 0.1}]\)], "Input"],

Cell[BoxData[
    \(Fr\_2 = 1\/\@\[Beta] /. sol2\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numeric solution", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Alpha] = 0.5;\)\), "\[IndentingNewLine]", 
    \(\(b\_0 = 0.3;\)\), "\[IndentingNewLine]", 
    \(\(a1 = 5;\)\), "\[IndentingNewLine]", 
    \(\(a2 = 10;\)\), "\[IndentingNewLine]", 
    \(\(Fr\_1 = 0.36549;\)\), "\[IndentingNewLine]", 
    \(\(Fr\_2 = 1.69020;\)\)}], "Input"],

Cell[BoxData[
    \(\[Beta] := 1\/Fr\^2\)], "Input"],

Cell["Bump shape", "Text"],

Cell[BoxData[
    \(\(b[x_] = \(1\/2\) b\_0\ \ \((1 + Tanh[\[Alpha]\ x])\);\)\)], "Input"],

Cell[BoxData[
    \(\(fig3a = 
        Plot[b[x], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Thickness[0.01], 
              Text["\<bed\>", {10.6, 0.3}, {\(-1\), 0}]}];\)\)], "Input"],

Cell["Solve for height", "Text"],

Cell[BoxData[
    \(h[b_, \[Beta]_] := 
      b + d /. FindRoot[\((1 - 2  \[Beta] \((b + d - 1)\))\) d\^2 \[Equal] 
            1, {d, 1}]\)], "Input"],

Cell[BoxData[{
    \(\(Fr = Fr\_2;\)\), "\[IndentingNewLine]", 
    \(\(fig3b = 
        Plot[h[b[x], \[Beta]], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Text["\<1.69\>", {11, 1.71}]}];\)\)}], "Input"],

Cell[BoxData[
    \(\[Delta]\_2 = h[b\_0, \[Beta]] - 1\)], "Input"],

Cell[BoxData[{
    \(\(Fr = 1.8;\)\), "\n", 
    \(\(fig3e = 
        Plot[h[b[x], \[Beta]], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Text["\<1.80\>", {11, 1.5}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = 10\^6;\)\), "\n", 
    \(\(fig3f = 
        Plot[h[b[x], \[Beta]], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Text["\<\[Infinity]\>", {11, 
                  1.3}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = Fr\_1;\)\), "\[IndentingNewLine]", 
    \(\(fig3c = 
        Plot[h[b[x], \[Beta]], {x, \(-a1\), a2}, 
          PlotStyle -> {Text["\<0.37\>", {11, 0.813}]}];\)\)}], "Input"],

Cell[BoxData[
    \(\[Delta]\_1 = h[b\_0, \[Beta]] - 1\)], "Input"],

Cell[BoxData[{
    \(\(Fr = 0.30;\)\), "\n", 
    \(\(fig3d = 
        Plot[h[b[x], \[Beta]], {x, \(-a1\), a2}, 
          PlotStyle -> {Text["\<0.30\>", {11, 0.933}]}];\)\)}], "Input"],

Cell[BoxData[
    \(\(Show[fig3a, fig3b, fig3c, fig3d, fig3e, fig3f, 
          Graphics[{Text[\*"\"\<\!\(Fr\_in\)\>\"", {11, 0.70}], 
              Dashing[{0.01}], Line[{{\(-a1\), 1}, {a2, 1}}]}], 
          PlotRange \[Rule] All, 
          AxesLabel \[Rule] {"\<x\>", "\<z\>"}] \
// \[IndentingNewLine]Figure["\<ideal3\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Duct flow", "Title"],

Cell[CellGroupData[{

Cell["Plot of various quantities", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\(\[Gamma] = 7/5;\)\)], "Input"],

Cell[BoxData[
    RowBox[{"Tratio", "=", 
      RowBox[{"1", "+", 
        RowBox[{\(\(\[Gamma] - 1\)\/\(\[Gamma] + 1\)\), \((Ma\^2 - 1)\), Cell[
          ""], Cell[""]}]}]}]], "Input"],

Cell[BoxData[
    \(\[Rho]ratio = Tratio\^\(\(\ \)\(1/\((\[Gamma] - 1)\)\)\)\)], "Input"],

Cell[BoxData[
    \(pratio = Tratio\^\(\[Gamma]/\((\[Gamma] - 1)\)\)\)], "Input"],

Cell[BoxData[
    \(Uratio = \(1\/Ma\) \@Tratio\)], "Input"],

Cell[BoxData[
    \(Aratio = 
      Ma\ Tratio\^\(\(\(-\((\[Gamma] + 1)\)\)/2\)/\((\[Gamma] - 1)\)\)\)], \
"Input"],

Cell[BoxData[
    \(\(Plot[{1/Aratio, 1/Tratio, 1/\[Rho]ratio, 1/pratio, 1/Uratio}, {Ma, 0, 
            3}, AxesLabel \[Rule] {"\<Ma\>", "\<\>"}, 
          PlotRange \[Rule] {0, 3}, 
          PlotStyle \[Rule] {{Dashing[{}], 
                Text[\*"\"\<A/\!\(A\_1\)\>\"", {2.4, 
                    2.7}]}, {Dashing[{0.04}], 
                Text[\*"\"\<T/\!\(T\_1\)\>\"", {2, 0.8}]}, {Dashing[{0.02}], 
                Text[\*"\"\<\[Rho]/\!\(\[Rho]\_1\)\>\"", {2, 
                    0.5}]}, {Dashing[{0.01}], 
                Text[\*"\"\<p/\!\(p\_1\)\>\"", {1.5, 
                    0.4}]}, {Dashing[{0.005}], 
                Text[\*"\"\<U/\!\(U\_1\)\>\"", {2.4, 
                    1.9}]}}] // \[IndentingNewLine]Figure["\<ideal4\>", 
          SaveFigure \[Rule] True];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Duct model", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\(\[Gamma] = 7/5;\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(f[m_]\), "=", 
        RowBox[{\(1\/m\), " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "+", 
                
                RowBox[{\(\(\[Gamma] - 1\)\/\(\[Gamma] + 1\)\), \((m\^2 - 
                      1)\), Cell[""], Cell[""]}]}], 
              ")"}], \(\(\((\[Gamma] + 1)\)/2\)/\((\[Gamma] - 1)\)\)]}]}], 
      ";"}]], "Input"],

Cell[BoxData[{
    \(left[a_] := 
      m /. FindRoot[f[m] \[Equal] a, {m, 0.5}]\), "\[IndentingNewLine]", 
    \(right[a_] := m /. FindRoot[f[m] \[Equal] a, {m, 1.5}]\)}], "Input"],

Cell[BoxData[
    \(\(k = 0.1;\)\)], "Input"],

Cell[BoxData[
    \(\(A[x_?NumericQ] = \ 1 + k\ x\^2;\)\)], "Input"],

Cell[BoxData[{
    \(sub[x_?NumericQ, a_] := left[a\ A[x]]\), "\n", 
    \(super[x_?NumericQ, a_] := right[\ a\ A[x]]\ \), "\n", 
    \(crit[x_?NumericQ] := If[x < 0, left[A[x]], right[A[x]]]\)}], "Input"],

Cell[BoxData[
    \(\(curves = {crit[x], sub[x, 1.01], sub[x, 1.1], 
          sub[x, 1.6]};\)\)], "Input"],

Cell[BoxData[
    \(\(x0 = 1\/\@k;\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[curves // Evaluate, {x, \(-x0\), x0}, 
          PlotRange \[Rule] {{\(-x0\) - 0.2, 3}, {0, 2}}, 
          AxesOrigin \[Rule] {\(-x0\) - 0.2, 0}, PlotLabel -> "\<(a)\>", 
          AxesLabel \[Rule] {"\<x\>", "\<Ma\>"}, 
          PlotStyle \[Rule] {Text["\<1.6\>", {0, 0.3}], 
              Text["\<1.1\>", {0, 0.6}], Text["\<1.01\>", {0, 0.8}], 
              Text["\<1\>", {0, 
                  1.1}]}] // \[IndentingNewLine]Figure["\<ideal5\>", 
          SaveFigure \[Rule] True];\)\)], "Input"],

Cell[BoxData[
    \(t[m_] := \((1 + \(\(\[Gamma] - 1\)\/\(\[Gamma] + 1\)\) \((m\^2 - \
1)\))\)\^\(-1\)\)], "Input"],

Cell[BoxData[
    \(p[m_] := t[m]\^\(\[Gamma]/\((\[Gamma] - 1)\)\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[Map[p, curves] // Evaluate, {x, \(-x0\), x0}, 
          PlotRange \[Rule] {{\(-x0\) - 0.2, 3}, {0, 2}}, 
          AxesOrigin \[Rule] {\(-x0\) - 0.2, 0}, PlotLabel -> "\<(b)\>", 
          AxesLabel \[Rule] {"\<x\>", \*"\"\<p/\!\(p\_1\)\>\""}, 
          PlotStyle \[Rule] {Text["\<1.6\>", {0, 1.75}], 
              Text["\<1.1\>", {0, 1.45}], Text["\<1.01\>", {0, 1.2}], 
              Text["\<1\>", {0, 
                  0.85}]}] // \[IndentingNewLine]Figure["\<ideal6\>", 
          SaveFigure \[Rule] True];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gaussian weir", "Title"],

Cell["\<\
A weir in a shallow river changes the shape of the opens urface\
\>", "Text"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Solve algebraic equation", "Section"],

Cell[BoxData[{
    \(\(\[Alpha] = 0.2;\)\), "\[IndentingNewLine]", 
    \(\(k = 0.3;\)\), "\[IndentingNewLine]", 
    \(\(a1 = 5;\)\), "\[IndentingNewLine]", 
    \(\(a2 = 5;\)\), "\[IndentingNewLine]", 
    \(\(\[Beta] := 1\/Fr\^2;\)\), "\[IndentingNewLine]", 
    \(\(b[x_] = k\ \ \[ExponentialE]\^\(\(-\[Alpha]\)\ x\^2\);\)\)}], "Input"],

Cell[BoxData[
    \(\(fig3a = 
        Plot[b[x], {x, \(-a1\), a2}, PlotRange \[Rule] All, 
          PlotStyle \[Rule] {Thickness[0.01], 
              Text["\<bed\>", {10.6, 0.3}, {\(-1\), 0}]}];\)\)], "Input"],

Cell[BoxData[
    \(h[y_] := 
      1 + z /. FindRoot[\((1 - 2  \[Beta]\ z)\) \((1 + z - y)\)\^2 \[Equal] 
            1, {z, 0, 0.01}]\)], "Input"],

Cell[BoxData[{
    \(\(Fr = 1.690195304152707`;\)\), "\[IndentingNewLine]", 
    \(\(fig3b = 
        Plot[h[b[x]], {x, \(-a1\), a2}, PlotRange \[Rule] All, 
          PlotStyle \[Rule] {Text["\<1.69\>", {11, 1.71}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = 1.80;\)\), "\n", 
    \(\(fig3e = 
        Plot[h[b[x]], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Text["\<1.8\>", {11, 1.5}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = 10\^6;\)\), "\n", 
    \(\(fig3f = 
        Plot[h[b[x]], {x, \(-a1\), a2}, 
          PlotStyle \[Rule] {Text["\<\[Infinity]\>", {11, 
                  1.3}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = 0.36549401266568804`;\)\), "\n", 
    \(\[Beta]\), "\[IndentingNewLine]", 
    \(\(fig3c = 
        Plot[h[b[x]], {x, \(-a1\), a2}, 
          PlotStyle -> {Text["\<0.37\>", {11, 1 - 0.188}]}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Fr = 0.33;\)\), "\n", 
    \(\(fig3d = 
        Plot[h[b[x]], {x, \(-a1\), a2}, 
          PlotStyle -> {Text["\<0.33\>", {11, 1 - 0.094}]}];\)\)}], "Input"],

Cell[BoxData[
    \(\(Show[fig3a, fig3b, fig3c, fig3d, fig3e, fig3f, 
        Graphics[{Text[\*"\"\<\!\(Fr\_0\)\>\"", {11, 0.70}], Dashing[{0.01}], 
            Line[{{\(-a1\), 1}, {a2, 1}}]}], AxesOrigin \[Rule] {\(-a1\), 0}, 
        PlotRange \[Rule] {{\(-a1\), a2}, All}, 
        AxesLabel \[Rule] {"\<x\>", "\<z\>"}];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Potential flow around cylinder", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Plot of streamlines", "Section"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 1;\)\), "\[IndentingNewLine]", 
    \(\(U = 1;\)\)}], "Input"],

Cell["Field in Cartesian coordinates", "Text"],

Cell[BoxData[{
    \(\(v\_x = 
        U \((1 - \(\(a\^2\) \((x\^2 - y\^2)\)\)\/\((x\^2 + \
y\^2)\)\^2)\);\)\), "\[IndentingNewLine]", 
    \(\(v\_y = \(-\ 
            U\)\ \(2 \( a\^2\) x\ y\)\/\((x\^2 + y\^2)\)\^2;\)\)}], "Input"],

Cell["Introduce running curve parameter t", "Text"],

Cell[BoxData[
    \(\(sub = {x \[Rule] x[t], y \[Rule] y[t]};\)\)], "Input"],

Cell[BoxData[{
    \(\(vx = v\_x /. sub;\)\), "\[IndentingNewLine]", 
    \(\(vy = v\_y /. sub;\)\)}], "Input"],

Cell["Solve differential equation for streamlines", "Text"],

Cell[BoxData[
    \(sol[x0_, y0_] := 
      NDSolve[{\(x'\)[t] \[Equal] vx, \(y'\)[t] \[Equal] vy, 
            x[0] \[Equal] x0, y[0] \[Equal] y0}, {x, y}, {t, 0, 40}] // 
        First\)], "Input"],

Cell["Choice of streamlines", "Text"],

Cell[BoxData[
    \(\(table = 
        Table[{x[t], y[t]} /. sol[\(-20\), y0], {y0, 0, 3, 
            0.2}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig1a = 
        ParametricPlot[table // Evaluate, {t, 0, 40}, 
          PlotRange \[Rule] {{\(-3\), 3}, {0, 3}}, AspectRatio \[Rule] 0.5, 
          Frame \[Rule] True, 
          FrameLabel \[Rule] {"\<x\>", "\<y\>"}];\)\)], "Input"],

Cell["Pressure", "Text"],

Cell[BoxData[
    \(p = \(1\/2\) \(\[Rho]\_0\) \((U\^2 - v\_x\^2 - v\_y\^2)\) // 
        Simplify\)], "Input"],

Cell["Plot zero pressure lines", "Text"],

Cell[BoxData[
    \(\(fig1b = 
        Plot[\@\(x\^2 - a\^2\/2\), {x, a\/\@2, 3}, 
          PlotStyle \[Rule] {Dashing[{0.02}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig1c = 
        Plot[\@\(x\^2 - a\^2\/2\), {x, \(-\(a\/\@2\)\), \(-3\)}, 
          PlotStyle \[Rule] {Dashing[{0.02}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig1d = 
        Graphics[{GrayLevel[0.5], Disk[{0, 0}, 1, {0, \[Pi]}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig1a, fig1b, fig1c, fig1d, 
          AspectRatio \[Rule] 
            1/2] // \[IndentingNewLine]Figure["\<ideal1\>"];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Lift", "Section"],

Cell[BoxData[
    \(lift = \(-\(1\/2\)\) \(\[Rho]\_0\) \(U\^2\) 
        a\ L \(\[Integral]\_0\%\[Pi]\((4\ Cos[\[Phi]]\^2 - 3)\) 
            Sin[\[Phi]] \[DifferentialD]\[Phi]\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Potential flow around sphere", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Plot of streamlines", "Section"],

Cell["Potential in Cartesian coordinates in the (s,z) plane", "Text"],

Cell[BoxData[{
    \(\(r = \@\(s\^2 + z\^2\);\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalPsi] = U\ z\ \((1 + a\^3\/\(2  r\^3\))\);\)\)}], "Input"],

Cell["Velocity field", "Text"],

Cell[BoxData[
    \(v\_z = \[PartialD]\_z \[CapitalPsi] // Simplify\)], "Input"],

Cell[BoxData[
    \(v\_s = \[PartialD]\_s \[CapitalPsi] // Simplify\)], "Input"],

Cell["Choice of parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_0 = 1;\)\), "\[IndentingNewLine]", 
    \(\(a = 1;\)\), "\[IndentingNewLine]", 
    \(\(U = 1;\)\)}], "Input"],

Cell["Introduce curve parameter", "Text"],

Cell[BoxData[
    \(\(sub = {z \[Rule] z[t], s \[Rule] s[t]};\)\)], "Input"],

Cell[BoxData[{
    \(\(vz = v\_z /. sub;\)\), "\[IndentingNewLine]", 
    \(\(vs = v\_s /. sub;\)\)}], "Input"],

Cell["Differential equation for streamlines", "Text"],

Cell[BoxData[
    \(sol[z0_, s0_] := 
      NDSolve[{\(z'\)[t] \[Equal] vz, \(s'\)[t] \[Equal] vs, 
            z[0] \[Equal] z0, s[0] \[Equal] s0}, {z, s}, {t, 0, 40}] // 
        First\)], "Input"],

Cell["Choice of streamlines", "Text"],

Cell[BoxData[
    \(\(table = 
        Table[{z[t], s[t]} /. sol[\(-20\), s0], {s0, 0, 3, 
            0.2}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig2a = 
        ParametricPlot[table // Evaluate, {t, 0, 40}, 
          PlotRange \[Rule] {{\(-3\), 3}, {0, 3}}, AspectRatio \[Rule] 0.5, 
          Frame \[Rule] True, 
          FrameLabel \[Rule] {"\<z\>", \*"\"\<\!\(\@\(x\^2 + y\^2\)\)\>\""}];\
\)\)], "Input"],

Cell[BoxData[
    \(\(fig2b = 
        Graphics[{GrayLevel[0.5], Disk[{0, 0}, 1, {0, \[Pi]}]}];\)\)], "Input"],

Cell["Pressure", "Text"],

Cell[BoxData[
    \(p[s_, z_] = \(1\/2\) \(\[Rho]\_0\) \((U\^2 - v\_z\^2 - v\_s\^2)\) // 
        Simplify\)], "Input"],

Cell["Zero pressure line", "Text"],

Cell[BoxData[
    \(zp[z_] := s /. FindRoot[p[s, z] \[Equal] 0, {s, 1, 1.01}]\)], "Input"],

Cell[BoxData[
    \(zp[2]\)], "Input"],

Cell["Starting point", "Text"],

Cell[BoxData[
    \(z\_0 = 0.001 + \@\(5\/9\) // N\)], "Input"],

Cell[BoxData[
    \(\(fig2c = 
        Plot[zp[z], {z, z\_0, 3}, 
          PlotStyle \[Rule] {Dashing[{0.02}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(fig2d = 
        Plot[zp[z], {z, \(-z\_0\), \(-3\)}, 
          PlotStyle \[Rule] {Dashing[{0.02}]}];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig2a, fig2b, fig2c, fig2d, 
          AspectRatio \[Rule] 
            1/2] // \[IndentingNewLine]Figure["\<ideal2\>"];\)\)], "Input"],

Cell["Zero pressure angle", "Text"],

Cell[BoxData[
    \(ArcCos[\@\(5\/9\)]\/Degree // N\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Lift", "Section"],

Cell[BoxData[
    \(lift = \(-\(1\/2\)\) \(\[Rho]\_0\) \(U\^2\) \(a\^2\) \(\[Integral]\_0\%\
\[Pi]\((\[Integral]\_0\%\[Pi]\ 
                Sin[\[Phi]] \[DifferentialD]\[Phi])\) \(Sin[\[Theta]]\^2\) \(\
\(9  Cos[\[Theta]]\^2 - 5\)\/4\) \[DifferentialD]\[Theta]\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["16.2.1 Atmospheric sound speed. Dependence on height", "Section"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Gamma] = 7\/5;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(c\_0 = 343.154\ Meter/Second;\)\)}], "Input"],

Cell["Insentropic  height of the atmosphere", "Text"],

Cell[BoxData[
    \(h\_2 = 
      c\_0\^2\/\(\((\[Gamma] - 1)\) g\_0\) // ToUnit[Kilometer]\)], "Input"],

Cell["Sound velocity at 10 km", "Text"],

Cell[BoxData[
    \(c = \(c\_0\) \@\(1 - z\/h\_2\) /. z \[Rule] 10\ Kilometer // 
        ToSI\)], "Input"],

Cell[BoxData[
    \(c // ToUnit[Kilometer/Hour]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.3.2 Wine barrel", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(R = 0.5  Meter;\)\), "\[IndentingNewLine]", 
    \(\(h = 2\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(a = 2.5\ Centimeter;\)\)}], "Input"],

Cell["Exit velocity", "Text"],

Cell[BoxData[
    \(U = \@\(2 \( g\_0\) h\) // ToSI\)], "Input"],

Cell["Volumetric discharge rate", "Text"],

Cell[BoxData[
    \(Q = \[Pi]\ \(a\^2\) U // ToUnit[Liter]\)], "Input"],

Cell["Volume", "Text"],

Cell[BoxData[
    \(V = \(\[Pi]\ \(R\^2\) h // ToSI\) // N\)], "Input"],

Cell["Discharge time", "Text"],

Cell[BoxData[
    \(t = V\/Q // ToUnit[Minute]\)], "Input"],

Cell["Free fall time", "Text"],

Cell[BoxData[
    \(\[Tau] = \@\(\(2  h\)\/g\_0\) // ToSI\)], "Input"],

Cell["Time to empty the barrel", "Text"],

Cell[BoxData[
    \(T = \(\((R\/a)\)\^2\) \[Tau] // ToUnit[Minute]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.3.3 Water scoop", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(U = 120\ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(A = 300\ Centimeter\^2;\)\)}], "Input"],

Cell["Maximal Pitot height", "Text"],

Cell[BoxData[
    \(h = U\^2\/\(2  g\_0\) // ToSI\)], "Input"],

Cell["Discharge rate", "Text"],

Cell[BoxData[
    \(Q = 1. \ U\ A // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.3.4 Shallow river", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(d\_in = 1\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(Fr\_1 = 0.36549;\)\), "\[IndentingNewLine]", 
    \(\(Fr\_2 = 1.69020;\)\), "\[IndentingNewLine]", 
    \(\(\[Delta]\_1 = \(-0.18762\);\)\), "\[IndentingNewLine]", 
    \(\(\[Delta]\_2 = 0.71698;\)\)}], "Input"],

Cell["Critical velocities", "Text"],

Cell[BoxData[
    \(U\_1 = \(Fr\_1\) \@\(\(g\_0\) d\_in\) // ToSI\)], "Input"],

Cell[BoxData[
    \(U\_2 = \(Fr\_2\) \@\(\(g\_0\) d\_in\) // ToSI\)], "Input"],

Cell["Maximal change in height", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]h\_1 = \(\[Delta]\_1\) d\_in // 
        ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]h\_2 = \(\[Delta]\_2\) d\_in // 
        ToUnit[Centimeter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.4.1 Mach numbers", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(v = {3\ Meter/Second, 120\ Kilometer/Hour, 
          900\ Kilometer/Hour};\)\), "\[IndentingNewLine]", 
    \(\(c = {343. , 343. , 280. } Meter/Second;\)\)}], "Input"],

Cell[BoxData[
    \(Ma = v\/c // ToSI\)], "Input"],

Cell[BoxData[
    \(Thread[{v, c, Ma}] // MatrixForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.4.2 Stagnation point rises", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(M\_mol = AirMolarMass;\)\), "\[IndentingNewLine]", 
    \(\(R = MolarGasConstant;\)\), "\[IndentingNewLine]", 
    \(\(\[Gamma] = 7\/5;\)\)}], "Input"],

Cell["Stagnation point rise", "Text"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = \(M\_mol\/R\) \(\(\[Gamma] - 
                1\)\/\(2  \[Gamma]\)\) U\^2;\)\)], "Input"],

Cell["Velocities", "Text"],

Cell[BoxData[
    \(\(Us = {100\ Kilometer/Hour, 900  Kilometer/Hour, 2300  Kilometer/Hour, 
          3\ Kilometer/Second, 8  Kilometer/Second};\)\)], "Input"],

Cell["Stagnation point rises", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]Ts = 
      Map[Function[U, \[CapitalDelta]T // Evaluate], Us] // ToSI\)], "Input"],

Cell[BoxData[
    \(Thread[{Us, \[CapitalDelta]Ts}] // MatrixForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.9.1 Half-buried sphere", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 10\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_1 = 1.1\ \[Rho]\_0;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell[BoxData[
    \(U\_c = \@\(\(128\/33\) \((\[Rho]\_1\/\[Rho]\_0 - 1)\) a\ g\_0\) // 
        ToSI\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["16.2 Siphon", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 0.5\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(h = 20\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(V = 10\ Liter;\)\)}], "Input"],

Cell["Free fall velocity", "Text"],

Cell[BoxData[
    \(v = \@\(2 \( g\_0\) h\) // ToSI\)], "Input"],

Cell[BoxData[
    \(Q = \[Pi]\ \(a\^2\) v // ToUnit[Liter]\)], "Input"],

Cell[BoxData[
    \(t = V\/Q // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.7 Stagnation pressure at constant temperature", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(T\_0 = 223\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(U = 250\ Meter/Second;\)\), "\[IndentingNewLine]", 
    \(\(M\_mol = AirMolarMass;\)\), "\[IndentingNewLine]", 
    \(\(R = MolarGasConstant;\)\)}], "Input"],

Cell["Pressure reduction", "Text"],

Cell[BoxData[
    \(Exp[\(M\_mol\ U\^2\)\/\(2\ R\ T\_0\)] // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.12 Buried worm", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(a = 3\ Millimeter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_1 = 1.1  \[Rho]\_0;\)\)}], "Input"],

Cell["Critical velocity", "Text"],

Cell[BoxData[
    \(U = \@\(\(\(3  \[Pi]\)\/5\) \((\[Rho]\_1\/\[Rho]\_0 - 1)\) a\ g\_0\) // 
        ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.16 Ignoring gravity", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(c\_0 = 343\ Meter/Second;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Critical wavelength", "Text"],

Cell[BoxData[
    \(\[Lambda]\_c = c\_0\^2\/g\_0 // ToSI\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{729, 880},
WindowMargins->{{51, Automatic}, {Automatic, 9}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 38, 0, 95, "Title"],
Cell[1817, 55, 103, 3, 50, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1957, 63, 31, 0, 95, "Title"],
Cell[1991, 65, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2062, 70, 40, 0, 73, "Section"],
Cell[2105, 72, 303, 5, 123, "Input"],
Cell[2411, 79, 92, 1, 51, "Input"],
Cell[2506, 82, 63, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2606, 88, 42, 0, 73, "Section"],
Cell[2651, 90, 138, 2, 50, "Input"],
Cell[2792, 94, 75, 1, 51, "Input"],
Cell[2870, 97, 60, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2979, 104, 26, 0, 95, "Title"],
Cell[3008, 106, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[3079, 111, 36, 0, 73, "Section"],
Cell[3118, 113, 1121, 31, 282, "Text"],
Cell[4242, 146, 185, 3, 31, "Input"],
Cell[4430, 151, 51, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4506, 155, 36, 0, 38, "Subsection"],
Cell[4545, 157, 46, 1, 30, "Input"],
Cell[4594, 160, 49, 1, 30, "Input"],
Cell[4646, 163, 49, 1, 30, "Input"],
Cell[4698, 166, 51, 1, 42, "Input"],
Cell[4752, 169, 31, 0, 33, "Text"],
Cell[4786, 171, 229, 4, 50, "Input"],
Cell[5018, 177, 36, 0, 33, "Text"],
Cell[5057, 179, 229, 4, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5335, 189, 39, 0, 73, "Section"],
Cell[5377, 191, 46, 1, 30, "Input"],
Cell[5426, 194, 191, 4, 52, "Text"],
Cell[5620, 200, 142, 2, 65, "Input"],
Cell[5765, 204, 65, 1, 30, "Input"],
Cell[5833, 207, 86, 2, 33, "Text"],
Cell[5922, 211, 48, 1, 30, "Input"],
Cell[5973, 214, 79, 1, 30, "Input"],
Cell[6055, 217, 61, 1, 48, "Input"],
Cell[6119, 220, 81, 1, 30, "Input"],
Cell[6203, 223, 61, 1, 48, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6301, 229, 35, 0, 73, "Section"],
Cell[6339, 231, 46, 1, 30, "Input"],
Cell[6388, 234, 26, 0, 33, "Text"],
Cell[6417, 236, 302, 6, 130, "Input"],
Cell[6722, 244, 52, 1, 42, "Input"],
Cell[6777, 247, 26, 0, 33, "Text"],
Cell[6806, 249, 90, 1, 42, "Input"],
Cell[6899, 252, 189, 4, 50, "Input"],
Cell[7091, 258, 32, 0, 33, "Text"],
Cell[7126, 260, 151, 3, 31, "Input"],
Cell[7280, 265, 207, 4, 50, "Input"],
Cell[7490, 271, 67, 1, 30, "Input"],
Cell[7560, 274, 187, 4, 50, "Input"],
Cell[7750, 280, 215, 5, 50, "Input"],
Cell[7968, 287, 203, 4, 50, "Input"],
Cell[8174, 293, 67, 1, 30, "Input"],
Cell[8244, 296, 185, 4, 50, "Input"],
Cell[8432, 302, 342, 6, 90, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8823, 314, 26, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[8874, 318, 45, 0, 73, "Section"],
Cell[8922, 320, 46, 1, 30, "Input"],
Cell[8971, 323, 52, 1, 30, "Input"],
Cell[9026, 326, 186, 4, 44, "Input"],
Cell[9215, 332, 89, 1, 30, "Input"],
Cell[9307, 335, 81, 1, 30, "Input"],
Cell[9391, 338, 60, 1, 42, "Input"],
Cell[9454, 341, 115, 3, 30, "Input"],
Cell[9572, 346, 796, 14, 170, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10405, 365, 29, 0, 73, "Section"],
Cell[10437, 367, 46, 1, 30, "Input"],
Cell[10486, 370, 52, 1, 30, "Input"],
Cell[10541, 373, 418, 11, 45, "Input"],
Cell[10962, 386, 181, 3, 50, "Input"],
Cell[11146, 391, 45, 1, 30, "Input"],
Cell[11194, 394, 68, 1, 30, "Input"],
Cell[11265, 397, 205, 3, 70, "Input"],
Cell[11473, 402, 107, 2, 30, "Input"],
Cell[11583, 406, 49, 1, 48, "Input"],
Cell[11635, 409, 529, 9, 110, "Input"],
Cell[12167, 420, 115, 2, 45, "Input"],
Cell[12285, 424, 79, 1, 30, "Input"],
Cell[12367, 427, 556, 9, 110, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12972, 442, 30, 0, 95, "Title"],
Cell[13005, 444, 87, 2, 33, "Text"],
Cell[13095, 448, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[13166, 453, 43, 0, 73, "Section"],
Cell[13212, 455, 340, 6, 147, "Input"],
Cell[13555, 463, 212, 4, 50, "Input"],
Cell[13770, 469, 148, 3, 31, "Input"],
Cell[13921, 474, 234, 4, 70, "Input"],
Cell[14158, 480, 178, 4, 50, "Input"],
Cell[14339, 486, 206, 5, 50, "Input"],
Cell[14548, 493, 237, 5, 70, "Input"],
Cell[14788, 500, 180, 4, 50, "Input"],
Cell[14971, 506, 338, 5, 70, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15358, 517, 47, 0, 95, "Title"],
Cell[15408, 519, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[15484, 524, 38, 0, 73, "Section"],
Cell[15525, 526, 26, 0, 33, "Text"],
Cell[15554, 528, 89, 2, 50, "Input"],
Cell[15646, 532, 46, 0, 33, "Text"],
Cell[15695, 534, 233, 5, 89, "Input"],
Cell[15931, 541, 51, 0, 33, "Text"],
Cell[15985, 543, 76, 1, 30, "Input"],
Cell[16064, 546, 111, 2, 50, "Input"],
Cell[16178, 550, 59, 0, 33, "Text"],
Cell[16240, 552, 199, 4, 50, "Input"],
Cell[16442, 558, 37, 0, 33, "Text"],
Cell[16482, 560, 124, 3, 30, "Input"],
Cell[16609, 565, 258, 5, 50, "Input"],
Cell[16870, 572, 24, 0, 33, "Text"],
Cell[16897, 574, 111, 2, 42, "Input"],
Cell[17011, 578, 40, 0, 33, "Text"],
Cell[17054, 580, 145, 3, 55, "Input"],
Cell[17202, 585, 159, 3, 55, "Input"],
Cell[17364, 590, 110, 2, 30, "Input"],
Cell[17477, 594, 162, 3, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17676, 602, 23, 0, 73, "Section"],
Cell[17702, 604, 191, 3, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17942, 613, 45, 0, 95, "Title"],
Cell[17990, 615, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[18066, 620, 38, 0, 73, "Section"],
Cell[18107, 622, 69, 0, 33, "Text"],
Cell[18179, 624, 149, 2, 70, "Input"],
Cell[18331, 628, 30, 0, 33, "Text"],
Cell[18364, 630, 80, 1, 30, "Input"],
Cell[18447, 633, 80, 1, 30, "Input"],
Cell[18530, 636, 36, 0, 33, "Text"],
Cell[18569, 638, 141, 3, 70, "Input"],
Cell[18713, 643, 41, 0, 33, "Text"],
Cell[18757, 645, 76, 1, 30, "Input"],
Cell[18836, 648, 111, 2, 50, "Input"],
Cell[18950, 652, 53, 0, 33, "Text"],
Cell[19006, 654, 199, 4, 50, "Input"],
Cell[19208, 660, 37, 0, 33, "Text"],
Cell[19248, 662, 124, 3, 30, "Input"],
Cell[19375, 667, 288, 6, 55, "Input"],
Cell[19666, 675, 110, 2, 30, "Input"],
Cell[19779, 679, 24, 0, 33, "Text"],
Cell[19806, 681, 119, 2, 42, "Input"],
Cell[19928, 685, 34, 0, 33, "Text"],
Cell[19965, 687, 90, 1, 30, "Input"],
Cell[20058, 690, 38, 1, 30, "Input"],
Cell[20099, 693, 30, 0, 33, "Text"],
Cell[20132, 695, 63, 1, 51, "Input"],
Cell[20198, 698, 128, 3, 30, "Input"],
Cell[20329, 703, 138, 3, 30, "Input"],
Cell[20470, 708, 162, 3, 50, "Input"],
Cell[20635, 713, 35, 0, 33, "Text"],
Cell[20673, 715, 64, 1, 69, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20774, 721, 23, 0, 73, "Section"],
Cell[20800, 723, 276, 4, 45, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21125, 733, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[21175, 737, 71, 0, 73, "Section"],
Cell[21249, 739, 26, 0, 33, "Text"],
Cell[21278, 741, 158, 3, 83, "Input"],
Cell[21439, 746, 53, 0, 33, "Text"],
Cell[21495, 748, 104, 2, 47, "Input"],
Cell[21602, 752, 39, 0, 33, "Text"],
Cell[21644, 754, 107, 2, 51, "Input"],
Cell[21754, 758, 60, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21851, 764, 37, 0, 73, "Section"],
Cell[21891, 766, 46, 1, 30, "Input"],
Cell[21940, 769, 26, 0, 33, "Text"],
Cell[21969, 771, 233, 4, 90, "Input"],
Cell[22205, 777, 29, 0, 33, "Text"],
Cell[22237, 779, 64, 1, 32, "Input"],
Cell[22304, 782, 41, 0, 33, "Text"],
Cell[22348, 784, 71, 1, 30, "Input"],
Cell[22422, 787, 22, 0, 33, "Text"],
Cell[22447, 789, 71, 1, 30, "Input"],
Cell[22521, 792, 30, 0, 33, "Text"],
Cell[22554, 794, 59, 1, 43, "Input"],
Cell[22616, 797, 30, 0, 33, "Text"],
Cell[22649, 799, 70, 1, 51, "Input"],
Cell[22722, 802, 40, 0, 33, "Text"],
Cell[22765, 804, 79, 1, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22881, 810, 37, 0, 73, "Section"],
Cell[22921, 812, 46, 1, 30, "Input"],
Cell[22970, 815, 26, 0, 33, "Text"],
Cell[22999, 817, 177, 3, 70, "Input"],
Cell[23179, 822, 36, 0, 33, "Text"],
Cell[23218, 824, 62, 1, 47, "Input"],
Cell[23283, 827, 30, 0, 33, "Text"],
Cell[23316, 829, 54, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23407, 835, 39, 0, 73, "Section"],
Cell[23449, 837, 46, 1, 30, "Input"],
Cell[23498, 840, 26, 0, 33, "Text"],
Cell[23527, 842, 359, 6, 130, "Input"],
Cell[23889, 850, 35, 0, 33, "Text"],
Cell[23927, 852, 78, 1, 32, "Input"],
Cell[24008, 855, 78, 1, 32, "Input"],
Cell[24089, 858, 40, 0, 33, "Text"],
Cell[24132, 860, 107, 2, 30, "Input"],
Cell[24242, 864, 107, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24386, 871, 38, 0, 73, "Section"],
Cell[24427, 873, 46, 1, 30, "Input"],
Cell[24476, 876, 191, 3, 50, "Input"],
Cell[24670, 881, 50, 1, 40, "Input"],
Cell[24723, 884, 65, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24825, 890, 48, 0, 73, "Section"],
Cell[24876, 892, 46, 1, 30, "Input"],
Cell[24925, 895, 26, 0, 33, "Text"],
Cell[24954, 897, 174, 3, 83, "Input"],
Cell[25131, 902, 37, 0, 33, "Text"],
Cell[25171, 904, 130, 2, 44, "Input"],
Cell[25304, 908, 26, 0, 33, "Text"],
Cell[25333, 910, 160, 2, 50, "Input"],
Cell[25496, 914, 38, 0, 33, "Text"],
Cell[25537, 916, 118, 2, 30, "Input"],
Cell[25658, 920, 78, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25773, 926, 44, 0, 73, "Section"],
Cell[25820, 928, 46, 1, 30, "Input"],
Cell[25869, 931, 26, 0, 33, "Text"],
Cell[25898, 933, 267, 4, 90, "Input"],
Cell[26168, 939, 113, 2, 51, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[26330, 947, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[26380, 951, 30, 0, 73, "Section"],
Cell[26413, 953, 46, 1, 30, "Input"],
Cell[26462, 956, 26, 0, 33, "Text"],
Cell[26491, 958, 238, 4, 90, "Input"],
Cell[26732, 964, 34, 0, 33, "Text"],
Cell[26769, 966, 64, 1, 32, "Input"],
Cell[26836, 969, 71, 1, 30, "Input"],
Cell[26910, 972, 49, 1, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26996, 978, 67, 0, 73, "Section"],
Cell[27066, 980, 46, 1, 30, "Input"],
Cell[27115, 983, 26, 0, 33, "Text"],
Cell[27144, 985, 237, 4, 90, "Input"],
Cell[27384, 991, 34, 0, 33, "Text"],
Cell[27421, 993, 78, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27536, 999, 36, 0, 73, "Section"],
Cell[27575, 1001, 46, 1, 30, "Input"],
Cell[27624, 1004, 192, 3, 70, "Input"],
Cell[27819, 1009, 33, 0, 33, "Text"],
Cell[27855, 1011, 118, 2, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28010, 1018, 41, 0, 73, "Section"],
Cell[28054, 1020, 46, 1, 30, "Input"],
Cell[28103, 1023, 26, 0, 33, "Text"],
Cell[28132, 1025, 134, 2, 50, "Input"],
Cell[28269, 1029, 35, 0, 33, "Text"],
Cell[28307, 1031, 69, 1, 47, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

