(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12082,        461]*)
(*NotebookOutlinePosition[     12754,        484]*)
(*  CellTagsIndexPosition[     12710,        480]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["22. Global laws of balance", "Title",
  InitializationCell->True],

Cell[BoxData[
    \(<< Lautrup`\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Launch of a small rocket", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Initial data", "Section"],

Cell["Parameters normalized to SI with all units removed", "Text"],

Cell[BoxData[{
    \(\(M\_0 = 1\ Kilogram // NToSI;\)\  (*\ Mass\ of\ rocket\ *) \), "\n", 
    \(\(M\_1 = 200\ Gram // NToSI;\)\  (*\ Mass\ of\ fuel\ *) \), "\n", 
    \(U = 50  Meter\/Second // NToSI; \  (*\ Gas\ velocity\ *) \n
    g\_0 = AccelerationDueToGravity // NToSI;\), "\n", 
    \(\(t\_1 = 3  Second // NToSI;\)\  (*\ 
      Burn\ time\ *) \), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1.2\ Kilogram\/Meter\^3 // NToSI;\)\  (*\ 
      Air\ density\ *) \), "\[IndentingNewLine]", 
    \(\(d\_0 = 6\ Centimeter // NToSI;\)\  (*\ 
      Rocket\ diameter\ *) \), "\[IndentingNewLine]", 
    \(\(\(C\_D = \ 1\)\(;\)\(\ \)\( (*\ 
      Drag\ coefficient\ *) \)\(\[IndentingNewLine]\)\)\)}], "Input"],

Cell[TextData[{
  "Mass loss rate (in ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(Kilogram/Second\)\()\)\)\)]]
}], "Text"],

Cell[BoxData[
    \(Q = \(M\_0 - M\_1\)\/t\_1\)], "Input"],

Cell["Rocket cross section", "Text"],

Cell[BoxData[
    \(\(A\_0 = \(1\/4\) \[Pi]\ d\_0\^2;\)\)], "Input"],

Cell["Takeoff parameter (must be >1)", "Text"],

Cell[BoxData[
    \(\(U\ Q\)\/\(\(M\_0\) g\_0\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution with no drag", "Section"],

Cell["Differential equation during burn", "Text"],

Cell[BoxData[
    \(\(va = 
        Block[{v}, 
          v /. \(NDSolve[{\(v'\)[
                      t] \[Equal] \(-g\_0\) + \(\(U\)\(\ \)\(Q\)\(\ \)\)\/\(M\
\_0 - Q\ t\), v[0] \[Equal] 0}, v, {t, 0, t\_1}]\)[\([1]\)]];\)\)], "Input"],

Cell["Velocity at end of burn", "Text"],

Cell[BoxData[
    \(v\_1 = va[t\_1]\)], "Input"],

Cell["Differential equation after burn", "Text"],

Cell[BoxData[
    \(\(vb = 
        Block[{v}, 
          v /. \(NDSolve[{\(v'\)[t] \[Equal] \(-g\_0\), 
                  v[t\_1] \[Equal] v\_1}, 
                v, {t, t\_1, 15}]\)[\([1]\)]];\)\)], "Input"],

Cell["Velocity function", "Text"],

Cell[BoxData[
    \(\(v[t_] = If[t < t\_1, va[t], vb[t]];\)\)], "Input"],

Cell["Differential equation for height", "Text"],

Cell[BoxData[
    \(\(z[t_] = 
        Block[{z}, 
          z[t] /. \(NDSolve[{\(z'\)[t] \[Equal] v[t], z[0] \[Equal] 0}, 
                z[t], {t, 0, 15}]\)[\([1]\)]];\)\)], "Input"],

Cell["Height at end of burn", "Text"],

Cell[BoxData[
    \(z\_1 = z[t\_1]\)], "Input"],

Cell["Time at impact on ground", "Text"],

Cell[BoxData[
    \(t\_2 = t /. FindRoot[z[t] \[Equal] 0, {t, 12, 13}]\)], "Input"],

Cell["Velocity at impact", "Text"],

Cell[BoxData[
    \(v[t\_2]\)], "Input"],

Cell["Maximum height", "Text"],

Cell[BoxData[
    \(FindMaximum[z[t], {t, 8, 9}]\)], "Input"],

Cell["Plot height as function of time", "Text"],

Cell[BoxData[
    \(\(fig1a = 
        Plot[z[t], {t, 0, t\_2}, 
          PlotStyle \[Rule] Dashing[{0.02}]];\)\)], "Input"],

Cell["Plot velocity as function of time", "Text"],

Cell[BoxData[
    \(\(fig2a = 
        Plot[v[t], {t, 0, t\_2}, 
          PlotStyle \[Rule] Dashing[{0.02}]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution with drag", "Section"],

Cell["Velocity during burn", "Text"],

Cell[BoxData[
    \(\(va = 
        Block[{v}, 
          v /. \(NDSolve[{\(v'\)[
                      t] \[Equal] \(-g\_0\) + \(U\ Q\  - \(1\/2\) \(\[Rho]\_0\
\) \(A\_0\) \(v[t]\^2\) C\_D\)\/\(M\_0 - Q\ t\), v[0] \[Equal] 0}, 
                v, {t, 0, t\_1}]\)[\([1]\)]];\)\)], "Input"],

Cell["Velocity at end of burn", "Text"],

Cell[BoxData[
    \(v\_1 = va[t\_1]\)], "Input"],

Cell["Velocity after burn", "Text"],

Cell[BoxData[
    \(\(vb = 
        Block[{v}, 
          v /. \(NDSolve[{\(v'\)[
                      t] \[Equal] \(-g\_0\) - \(\(\ \)\(\(1\/2\) \
\(\[Rho]\_0\) \(A\_0\) v[t] Abs[v[t]] C\_D\)\)\/M\_1, v[t\_1] \[Equal] v\_1}, 
                v, {t, t\_1, 15}]\)[\([1]\)]];\)\)], "Input"],

Cell["Velocity function", "Text"],

Cell[BoxData[
    \(\(v[t_] = If[t < t\_1, va[t], vb[t]];\)\)], "Input"],

Cell["Height", "Text"],

Cell[BoxData[
    \(\(z[t_] = 
        Block[{z}, 
          z[t] /. \(NDSolve[{\(z'\)[t] \[Equal] v[t], z[0] \[Equal] 0}, 
                z[t], {t, 0, 15}]\)[\([1]\)]];\)\)], "Input"],

Cell["Height at end of burn", "Text"],

Cell[BoxData[
    \(z\_1 = z[t\_1]\)], "Input"],

Cell["Time of impact on ground", "Text"],

Cell[BoxData[
    \(t\_2 = t /. FindRoot[z[t] \[Equal] 0, {t, 10, 11}]\)], "Input"],

Cell["Impact velocity", "Text"],

Cell[BoxData[
    \(v[t\_2]\)], "Input"],

Cell["Maximal height", "Text"],

Cell[BoxData[
    \(FindMaximum[z[t], {t, 8, 9}]\)], "Input"],

Cell["Plot height", "Text"],

Cell[BoxData[
    \(\(fig1b = Plot[z[t], {t, 0, t\_2}];\)\)], "Input"],

Cell["Plot velocity", "Text"],

Cell[BoxData[
    \(\(fig2b = Plot[v[t], {t, 0, t\_2}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Make figures", "Section"],

Cell[BoxData[
    \(\(Show[{fig1a, fig1b, Graphics[Text["\<(a)\>", {3, 170}]]}, 
          AxesLabel \[Rule] {"\<t\>", "\<z\>"}] \
// \[IndentingNewLine]Figure["\<transport1\>", 
          ImageSize \[Rule] 200];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[fig2a, fig2b, Graphics[Text["\<(b)\>", {8, 50}]], 
          AxesLabel \[Rule] {"\<t\>", "\<u\>"}, 
          PlotRange \[Rule] {\(-60\), 
              60}] // \[IndentingNewLine]Figure["\<transport2\>", 
          ImageSize \[Rule] 200];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["22.6.1 Lawn Sprinkler", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(R = 20\ Centimeter // N;\)\), "\[IndentingNewLine]", 
    \(\(d = 4  Millimeter // N;\)\), "\[IndentingNewLine]", 
    \(\(DD\  = \ 0.5\ Inch // N;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1\ Gram\/Centimeter\^3 // N;\)\), "\[IndentingNewLine]", 
    \(\(Q = \(\[Rho]\_0\) 0.25  Liter\/Second // 
          N;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity // N;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 30  \[Degree];\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega]\_0 = 1\/2*2\ \[Pi]/Second;\)\)}], "Input"],

Cell["Outlet cross section", "Text"],

Cell[BoxData[
    \(A = \(1\/4\) \[Pi]\ d\^2\)], "Input"],

Cell["Outlet velocity", "Text"],

Cell[BoxData[
    \(U = Q\/\(2 \( \[Rho]\_0\) A\) // ToSI\)], "Input"],

Cell["Azimuthal velocity relative to lawn", "Text"],

Cell[BoxData[
    \(v\_\[Phi] = R\ \[CapitalOmega]\_0 - U\ Cos[\[Alpha]] // ToSI\)], "Input"],

Cell["Friction moment", "Text"],

Cell[BoxData[
    \(N\_frict = 
      R\ U\ Q\ Cos[\[Alpha]] - \[CapitalOmega]\_0\ \(R\^2\) Q // 
        ToUnit[Newton]\)], "Input"],

Cell["Vertical velocity relative to lawn", "Text"],

Cell[BoxData[
    \(v\_z = U\ Sin[\[Alpha]]\)], "Input"],

Cell["Height of squirt", "Text"],

Cell[BoxData[
    \(h = \((U\ Sin[\[Alpha]])\)\^2\/\(2\ g\_0\)\)], "Input"],

Cell["Diameter of sprinkled region", "Text"],

Cell[BoxData[
    \(L = 4\ \(v\_\[Phi]\) v\_z\/g\_0 // ToSI\)], "Input"],

Cell["Area of sprinkled region", "Text"],

Cell[BoxData[
    \(A\_0 = \(1\/4\) \[Pi]\ L\^2\)], "Input"],

Cell["Rain per hour", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(Q\/\(\(\[Rho]\_0\) A\_0\) // 
      ToUnit[Millimeter/Hour]\)\)\)], "Input"],

Cell["Pressure drop", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]p = \(1\/2\) \(\[Rho]\_0\) \((U\^2 - \
\(\[CapitalOmega]\_0\^2\) R\^2)\) // ToUnit[Bar]\)], "Input"],

Cell["Total power consumption", "Text"],

Cell[BoxData[
    \(P\_0 = \(\[CapitalDelta]p\ Q\)\/\[Rho]\_0 // ToUnit[Watt]\)], "Input"],

Cell["Shaft power", "Text"],

Cell[BoxData[
    \(P = \(-\ N\_frict\)\ \[CapitalOmega]\_0 // ToUnit[Watt]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["22.7.1 Barrel of wine", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a\_0 = 1\/2\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(h = 2\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(a = 1\/2*5\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(L = 20\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(T = 0.5\ Second;\)\)}], "Input"],

Cell["Cross sections of barrel and spout", "Text"],

Cell[BoxData[{
    \(\(A = \[Pi]\ a\^2 // N;\)\), "\[IndentingNewLine]", 
    \(\(A\_0 = \[Pi]\ a\_0\^2 // N;\)\)}], "Input"],

Cell["Effective Length", "Text"],

Cell[BoxData[
    \(L\_0 = L + h\ A\/A\_0 // ToUnit[Centimeter]\)], "Input"],

Cell["Exit velocity", "Text"],

Cell[BoxData[
    \(U = \@\(2 \( g\_0\) h\) // ToSI\)], "Input"],

Cell["Rise time", "Text"],

Cell[BoxData[
    \(\[Tau] = \(2\ L\_0\)\/U // ToSI\)], "Input"],

Cell["96% rise time", "Text"],

Cell[BoxData[
    \(\[Tau]\_96 = 2  \[Tau]\)], "Input"],

Cell["Spilled wine in rise time", "Text"],

Cell[BoxData[
    \(V = A\ U\ \[Tau]\ Log[Cosh[T\/\[Tau]]] // ToUnit[Liter]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["Great belt", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(L = 20\ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 1\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = \(10\^\(-6\)\) Meter\^2/Second;\)\)}], "Input"],

Cell["Oscillation frequency", "Text"],

Cell[BoxData[
    \(\[Omega] = \@\(\(2  g\_0\)\/L\) // ToSI\)], "Input"],

Cell["Oscillation period", "Text"],

Cell[BoxData[
    \(T = \(2  \[Pi]\)\/\[Omega] // ToSI\)], "Input"],

Cell["Damping parameter", "Text"],

Cell[BoxData[
    \(\[Gamma] = \(\(4\ \[Nu]\)\/a\^2 // N\) // ToSI\)], "Input"],

Cell["Damping rate", "Text"],

Cell[BoxData[
    \(\[Lambda] = \(-\[Gamma]\) + \@\(\[Gamma]\^2 - \[Omega]\^2\) // 
        ToSI\)], "Input"],

Cell["Damping time", "Text"],

Cell[BoxData[
    \(1\/\[Lambda]\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 937}},
AutoGeneratedPackage->None,
WindowSize->{665, 912},
WindowMargins->{{75, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 71, 1, 95, "Title",
  InitializationCell->True],
Cell[1850, 56, 72, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1959, 63, 41, 0, 95, "Title"],
Cell[2003, 65, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2074, 70, 31, 0, 73, "Section"],
Cell[2108, 72, 66, 0, 33, "Text"],
Cell[2177, 74, 707, 12, 218, "Input"],
Cell[2887, 88, 126, 4, 33, "Text"],
Cell[3016, 94, 58, 1, 44, "Input"],
Cell[3077, 97, 36, 0, 33, "Text"],
Cell[3116, 99, 68, 1, 42, "Input"],
Cell[3187, 102, 46, 0, 33, "Text"],
Cell[3236, 104, 60, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3333, 110, 40, 0, 73, "Section"],
Cell[3376, 112, 49, 0, 33, "Text"],
Cell[3428, 114, 237, 5, 65, "Input"],
Cell[3668, 121, 39, 0, 33, "Text"],
Cell[3710, 123, 48, 1, 30, "Input"],
Cell[3761, 126, 48, 0, 33, "Text"],
Cell[3812, 128, 209, 5, 30, "Input"],
Cell[4024, 135, 33, 0, 33, "Text"],
Cell[4060, 137, 72, 1, 30, "Input"],
Cell[4135, 140, 48, 0, 33, "Text"],
Cell[4186, 142, 185, 4, 50, "Input"],
Cell[4374, 148, 37, 0, 33, "Text"],
Cell[4414, 150, 47, 1, 30, "Input"],
Cell[4464, 153, 40, 0, 33, "Text"],
Cell[4507, 155, 83, 1, 30, "Input"],
Cell[4593, 158, 34, 0, 33, "Text"],
Cell[4630, 160, 40, 1, 30, "Input"],
Cell[4673, 163, 30, 0, 33, "Text"],
Cell[4706, 165, 61, 1, 30, "Input"],
Cell[4770, 168, 47, 0, 33, "Text"],
Cell[4820, 170, 125, 3, 30, "Input"],
Cell[4948, 175, 49, 0, 33, "Text"],
Cell[5000, 177, 125, 3, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5162, 185, 37, 0, 73, "Section"],
Cell[5202, 187, 36, 0, 33, "Text"],
Cell[5241, 189, 289, 6, 118, "Input"],
Cell[5533, 197, 39, 0, 33, "Text"],
Cell[5575, 199, 48, 1, 30, "Input"],
Cell[5626, 202, 35, 0, 33, "Text"],
Cell[5664, 204, 289, 6, 118, "Input"],
Cell[5956, 212, 33, 0, 33, "Text"],
Cell[5992, 214, 72, 1, 30, "Input"],
Cell[6067, 217, 22, 0, 33, "Text"],
Cell[6092, 219, 185, 4, 50, "Input"],
Cell[6280, 225, 37, 0, 33, "Text"],
Cell[6320, 227, 47, 1, 30, "Input"],
Cell[6370, 230, 40, 0, 33, "Text"],
Cell[6413, 232, 83, 1, 30, "Input"],
Cell[6499, 235, 31, 0, 33, "Text"],
Cell[6533, 237, 40, 1, 30, "Input"],
Cell[6576, 240, 30, 0, 33, "Text"],
Cell[6609, 242, 61, 1, 30, "Input"],
Cell[6673, 245, 27, 0, 33, "Text"],
Cell[6703, 247, 70, 1, 30, "Input"],
Cell[6776, 250, 29, 0, 33, "Text"],
Cell[6808, 252, 70, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6915, 258, 31, 0, 73, "Section"],
Cell[6949, 260, 227, 4, 50, "Input"],
Cell[7179, 266, 281, 5, 70, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7509, 277, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[7559, 281, 40, 0, 73, "Section"],
Cell[7602, 283, 46, 1, 30, "Input"],
Cell[7651, 286, 26, 0, 33, "Text"],
Cell[7680, 288, 574, 9, 211, "Input"],
Cell[8257, 299, 36, 0, 33, "Text"],
Cell[8296, 301, 57, 1, 42, "Input"],
Cell[8356, 304, 31, 0, 33, "Text"],
Cell[8390, 306, 70, 1, 44, "Input"],
Cell[8463, 309, 51, 0, 33, "Text"],
Cell[8517, 311, 93, 1, 30, "Input"],
Cell[8613, 314, 31, 0, 33, "Text"],
Cell[8647, 316, 133, 3, 30, "Input"],
Cell[8783, 321, 50, 0, 33, "Text"],
Cell[8836, 323, 56, 1, 30, "Input"],
Cell[8895, 326, 32, 0, 33, "Text"],
Cell[8930, 328, 75, 1, 47, "Input"],
Cell[9008, 331, 44, 0, 33, "Text"],
Cell[9055, 333, 72, 1, 42, "Input"],
Cell[9130, 336, 40, 0, 33, "Text"],
Cell[9173, 338, 60, 1, 42, "Input"],
Cell[9236, 341, 29, 0, 33, "Text"],
Cell[9268, 343, 106, 2, 44, "Input"],
Cell[9377, 347, 29, 0, 33, "Text"],
Cell[9409, 349, 135, 2, 42, "Input"],
Cell[9547, 353, 39, 0, 33, "Text"],
Cell[9589, 355, 90, 1, 44, "Input"],
Cell[9682, 358, 27, 0, 33, "Text"],
Cell[9712, 360, 88, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9837, 366, 40, 0, 73, "Section"],
Cell[9880, 368, 46, 1, 30, "Input"],
Cell[9929, 371, 26, 0, 33, "Text"],
Cell[9958, 373, 351, 6, 157, "Input"],
Cell[10312, 381, 50, 0, 33, "Text"],
Cell[10365, 383, 125, 2, 50, "Input"],
Cell[10493, 387, 32, 0, 33, "Text"],
Cell[10528, 389, 76, 1, 44, "Input"],
Cell[10607, 392, 29, 0, 33, "Text"],
Cell[10639, 394, 64, 1, 32, "Input"],
Cell[10706, 397, 25, 0, 33, "Text"],
Cell[10734, 399, 64, 1, 42, "Input"],
Cell[10801, 402, 29, 0, 33, "Text"],
Cell[10833, 404, 55, 1, 30, "Input"],
Cell[10891, 407, 41, 0, 33, "Text"],
Cell[10935, 409, 88, 1, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11072, 416, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[11122, 420, 29, 0, 73, "Section"],
Cell[11154, 422, 46, 1, 30, "Input"],
Cell[11203, 425, 26, 0, 33, "Text"],
Cell[11232, 427, 260, 4, 90, "Input"],
Cell[11495, 433, 37, 0, 33, "Text"],
Cell[11535, 435, 72, 1, 51, "Input"],
Cell[11610, 438, 34, 0, 33, "Text"],
Cell[11647, 440, 67, 1, 42, "Input"],
Cell[11717, 443, 33, 0, 33, "Text"],
Cell[11753, 445, 79, 1, 42, "Input"],
Cell[11835, 448, 28, 0, 33, "Text"],
Cell[11866, 450, 109, 2, 33, "Input"],
Cell[11978, 454, 28, 0, 33, "Text"],
Cell[12009, 456, 45, 1, 42, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

