(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13142,        524]*)
(*NotebookOutlinePosition[     13816,        547]*)
(*  CellTagsIndexPosition[     13772,        543]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["23. Reactions", "Title",
  InitializationCell->True],

Cell[BoxData[
    \(<< Lautrup`\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["23.1.1 Jet on Wall", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(d = 1.5\ Inch // ToSI;\)\), "\n", 
    \(\(Q = 20\ Liter\/Second;\)\), "\n", 
    \(\(\[Rho]\_0 = 1  Gram\/\((Centimeter)\)\^3;\)\)}], "Input"],

Cell["Cross section", "Text"],

Cell[BoxData[
    \(A = \(1\/4\) \[Pi]\ d\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Average velocity", "Text"],

Cell[BoxData[
    \(v = Q\/A // ToSI\)], "Input"],

Cell[BoxData[
    \(v // ToUnit[Kilometer/Hour]\)], "Input"],

Cell["Reaction force", "Text"],

Cell[BoxData[
    \(R = \[Rho]\_0\ A\ v\^2 // ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.1.2 Barrel of wine", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 1\/2*5. \ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(b\  = \ 1\/2*1. \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(h = 2\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram\/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Spout cross section", "Text"],

Cell[BoxData[
    \(A = \[Pi]\ a\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Barrel cross section", "Text"],

Cell[BoxData[
    \(A\_0 = \ \[Pi]\ b\^2 // ToSI\)], "Input"],

Cell["Free fall velocity", "Text"],

Cell[BoxData[
    \(v = \@\(2 \( g\_0\) h\) // ToSI\)], "Input"],

Cell["Horizontal force", "Text"],

Cell[BoxData[
    \(F\_x = \(\[Rho]\_0\) \(v\^2\) A // ToUnit[Newton]\)], "Input"],

Cell["Vertical force", "Text"],

Cell[BoxData[
    \(F\_z = \(A\/A\_0\) F\_x // ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.1.3 Bent Pipe 90\[Degree]", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_0 = 1000. \ Kilogram\/Meter\^3;\)\), "\n", 
    \(\(v = 3\ Meter\/Second;\)\), "\n", 
    \(\(a = \(1\/2\) 1. \ Inch;\)\)}], "Input"],

Cell["Cross section", "Text"],

Cell[BoxData[
    \(A = \[Pi]\ a\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell[BoxData[
    \(F = \@2\ \(\[Rho]\_0\) \(v\^2\) A // ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fire hose ", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Older firehose", "Subsection"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(d\_1 = 2.5\ Inch;\)\), "\n", 
    \(\(d\_2 = 1.5\ Inch;\)\), "\n", 
    \(\(\[Rho]\_0 = 1  Gram/Centimeter\^3;\)\), "\[IndentingNewLine]", 
    \(\(Q = 20\ Liter/Second;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Hose cross section", "Text"],

Cell[BoxData[
    \(A\_1 = \(1\/4\) \[Pi]\ d\_1\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Nozzle cross section", "Text"],

Cell[BoxData[
    \(A\_2 = \(1\/4\) \[Pi]\ d\_2\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Hose average velocity", "Text"],

Cell[BoxData[
    \(v\_1 = Q\/A\_1 // ToSI\)], "Input"],

Cell["Nozzle average velocity", "Text"],

Cell[BoxData[
    \(v\_2 = Q\/A\_2 // ToSI\)], "Input"],

Cell["Hose with a bend", "Text"],

Cell[BoxData[
    \(F = \(\(\[Rho]\_0\) Q\^2\)\/A\_2 // ToUnit[Newton]\)], "Input"],

Cell["Nozzle itself", "Text"],

Cell[BoxData[
    \(F = \(\[Rho]\_0\) \(Q\^2\) \((1\/A\_2 - 1\/A\_1)\) // 
        ToUnit[Newton]\)], "Input"],

Cell["Nozzle reaction", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]R = \(\((A\_1 - A\_2)\)\^2\/\(2 \( A\_1\) 
                A\_2\)\) \(\(\[Rho]\_0\) Q\^2\)\/A\_2 // 
        ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Modern firehose", "Subsection"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(d\_1 = 5. \ Inch;\)\), "\n", 
    \(\(d\_2 = 3. \ Inch;\)\), "\n", 
    \(\(\[Rho]\_0 = 1.  Gram/Centimeter\^3;\)\), "\[IndentingNewLine]", 
    \(\(Q = 100. \ Liter/Second;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Hose cross section", "Text"],

Cell[BoxData[
    \(A\_1 = \(1\/4\) \[Pi]\ d\_1\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Nozzle cross section", "Text"],

Cell[BoxData[
    \(A\_2 = \(1\/4\) \[Pi]\ d\_2\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell["Hose average velocity", "Text"],

Cell[BoxData[
    \(v\_1 = Q\/A\_1 // ToSI\)], "Input"],

Cell["Nozzle average velocity", "Text"],

Cell[BoxData[
    \(v\_2 = Q\/A\_2 // ToSI\)], "Input"],

Cell["Hose with a bend", "Text"],

Cell[BoxData[
    \(F = \(\(\[Rho]\_0\) Q\^2\)\/A\_2 // ToUnit[Newton]\)], "Input"],

Cell["Nozzle itself", "Text"],

Cell[BoxData[
    \(F = \(\[Rho]\_0\) \(Q\^2\) \((1\/A\_2 - 1\/A\_1)\) // 
        ToUnit[Newton]\)], "Input"],

Cell["Nozzle reaction", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]R = \(\((A\_1 - A\_2)\)\^2\/\(2 \( A\_1\) 
                A\_2\)\) \(\(\[Rho]\_0\) Q\^2\)\/A\_2 // 
        ToUnit[Newton]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.3.2 Hydraulic turbine", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 2. \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(b = \ 3. \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(L = 0.5\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1\ Gram/Centimeter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Eta] = \(10\^\(-3\)\) Pascal\ Second;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega] = 60  RPM;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha]\_b = 0;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha]\_a = 1/3;\)\)}], "Input"],

Cell["Angular velocity", "Text"],

Cell[BoxData[
    \(\[CapitalOmega] // ToSI\)], "Input"],

Cell["Azimuthal velocity at exit", "Text"],

Cell[BoxData[
    \(\(v\_a = 0;\)\)], "Input"],

Cell["Azimuthal velocity at entrance", "Text"],

Cell[BoxData[
    \(v\_b = \[CapitalOmega]\ b // ToSI\)], "Input"],

Cell["Radial velocity at exit", "Text"],

Cell[BoxData[
    \(u\_a = \(-\(\(\[CapitalOmega]\ a\)\/\[Alpha]\_a\)\) // ToSI\)], "Input"],

Cell["Radial velocity at entrance from mass conservation", "Text"],

Cell[BoxData[
    \(u\_b = \(a\/b\) u\_a\)], "Input"],

Cell["Total velocity at entrance", "Text"],

Cell[BoxData[
    \(v = \@\(v\_b\^2 + u\_b\^2\) // ToSI\)], "Input"],

Cell["Free fall height at tentrance", "Text"],

Cell[BoxData[
    \(v\^2\/\(2  g\_0\) // ToSI\)], "Input"],

Cell["Total volume flux", "Text"],

Cell[BoxData[
    \(Q = 2  \[Pi]\ \ a\ \(u\_a\) L // ToSI\)], "Input"],

Cell["Total shaft moment", "Text"],

Cell[BoxData[
    \(M\_z = \((\((b\^2 - 
                    a\^2)\) \[CapitalOmega] + \(Q\/\(\(2\)  \(\[Pi]\)\(\ \
\)\(L\)\(\ \)\)\) \((\[Alpha]\_b - \[Alpha]\_a)\))\) \(\[Rho]\_0\) Q // 
        ToUnit[Mega\ Newton]\)], "Input"],

Cell["Total shaft power ", "Text"],

Cell[BoxData[
    \(P = \ \(M\_z\) \[CapitalOmega]\  // ToUnit[Mega\ Watt]\)], "Input"],

Cell["Pressure drop in rotor", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]p = \ \(1\/2\) \(\[Rho]\_0\) \((\(\[CapitalOmega]\^2\) \
\((b\^2 - 
                    a\^2)\) + \(Q\^2\/\((2  \[Pi]\ L)\)\^2\) \((\(1 + \
\[Alpha]\_a\^2\)\/a\^2 - \(1 + \[Alpha]\_b\^2\)\/b\^2)\))\) // 
        ToUnit[Bar]\)], "Input"],

Cell[BoxData[
    \(h = 
      v\^2\/\(2  g\_0\) + \[CapitalDelta]p\/\(\(\[Rho]\_0\) g\_0\) // 
        ToSI\)], "Input"],

Cell["Total pressure head from the reservoir", "Text"],

Cell[BoxData[
    \(\(\[Rho]\_0\) \(g\_0\) h // ToUnit[Bar]\)], "Input"],

Cell["Power of the excess pressure", "Text"],

Cell[BoxData[
    \(P\_0 = \[CapitalDelta]p\ Q // ToUnit[Mega\ Watt]\)], "Input"],

Cell["Total power of raised water", "Text"],

Cell[BoxData[
    \(P\/P\_0 // ToSI\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Rey = \ \(\(\[Rho]\_0\) \((\(-Q\))\)\)\/\(\[Eta]\ 2  \[Pi]\ L\) // 
        ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.3.3 Free-stream air blower", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 10\ Centimeter;\)\), "\n", 
    \(\(b = 30  Centimeter;\)\), "\n", 
    \(\(\[CapitalOmega] = 600. \ RPM;\)\), "\n", 
    \(\(\[Rho]\_0 = \ 1.1\ \ Kilogram/Meter\^3;\)\), "\n", 
    \(\(\[Gamma]\_1 = \ \(-45. \)\ Degree;\)\)}], "Input"],

Cell["Angular velocity", "Text"],

Cell[BoxData[
    \(\[CapitalOmega] // ToSI\)], "Input"],

Cell["Radial slopes", "Text"],

Cell[BoxData[{
    \(\[Beta]\_1 = Tan[\[Gamma]\_1]/b // ToSI\), "\n", 
    \(\(\[Beta]\_2 = \ 0;\)\)}], "Input"],

Cell["Free-stream velocity", "Text"],

Cell[BoxData[
    \(U = \(\(-\[CapitalOmega]\)/\[Beta]\_1 // ToSI\) // N\)], "Input"],

Cell["Opening area", "Text"],

Cell[BoxData[
    \(A = \(\[Pi] \((b\^2 - a\^2)\) // ToSI\) // N\)], "Input"],

Cell["Volume flux", "Text"],

Cell[BoxData[
    \(Q = U\ A // ToSI\)], "Input"],

Cell[BoxData[
    \(P = \(1\/2\) \((a\^2 + 
              b\^2)\) \((\[Beta]\_2 - \[Beta]\_1)\) \[CapitalOmega]\ U\ \
\[Rho]\_0\ Q // ToUnit[Watt]\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]p\  = \(-\ \(1\/2\)\) \(\[Rho]\_0\) \((\[Beta]\_2\^2 - \
\[Beta]\_1\^2)\) \(U\^2\) b\^2 // ToUnit[Pascal]\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]p\ Q // ToUnit[Watt]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.3.4 Model turbine", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(DD = 3\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalOmega] = 60\ RPM;\)\), "\[IndentingNewLine]", 
    \(\(Q = 236.87\ Meter\^3/Second;\)\), "\[IndentingNewLine]", 
    \(\(scale = 0.1;\)\)}], "Input"],

Cell["Flux parameter", "Text"],

Cell[BoxData[
    \(q = Q\/\(\[CapitalOmega]\ DD\^3\) // ToSI\)], "Input"],

Cell["Reynolds number at same q but scaled down", "Text"],

Cell[BoxData[
    \(Rey = scale\^2*7.5398*^7\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["23.1 Bent Pipe 180\[Degree]", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_0 = 1000\ Kilogram\/Meter\^3;\)\), "\n", 
    \(\(v = 1  Meter\/Second;\)\), "\n", 
    \(\(a = \(1\/2\) 1. \ Inch;\)\)}], "Input"],

Cell["Cross section", "Text"],

Cell[BoxData[
    \(A = \[Pi]\ a\^2 // ToUnit[Centimeter\^2]\)], "Input"],

Cell[BoxData[
    \(F = 2\ \(\[Rho]\_0\) \(v\^2\) A // ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["23.2 Firefighter", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_0 = \(10\^3\) Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(a = \(1\/2\) 5. \ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(Q = 40. \ Liter/Second;\)\)}], "Input"],

Cell["Cross section", "Text"],

Cell[BoxData[
    \(A = \[Pi]\ a\^2\)], "Input"],

Cell["Average velocity", "Text"],

Cell[BoxData[
    \(U = Q\/A // ToSI\)], "Input"],

Cell["Total force", "Text"],

Cell[BoxData[
    \(F = \(\@2\) \(\[Rho]\_0\) A\ U\^2 // ToUnit[Newton]\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 937}},
AutoGeneratedPackage->None,
WindowSize->{630, 991},
WindowMargins->{{29, Automatic}, {-39, Automatic}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 58, 1, 95, "Title",
  InitializationCell->True],
Cell[1837, 56, 72, 2, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1946, 63, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[1996, 67, 37, 0, 73, "Section"],
Cell[2036, 69, 46, 1, 30, "Input"],
Cell[2085, 72, 26, 0, 33, "Text"],
Cell[2114, 74, 166, 3, 99, "Input"],
Cell[2283, 79, 29, 0, 33, "Text"],
Cell[2315, 81, 82, 1, 42, "Input"],
Cell[2400, 84, 32, 0, 33, "Text"],
Cell[2435, 86, 49, 1, 42, "Input"],
Cell[2487, 89, 60, 1, 30, "Input"],
Cell[2550, 92, 30, 0, 33, "Text"],
Cell[2583, 94, 73, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2693, 100, 40, 0, 73, "Section"],
Cell[2736, 102, 46, 1, 30, "Input"],
Cell[2785, 105, 26, 0, 33, "Text"],
Cell[2814, 107, 322, 5, 151, "Input"],
Cell[3139, 114, 35, 0, 33, "Text"],
Cell[3177, 116, 73, 1, 31, "Input"],
Cell[3253, 119, 36, 0, 33, "Text"],
Cell[3292, 121, 61, 1, 30, "Input"],
Cell[3356, 124, 34, 0, 33, "Text"],
Cell[3393, 126, 64, 1, 32, "Input"],
Cell[3460, 129, 32, 0, 33, "Text"],
Cell[3495, 131, 82, 1, 30, "Input"],
Cell[3580, 134, 30, 0, 33, "Text"],
Cell[3613, 136, 74, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3724, 142, 47, 0, 73, "Section"],
Cell[3774, 144, 46, 1, 30, "Input"],
Cell[3823, 147, 26, 0, 33, "Text"],
Cell[3852, 149, 164, 3, 110, "Input"],
Cell[4019, 154, 29, 0, 33, "Text"],
Cell[4051, 156, 73, 1, 31, "Input"],
Cell[4127, 159, 84, 1, 32, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4248, 165, 29, 0, 73, "Section"],
Cell[4280, 167, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[4351, 172, 36, 0, 38, "Subsection"],
Cell[4390, 174, 26, 0, 33, "Text"],
Cell[4419, 176, 261, 5, 110, "Input"],
Cell[4683, 183, 34, 0, 33, "Text"],
Cell[4720, 185, 88, 1, 42, "Input"],
Cell[4811, 188, 36, 0, 33, "Text"],
Cell[4850, 190, 88, 1, 42, "Input"],
Cell[4941, 193, 37, 0, 33, "Text"],
Cell[4981, 195, 55, 1, 44, "Input"],
Cell[5039, 198, 39, 0, 33, "Text"],
Cell[5081, 200, 55, 1, 44, "Input"],
Cell[5139, 203, 32, 0, 33, "Text"],
Cell[5174, 205, 83, 1, 47, "Input"],
Cell[5260, 208, 29, 0, 33, "Text"],
Cell[5292, 210, 110, 2, 44, "Input"],
Cell[5405, 214, 31, 0, 33, "Text"],
Cell[5439, 216, 171, 3, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5647, 224, 37, 0, 38, "Subsection"],
Cell[5687, 226, 26, 0, 33, "Text"],
Cell[5716, 228, 265, 5, 110, "Input"],
Cell[5984, 235, 34, 0, 33, "Text"],
Cell[6021, 237, 88, 1, 42, "Input"],
Cell[6112, 240, 36, 0, 33, "Text"],
Cell[6151, 242, 88, 1, 42, "Input"],
Cell[6242, 245, 37, 0, 33, "Text"],
Cell[6282, 247, 55, 1, 44, "Input"],
Cell[6340, 250, 39, 0, 33, "Text"],
Cell[6382, 252, 55, 1, 44, "Input"],
Cell[6440, 255, 32, 0, 33, "Text"],
Cell[6475, 257, 83, 1, 47, "Input"],
Cell[6561, 260, 29, 0, 33, "Text"],
Cell[6593, 262, 110, 2, 44, "Input"],
Cell[6706, 266, 31, 0, 33, "Text"],
Cell[6740, 268, 171, 3, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6960, 277, 43, 0, 73, "Section"],
Cell[7006, 279, 46, 1, 30, "Input"],
Cell[7055, 282, 26, 0, 33, "Text"],
Cell[7084, 284, 555, 9, 190, "Input"],
Cell[7642, 295, 32, 0, 33, "Text"],
Cell[7677, 297, 56, 1, 30, "Input"],
Cell[7736, 300, 42, 0, 33, "Text"],
Cell[7781, 302, 46, 1, 30, "Input"],
Cell[7830, 305, 46, 0, 33, "Text"],
Cell[7879, 307, 66, 1, 30, "Input"],
Cell[7948, 310, 39, 0, 33, "Text"],
Cell[7990, 312, 92, 1, 44, "Input"],
Cell[8085, 315, 66, 0, 33, "Text"],
Cell[8154, 317, 53, 1, 40, "Input"],
Cell[8210, 320, 42, 0, 33, "Text"],
Cell[8255, 322, 68, 1, 33, "Input"],
Cell[8326, 325, 45, 0, 33, "Text"],
Cell[8374, 327, 58, 1, 47, "Input"],
Cell[8435, 330, 33, 0, 33, "Text"],
Cell[8471, 332, 70, 1, 30, "Input"],
Cell[8544, 335, 34, 0, 33, "Text"],
Cell[8581, 337, 230, 4, 43, "Input"],
Cell[8814, 343, 34, 0, 33, "Text"],
Cell[8851, 345, 87, 1, 30, "Input"],
Cell[8941, 348, 38, 0, 33, "Text"],
Cell[8982, 350, 271, 5, 47, "Input"],
Cell[9256, 357, 121, 3, 47, "Input"],
Cell[9380, 362, 54, 0, 33, "Text"],
Cell[9437, 364, 72, 1, 30, "Input"],
Cell[9512, 367, 44, 0, 33, "Text"],
Cell[9559, 369, 81, 1, 30, "Input"],
Cell[9643, 372, 43, 0, 33, "Text"],
Cell[9689, 374, 48, 1, 44, "Input"],
Cell[9740, 377, 31, 0, 33, "Text"],
Cell[9774, 379, 113, 2, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9924, 386, 48, 0, 73, "Section"],
Cell[9975, 388, 46, 1, 30, "Input"],
Cell[10024, 391, 26, 0, 33, "Text"],
Cell[10053, 393, 263, 5, 110, "Input"],
Cell[10319, 400, 32, 0, 33, "Text"],
Cell[10354, 402, 56, 1, 30, "Input"],
Cell[10413, 405, 29, 0, 33, "Text"],
Cell[10445, 407, 112, 2, 50, "Input"],
Cell[10560, 411, 36, 0, 33, "Text"],
Cell[10599, 413, 85, 1, 30, "Input"],
Cell[10687, 416, 28, 0, 33, "Text"],
Cell[10718, 418, 77, 1, 31, "Input"],
Cell[10798, 421, 27, 0, 33, "Text"],
Cell[10828, 423, 49, 1, 30, "Input"],
Cell[10880, 426, 159, 3, 42, "Input"],
Cell[11042, 431, 153, 2, 42, "Input"],
Cell[11198, 435, 68, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11303, 441, 39, 0, 73, "Section"],
Cell[11345, 443, 46, 1, 30, "Input"],
Cell[11394, 446, 26, 0, 33, "Text"],
Cell[11423, 448, 233, 4, 90, "Input"],
Cell[11659, 454, 30, 0, 33, "Text"],
Cell[11692, 456, 74, 1, 43, "Input"],
Cell[11769, 459, 57, 0, 33, "Text"],
Cell[11829, 461, 57, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11935, 468, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[11985, 472, 46, 0, 73, "Section"],
Cell[12034, 474, 46, 1, 30, "Input"],
Cell[12083, 477, 26, 0, 33, "Text"],
Cell[12112, 479, 162, 3, 110, "Input"],
Cell[12277, 484, 29, 0, 33, "Text"],
Cell[12309, 486, 73, 1, 31, "Input"],
Cell[12385, 489, 82, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12504, 495, 35, 0, 73, "Section"],
Cell[12542, 497, 46, 1, 30, "Input"],
Cell[12591, 500, 26, 0, 33, "Text"],
Cell[12620, 502, 207, 3, 84, "Input"],
Cell[12830, 507, 29, 0, 33, "Text"],
Cell[12862, 509, 48, 1, 30, "Input"],
Cell[12913, 512, 32, 0, 33, "Text"],
Cell[12948, 514, 49, 1, 42, "Input"],
Cell[13000, 517, 27, 0, 33, "Text"],
Cell[13030, 519, 84, 1, 32, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

