(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     20592,        784]*)
(*NotebookOutlinePosition[     21265,        807]*)
(*  CellTagsIndexPosition[     21221,        803]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["24. Surface waves", "Title",
  InitializationCell->True],

Cell[BoxData[{
    \(<< Lautrup`\), "\n", 
    \(<< Graphics`Graphics`\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Figures", "Title"],

Cell[CellGroupData[{

Cell["Superposition", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Choice of parameters", "Text"],

Cell[BoxData[{
    \(\(\[CapitalDelta]k = 2  \[Pi];\)\), "\[IndentingNewLine]", 
    \(\(k = 8  \[CapitalDelta]k;\)\)}], "Input"],

Cell["Height of wave", "Text"],

Cell[BoxData[
    \(\(h[x_] = 2\ Cos[k\ x] Cos[\(1\/2\) \[CapitalDelta]k\ x];\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[h[x], {x, 0, 5}, PlotPoints \[Rule] 200, \ 
          AspectRatio \[Rule] 0.3, 
          AxesLabel \[Rule] {"\<x\>", "\<h\>"}] \
// \[IndentingNewLine]Figure["\<waves1\>"];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gaussian wave packet", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[Exp[\(-x\^2\)] Cos[2\ \[Pi]\ 2\ x], {x, \(-4\), 4}, 
          PlotRange \[Rule] {\(-1\), 1}, Ticks \[Rule] None, 
          Axes \[Rule] None] // \[IndentingNewLine]Figure["\<waves4\>", 
          ImageSize \[Rule] 200];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Deep-water waves", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Choice of parameters", "Text"],

Cell[BoxData[{
    \(\(g\_0 = 10;\)\), "\[IndentingNewLine]", 
    \(\(d = 10;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000;\)\)}], "Input"],

Cell["Circular frequency", "Text"],

Cell[BoxData[
    \(\(\[Omega] = \@\(\(g\_0\) \(k\)\(\ \)\);\)\)], "Input"],

Cell["Phase velocity", "Text"],

Cell[BoxData[
    \(\(c = \[Omega]\/k;\)\)], "Input"],

Cell["Group velocity", "Text"],

Cell[BoxData[
    \(\(c\_g = \[PartialD]\_k \[Omega];\)\)], "Input"],

Cell["Wave number", "Text"],

Cell[BoxData[
    \(\(k = \(2  \[Pi]\)\/\[Lambda];\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[{c, c\_g}, {\[Lambda], 0.1, 200}, 
          AxesLabel \[Rule] {"\<\[Lambda][m]\>", "\<[m/s]\>"}, 
          PlotRange \[Rule] {0, 17}, \ 
          PlotStyle \[Rule] {Text["\<phase\>", {100, 14}], 
              Text["\<group\>", {100, 
                  7}]}] // \[IndentingNewLine]Figure["\<waves3\>"];\)\)], \
"Input"],

Cell[BoxData[
    \(c /. \[Lambda] \[Rule] 150 // N\)], "Input"],

Cell[BoxData[
    \(%*3600\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gravity waves", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[TextData[{
  "Use normalization ",
  Cell[BoxData[
      \(TraditionalForm\`d = 1, \ \(g\_0 = 1;\)\)]]
}], "Text"],

Cell[BoxData[
    \(\(\[Omega] = \@\(k\ Tanh[k]\);\)\)], "Input"],

Cell[BoxData[
    \(\(c = \[Omega]\/k;\)\)], "Input"],

Cell[BoxData[
    \(\(c\_g = \[PartialD]\_k \[Omega];\)\)], "Input"],

Cell[BoxData[
    \(\(k = 1\/\[Xi];\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[{c, c\_g}, {\[Xi], 0.001, 2}, 
          AxesLabel \[Rule] {\*"\"\<\!\(\[Lambda]\/\(2  \[Pi]\\\ d\)\)\>\"", \
\*"\"\<\!\(c\/\@\(\(g\_0\) d\)\)\>\""}, PlotRange \[Rule] {0, 1}, 
          PlotStyle \[Rule] {Text["\<phase\>", {1, 0.93}], 
              Text["\<group\>", {1, 
                  0.75}]}] // \[IndentingNewLine]Figure["\<waves7\>"];\)\)], \
"Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Water energy surface density", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(v\_x = 
        a\ \[Omega]\ \(Cosh[k\ \((z + d)\)]\/Sinh[k\ d]\) 
          Cos[k\ x - \[Omega]\ t];\)\), "\[IndentingNewLine]", 
    \(\(v\_z = 
        a\ \[Omega]\ \(Sinh[k\ \((z + d)\)]\/Sinh[k\ d]\) 
          Sin[k\ x - \[Omega]\ t];\)\), "\[IndentingNewLine]", 
    \(\(h = a\ Cos[k\ x\  - \ \[Omega]\ t];\)\)}], "Input"],

Cell[BoxData[
    \(T = \[Integral]\_\(-d\)\%0\( 1\/2\) \(\[Rho]\_0\) \((v\_x\^2 + 
                v\_z\^2)\) \[DifferentialD]z // Simplify\)], "Input"],

Cell[BoxData[
    \(V = \(1\/2\) \(\[Rho]\_0\) \(g\_0\) h\^2 // Simplify\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Capillary Waves", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[{
    \(\(\[Lambda]\_c = 17.1;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = 9820;\)\)}], "Input"],

Cell[BoxData[
    \(\(\[Omega] = \@\(\(g\_0\) \(\(2  \[Pi]\)\/\[Lambda]\) \((1 + \
\[Lambda]\_c\^2\/\[Lambda]\^2)\)\);\)\)], "Input"],

Cell[BoxData[
    \(\(ParametricPlot[{\[Omega]\/\(2  \[Pi]\), \[Lambda]}, {\[Lambda], 0.1, 
          10}, PlotRange \[Rule] {{0, 500}, {0, 10}}];\)\)], "Input"],

Cell[BoxData[
    \(\[Omega]\/\[Pi] /. \[Lambda] \[Rule] 10\)], "Input"],

Cell[CellGroupData[{

Cell["Velocities", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[BoxData[
    \(\(\[Omega] = \@\(k\ \((1 + k\^2)\)\);\)\)], "Input"],

Cell[BoxData[
    \(\(c = \[Omega]\/k;\)\)], "Input"],

Cell[BoxData[
    \(\(c\_g = \[PartialD]\_k \[Omega];\)\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_k c // Simplify\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_k c\_g // Simplify\)], "Input"],

Cell[BoxData[
    \(sol = Solve[\(-1\) + 6\ k\^2 + 3\ k\^4 \[Equal] 0, k]\)], "Input"],

Cell[BoxData[
    \(sol // N\)], "Input"],

Cell[BoxData[
    \(1\/%\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot", "Section"],

Cell[BoxData[
    \(\(k = 1\/\[Xi];\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[{c, c\_g}, {\[Xi], 0.001, 5}, 
          AxesLabel \[Rule] {\*"\"\<\!\(\[Lambda]\/\(2  \[Pi]\\\ \
R\_c\)\)\>\"", \*"\"\<\!\(c\/\@\(\(g\_0\) R\_c\)\)\>\""}, 
          PlotRange \[Rule] {0, 5}, 
          PlotStyle \[Rule] {Text["\<phase\>", {3, 2}], 
              Text["\<group\>", {3, 
                  0.83}]}] // \[IndentingNewLine]Figure["\<waves8\>"];\)\)], \
"Input"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 0.073\ Joule/Meter\^2;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\)}], "Input"],

Cell[BoxData[
    \(R\_c = \@\(\(\[Alpha]\/\(\(\[Rho]\_0\) g\_0\)\)\(\ \)\) // 
        ToUnit[Milli\ Meter]\)], "Input"],

Cell[BoxData[
    \(\[Lambda]\_c = 2  \[Pi]\ R\_c // ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(c\_0 = \@\(\(g\_0\) R\_c\) // ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(FindMinimum[c\_g, {\[Xi], 2}]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Viscous attenuation", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters for water in SI units", "Text"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity // NToSI;\)\), "\n", 
    \(\(\[Alpha] = 0.073;\)\), "\n", 
    \(\(\[Rho]\_0 = 1000;\)\), "\n", 
    \(\(\[Nu] = 10\^\(-6\);\)\)}], "Input"],

Cell["Capillary constant", "Text"],

Cell[BoxData[
    \(R\_c = \@\(\[Alpha]\/\(\(\[Rho]\_0\) g\_0\)\)\)], "Input"],

Cell["Capillary wavelength", "Text"],

Cell[BoxData[
    \(\[Lambda]\_c = 2  \[Pi]\ R\_c\)], "Input"],

Cell["Circular frequency", "Text"],

Cell[BoxData[
    \(\(\[Omega] = \@\(\(g\_0\) k\ \((1 + \(k\^2\) R\_c\^2)\)\);\)\)], "Input"],

Cell["Phase velocity", "Text"],

Cell[BoxData[
    \(\(c = \[Omega]\/k;\)\)], "Input"],

Cell["Group velocity", "Text"],

Cell[BoxData[
    \(\(c\_g = \[PartialD]\_k \[Omega];\)\)], "Input"],

Cell["Attenuation length", "Text"],

Cell[BoxData[
    \(\(\[Xi] = c\_g\/\(2  \[Nu]\ k\^2\);\)\)], "Input"],

Cell["Wave number", "Text"],

Cell[BoxData[
    \(\(k = \(2  \[Pi]\)\/\[Lambda];\)\)], "Input"],

Cell["Plot", "Text"],

Cell[BoxData[
    \(\(Plot[Log[10, \[Xi] /. \[Lambda] \[Rule] \ 10\^x], {x, \(-3.2\), 0}, 
          AxesOrigin \[Rule] {\(-3.2\), \(-2.2\)}, 
          PlotRange \[Rule] {\(-2.2\), 4}, 
          Ticks \[Rule] {{{\(-3\), 0.001}, {\(-2\), 0.01}, {\(-1\), 0.1}, {0, 
                  1}}, {{\(-3\), 0.001}, {\(-2\), 0.01}, {\(-1\), 0.1}, {0, 
                  1}, {1, 10}, {2, 100}, {3, 1000}, {4, 10000}}}, 
          AxesLabel \[Rule] {"\<\[Lambda] [m]\>", "\<\[Xi][m]\>"}, 
          PlotRange \[Rule] 
            All] // \[IndentingNewLine]Figure["\<waves6\>"];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Internal waves", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters for air and water", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_2 = 1.2\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_1 = 1000\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = \ 0.073\ Newton/Meter;\)\)}], "Input"],

Cell[BoxData[
    \(R\_c = \ \@\(\[Alpha]\/\(\((\[Rho]\_1 - \[Rho]\_2)\) g\_0\)\) // 
        ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(\[Lambda]\_c = 2\ \[Pi]\ R\_c\)], "Input"],

Cell[BoxData[
    \(U\_c = \@\(2 \( g\_0\) \(R\_c\) \((\[Rho]\_1\/\[Rho]\_2 - \[Rho]\_2\/\
\[Rho]\_1)\)\) // ToSI\)], "Input"],

Cell[BoxData[
    \(\(LogLinearPlot[\@\(\(\[Lambda]\^2 + 1\)\/\(2  \[Lambda]\)\), \
{\[Lambda], 0.01, 100}, PlotRange \[Rule] {All, {0, 7}}, 
          AxesLabel \[Rule] {\*"\"\<\[Lambda]/\!\(\[Lambda]\_c\)\>\"", \
\*"\"\<U/\!\(U\_c\)\>\""}, PlotPoints \[Rule] 500, 
          PlotStyle \[Rule] {Text["\<unstable\>", {0, 4}], 
              Text["\<stable\>", {0, 
                  0.5}]}] // \[IndentingNewLine]Figure["\<waves9\>"];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Wave spectrum", "Title"],

Cell[CellGroupData[{

Cell["Pierson-Moskowitz spectrum", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Spectrum", "Text"],

Cell[BoxData[
    \(\(S[\[Omega]_, U_] = \[Alpha]\ \(g\_0\^2\/\[Omega]\^5\) 
          Exp[\(-\[Beta]\) \((g\_0\/\(U\ \[Omega]\))\)\^4];\)\)], "Input"],

Cell["Empirical values in SI units", "Text"],

Cell[BoxData[{
    \(\(\[Alpha] = 8.1\ 10\^\(-3\);\)\), "\[IndentingNewLine]", 
    \(\(\[Beta] = 0.74;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity // NToSI;\)\)}], "Input"],

Cell["Plot of spectrum", "Text"],

Cell[BoxData[
    \(\(Plot[
          Map[S[\[Omega], #] &, {12, 15, 18}] // Evaluate, {\[Omega], 0, 
            1.5}, PlotRange \[Rule] All, 
          AxesLabel \[Rule] {\*"\"\<\[Omega][\!\(s\^\(-1\)\)]\>\"", \*"\"\<S[\
\!\(m\^2\)s]\>\""}, 
          PlotStyle \[Rule] {Text["\<12\>", {0.7, 1.6}], 
              Text["\<15\>", {0.58, 3.9}], 
              Text["\<18\>", {0.48, 
                  9.2}]}] // \[IndentingNewLine]Figure["\<waves10\>"];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fetch", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Non-metric units", "Text"],

Cell[BoxData[
    \(knot = Knot // NToSI\)], "Input"],

Cell[BoxData[
    \(mile = Mile // NToUnit[Kilometer]\)], "Input"],

Cell["\<\
Data on wind speed (knots) and fetch  (miles) from Pierson and Moskowitz\
\>", "Text"],

Cell[BoxData[
    \(\(data = {{20\ , 75}, {25, 155}, {30, 280}, {35, 460}, {40, 
            710}};\)\)], "Input"],

Cell["Convert to metric units", "Text"],

Cell[BoxData[
    \(data1 = Map[{#[\([1]\)] knot, #[\([2]\)]\ mile} &, data]\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        ListPlot[data1, PlotStyle \[Rule] {PointSize[0.02]}];\)\)], "Input"],

Cell[BoxData[
    \(fit = FindFit[data1, A\ U\^\[Alpha], {\[Alpha], A}, U]\)], "Input"],

Cell[BoxData[
    \(\(figb = Plot[A\ U\^\[Alpha] /. fit, {U, 0, 25}];\)\)], "Input"],

Cell[BoxData[
    \(\(Show[figa, figb];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["25.3.1 Tsunami", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a\_0 = 1\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(d\_0 = 4000\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = \ AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Lambda]\_0 = 500\ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\)}], "Input"],

Cell["Shallow-water phase velocity", "Text"],

Cell[BoxData[
    \(c\_0 = \@\(\(g\_\(\(0\)\(\ \)\)\) d\_0\) // ToSI\)], "Input"],

Cell[BoxData[
    \(c\_0 // ToUnit[Kilometer/Hour]\)], "Input"],

Cell["Period", "Text"],

Cell[BoxData[
    \(\[Tau] = \[Lambda]\_0\/c\_0 // ToSI\)], "Input"],

Cell[BoxData[
    \(\[Tau] // ToUnit[Minute]\)], "Input"],

Cell["Energy per unit of  crest length ", "Text"],

Cell[BoxData[
    \(EL = \(1\/2\) \(\[Rho]\_0\) \(g\_0\) \(a\_0\^2\) \[Lambda]\_0 // 
        ToUnit[Joule]\)], "Input"],

Cell["Power per unit of crest length ", "Text"],

Cell[BoxData[
    \(PL = \(1\/2\) \(\[Rho]\_0\) \(g\_0\) \(a\_0\^2\) c\_0 // 
        ToUnit[Watt]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.5.1 Internal waves in a saline/brackish interface", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_2 = 1000\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_1 = 1.04\ \[Rho]\_2;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = \ 0.073\ Newton/Meter;\)\)}], "Input"],

Cell[BoxData[
    \(R\_c = \ \@\(\[Alpha]\/\(\((\[Rho]\_1 - \[Rho]\_2)\) g\_0\)\) // 
        ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(\[Lambda]\_c = 2\ \[Pi]\ R\_c\)], "Input"],

Cell[BoxData[
    \(\(k = 1\/R\_c;\)\)], "Input"],

Cell[BoxData[
    \(\(\[Omega] = \@\(\(g\_0\) k\ \(\(\[Rho]\_1 - \[Rho]\_2\)\/\(\[Rho]\_1 + \
\[Rho]\_2\)\) \((1 + \(k\^2\) R\_c\^2)\)\);\)\)], "Input"],

Cell[BoxData[
    \(\[Tau] = \(2  \[Pi]\)\/\[Omega] // ToSI\)], "Input"],

Cell[BoxData[
    \(c = \[Omega]\/k // ToUnit[Centimeter/Second]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.6.1 Wave energy", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho] = \(10\^3\) Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(a = 10  Centimeter;\)\)}], "Input"],

Cell["Surface energy density", "Text"],

Cell[BoxData[
    \(Es = \(1\/2\) \[Rho]\ \(g\_0\) a\^2 // ToUnit[Joule]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.6.2 Viscous attenuation", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(\[Alpha] = 0.073\ Newton/Meter;\)\), "\n", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\), "\n", 
    \(\(\[Nu] = 10\^\(-6\)\ Meter\^2/Second;\)\)}], "Input"],

Cell[BoxData[
    \(R\_c = \@\(\[Alpha]\/\(\(\[Rho]\_0\) g\_0\)\) // 
        ToUnit[Millimeter]\)], "Input"],

Cell[BoxData[
    \(\[Lambda]\_c = 2  \[Pi]\ R\_c // ToUnit[Centimeter]\)], "Input"],

Cell[BoxData[
    \(\(\[Omega] = \@\(\(g\_0\) k\ \((1 + \(k\^2\) R\_c\^2)\)\) // 
          ToSI;\)\)], "Input"],

Cell[BoxData[
    \(\(c = \[Omega]\/k;\)\)], "Input"],

Cell[BoxData[
    \(\(c\_g = \[PartialD]\_k \[Omega];\)\)], "Input"],

Cell[BoxData[
    \(\(k = \(2  \[Pi]\)\/\[Lambda];\)\)], "Input"],

Cell[BoxData[
    \(\(\[Xi] = c\_g\/\(2  \[Nu]\ k\^2\);\)\)], "Input"],

Cell[BoxData[
    \(\[Xi] /. \[Lambda] \[Rule] 1\ Meter // ToSI\)], "Input"],

Cell[BoxData[
    \(\[Xi] /. \[Lambda] \[Rule] \[Lambda]\_c // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.7.1 Peak wave properties", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Alpha] = 8.1\ 10\^\(-3\);\)\), "\[IndentingNewLine]", 
    \(\(\[Beta] = 0.74;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(U = 15\ Meter/Second;\)\)}], "Input"],

Cell["Coefficients of peak values", "Text"],

Cell[BoxData[
    \(\[Gamma] = \(1\/2\) \@\(\[Alpha]\/\[Beta]\)\)], "Input"],

Cell[BoxData[
    \(\[Delta] = \((\(4  \[Beta]\)\/5)\)\^\(1/4\)\)], "Input"],

Cell["rms amplitude", "Text"],

Cell[BoxData[
    \(a = \[Gamma]  U\^2\/g\_0\)], "Input"],

Cell["Frequency and period", "Text"],

Cell[BoxData[
    \(\[Omega] = \[Delta]  g\_0\/U\)], "Input"],

Cell[BoxData[
    \(\[Tau] = \(2  \[Pi]\)\/\[Omega]\)], "Input"],

Cell["Wave number and wavelength", "Text"],

Cell[BoxData[
    \(k = \[Omega]\^2\/g\_0\)], "Input"],

Cell[BoxData[
    \(\[Lambda] = \(2  \[Pi]\)\/k\)], "Input"],

Cell["Phase velocity", "Text"],

Cell[BoxData[
    \(c = \[Omega]\/k\)], "Input"],

Cell["Dimensionless amplitude", "Text"],

Cell[BoxData[
    \(k\ a\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.8.1 Sloshing", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Paremeters", "Text"],

Cell[BoxData[{
    \(\(a = 1. \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[Lambda] = 100. \ Meter;\)\)}], "Input"],

Cell["Mass flux per unit of crest length", "Text"],

Cell[BoxData[
    \(QL = \ a\ \[Lambda]\/\[Pi] // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.8.2 Momentum flux", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(L = 50\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(a = 10\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Force", "Text"],

Cell[BoxData[
    \(F\_x = \(1\/4\) \(\[Rho]\_0\) \(g\_0\) L\ a\^2 // 
        ToUnit[Newton]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["25.8.3 Wave power", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Lambda] = 500\ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(c\_g = 200\ Meter/Second;\)\), "\[IndentingNewLine]", 
    \(\(a = 1  Meter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1000\ Kilogram/Meter\^3;\)\)}], "Input"],

Cell["Power per unit of crest length", "Text"],

Cell[BoxData[
    \(PL = \(1\/2\) \(\[Rho]\_0\) \(g\_0\) a\^2\ c\_g // 
        ToUnit[Watt]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["25.7 Square jar", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Rho]\_1 = 1\ Gram/Centimeter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_2 = 0.8\ Gram/Centimeter\^3;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 0.3\ Newton/Meter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Critical size", "Text"],

Cell[BoxData[
    \(L = \[Pi] \@\( \[Alpha]\/\(\((\[Rho]\_1 - \[Rho]\_2)\) g\_0\)\) // 
        ToUnit[Centimeter]\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{836, 826},
WindowMargins->{{56, Automatic}, {Automatic, 19}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 62, 1, 95, "Title",
  InitializationCell->True],
Cell[1841, 56, 112, 3, 50, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1990, 64, 24, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[2039, 68, 32, 0, 73, "Section"],
Cell[2074, 70, 51, 1, 30, "Input"],
Cell[2128, 73, 36, 0, 33, "Text"],
Cell[2167, 75, 129, 2, 50, "Input"],
Cell[2299, 79, 30, 0, 33, "Text"],
Cell[2332, 81, 92, 1, 42, "Input"],
Cell[2427, 84, 215, 4, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2679, 93, 39, 0, 73, "Section"],
Cell[2721, 95, 51, 1, 30, "Input"],
Cell[2775, 98, 263, 4, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3075, 107, 35, 0, 73, "Section"],
Cell[3113, 109, 51, 1, 30, "Input"],
Cell[3167, 112, 36, 0, 33, "Text"],
Cell[3206, 114, 149, 3, 70, "Input"],
Cell[3358, 119, 34, 0, 33, "Text"],
Cell[3395, 121, 75, 1, 32, "Input"],
Cell[3473, 124, 30, 0, 33, "Text"],
Cell[3506, 126, 53, 1, 40, "Input"],
Cell[3562, 129, 30, 0, 33, "Text"],
Cell[3595, 131, 68, 1, 30, "Input"],
Cell[3666, 134, 27, 0, 33, "Text"],
Cell[3696, 136, 65, 1, 42, "Input"],
Cell[3764, 139, 349, 7, 70, "Input"],
Cell[4116, 148, 64, 1, 30, "Input"],
Cell[4183, 151, 39, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4259, 157, 32, 0, 73, "Section"],
Cell[4294, 159, 51, 1, 30, "Input"],
Cell[4348, 162, 119, 4, 33, "Text"],
Cell[4470, 168, 65, 1, 32, "Input"],
Cell[4538, 171, 53, 1, 40, "Input"],
Cell[4594, 174, 68, 1, 30, "Input"],
Cell[4665, 177, 50, 1, 44, "Input"],
Cell[4718, 180, 388, 7, 91, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5155, 193, 45, 0, 95, "Title"],
Cell[5203, 195, 46, 1, 30, "Input"],
Cell[5252, 198, 352, 7, 101, "Input"],
Cell[5607, 207, 153, 2, 42, "Input"],
Cell[5763, 211, 85, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5885, 217, 32, 0, 95, "Title"],
Cell[5920, 219, 51, 1, 30, "Input"],
Cell[5974, 222, 109, 2, 50, "Input"],
Cell[6086, 226, 133, 2, 51, "Input"],
Cell[6222, 230, 161, 2, 41, "Input"],
Cell[6386, 234, 72, 1, 40, "Input"],

Cell[CellGroupData[{
Cell[6483, 239, 29, 0, 73, "Section"],
Cell[6515, 241, 51, 1, 30, "Input"],
Cell[6569, 244, 72, 1, 33, "Input"],
Cell[6644, 247, 53, 1, 40, "Input"],
Cell[6700, 250, 68, 1, 30, "Input"],
Cell[6771, 253, 61, 1, 30, "Input"],
Cell[6835, 256, 64, 1, 30, "Input"],
Cell[6902, 259, 86, 1, 31, "Input"],
Cell[6991, 262, 41, 1, 30, "Input"],
Cell[7035, 265, 37, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7109, 271, 23, 0, 73, "Section"],
Cell[7135, 273, 50, 1, 44, "Input"],
Cell[7188, 276, 402, 8, 91, "Input"],
Cell[7593, 286, 216, 3, 70, "Input"],
Cell[7812, 291, 121, 2, 51, "Input"],
Cell[7936, 295, 84, 1, 30, "Input"],
Cell[8023, 298, 81, 1, 32, "Input"],
Cell[8107, 301, 62, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8218, 308, 36, 0, 95, "Title"],
Cell[8257, 310, 46, 1, 30, "Input"],
Cell[8306, 313, 48, 0, 33, "Text"],
Cell[8357, 315, 196, 4, 90, "Input"],
Cell[8556, 321, 34, 0, 33, "Text"],
Cell[8593, 323, 78, 1, 51, "Input"],
Cell[8674, 326, 36, 0, 33, "Text"],
Cell[8713, 328, 62, 1, 30, "Input"],
Cell[8778, 331, 34, 0, 33, "Text"],
Cell[8815, 333, 93, 1, 33, "Input"],
Cell[8911, 336, 30, 0, 33, "Text"],
Cell[8944, 338, 53, 1, 40, "Input"],
Cell[9000, 341, 30, 0, 33, "Text"],
Cell[9033, 343, 68, 1, 30, "Input"],
Cell[9104, 346, 34, 0, 33, "Text"],
Cell[9141, 348, 70, 1, 42, "Input"],
Cell[9214, 351, 27, 0, 33, "Text"],
Cell[9244, 353, 65, 1, 42, "Input"],
Cell[9312, 356, 20, 0, 33, "Text"],
Cell[9335, 358, 582, 9, 110, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9954, 372, 31, 0, 95, "Title"],
Cell[9988, 374, 51, 1, 30, "Input"],
Cell[10042, 377, 44, 0, 33, "Text"],
Cell[10089, 379, 289, 4, 90, "Input"],
Cell[10381, 385, 125, 2, 51, "Input"],
Cell[10509, 389, 62, 1, 30, "Input"],
Cell[10574, 392, 126, 2, 50, "Input"],
Cell[10703, 396, 451, 8, 94, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11191, 409, 30, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[11246, 413, 45, 0, 73, "Section"],
Cell[11294, 415, 51, 1, 30, "Input"],
Cell[11348, 418, 24, 0, 33, "Text"],
Cell[11375, 420, 151, 2, 46, "Input"],
Cell[11529, 424, 44, 0, 33, "Text"],
Cell[11576, 426, 198, 3, 70, "Input"],
Cell[11777, 431, 32, 0, 33, "Text"],
Cell[11812, 433, 470, 10, 93, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12319, 448, 24, 0, 73, "Section"],
Cell[12346, 450, 46, 1, 30, "Input"],
Cell[12395, 453, 32, 0, 33, "Text"],
Cell[12430, 455, 53, 1, 30, "Input"],
Cell[12486, 458, 66, 1, 30, "Input"],
Cell[12555, 461, 96, 2, 33, "Text"],
Cell[12654, 465, 114, 2, 30, "Input"],
Cell[12771, 469, 39, 0, 33, "Text"],
Cell[12813, 471, 89, 1, 30, "Input"],
Cell[12905, 474, 106, 2, 30, "Input"],
Cell[13014, 478, 87, 1, 30, "Input"],
Cell[13104, 481, 84, 1, 30, "Input"],
Cell[13191, 484, 54, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13294, 491, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[13344, 495, 33, 0, 73, "Section"],
Cell[13380, 497, 51, 1, 30, "Input"],
Cell[13434, 500, 26, 0, 33, "Text"],
Cell[13463, 502, 326, 5, 110, "Input"],
Cell[13792, 509, 44, 0, 33, "Text"],
Cell[13839, 511, 81, 1, 31, "Input"],
Cell[13923, 514, 63, 1, 30, "Input"],
Cell[13989, 517, 22, 0, 33, "Text"],
Cell[14014, 519, 68, 1, 44, "Input"],
Cell[14085, 522, 57, 1, 30, "Input"],
Cell[14145, 525, 49, 0, 33, "Text"],
Cell[14197, 527, 120, 2, 42, "Input"],
Cell[14320, 531, 47, 0, 33, "Text"],
Cell[14370, 533, 111, 2, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14518, 540, 71, 0, 73, "Section"],
Cell[14592, 542, 46, 1, 30, "Input"],
Cell[14641, 545, 26, 0, 33, "Text"],
Cell[14670, 547, 282, 4, 90, "Input"],
Cell[14955, 553, 125, 2, 51, "Input"],
Cell[15083, 557, 62, 1, 30, "Input"],
Cell[15148, 560, 49, 1, 44, "Input"],
Cell[15200, 563, 152, 2, 51, "Input"],
Cell[15355, 567, 72, 1, 42, "Input"],
Cell[15430, 570, 77, 1, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15544, 576, 37, 0, 73, "Section"],
Cell[15584, 578, 51, 1, 30, "Input"],
Cell[15638, 581, 26, 0, 33, "Text"],
Cell[15667, 583, 203, 3, 70, "Input"],
Cell[15873, 588, 38, 0, 33, "Text"],
Cell[15914, 590, 85, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16036, 596, 45, 0, 73, "Section"],
Cell[16084, 598, 46, 1, 30, "Input"],
Cell[16133, 601, 237, 4, 90, "Input"],
Cell[16373, 607, 109, 2, 51, "Input"],
Cell[16485, 611, 84, 1, 30, "Input"],
Cell[16572, 614, 112, 2, 33, "Input"],
Cell[16687, 618, 53, 1, 40, "Input"],
Cell[16743, 621, 68, 1, 30, "Input"],
Cell[16814, 624, 65, 1, 42, "Input"],
Cell[16882, 627, 70, 1, 42, "Input"],
Cell[16955, 630, 76, 1, 30, "Input"],
Cell[17034, 633, 80, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17151, 639, 46, 0, 73, "Section"],
Cell[17200, 641, 51, 1, 30, "Input"],
Cell[17254, 644, 26, 0, 33, "Text"],
Cell[17283, 646, 248, 4, 90, "Input"],
Cell[17534, 652, 43, 0, 33, "Text"],
Cell[17580, 654, 76, 1, 51, "Input"],
Cell[17659, 657, 76, 1, 43, "Input"],
Cell[17738, 660, 29, 0, 33, "Text"],
Cell[17770, 662, 57, 1, 47, "Input"],
Cell[17830, 665, 36, 0, 33, "Text"],
Cell[17869, 667, 61, 1, 40, "Input"],
Cell[17933, 670, 64, 1, 42, "Input"],
Cell[18000, 673, 42, 0, 33, "Text"],
Cell[18045, 675, 54, 1, 47, "Input"],
Cell[18102, 678, 60, 1, 42, "Input"],
Cell[18165, 681, 30, 0, 33, "Text"],
Cell[18198, 683, 48, 1, 40, "Input"],
Cell[18249, 686, 39, 0, 33, "Text"],
Cell[18291, 688, 37, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18365, 694, 34, 0, 73, "Section"],
Cell[18402, 696, 46, 1, 30, "Input"],
Cell[18451, 699, 26, 0, 33, "Text"],
Cell[18480, 701, 117, 2, 50, "Input"],
Cell[18600, 705, 50, 0, 33, "Text"],
Cell[18653, 707, 67, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18757, 713, 39, 0, 73, "Section"],
Cell[18799, 715, 46, 1, 30, "Input"],
Cell[18848, 718, 26, 0, 33, "Text"],
Cell[18877, 720, 259, 4, 90, "Input"],
Cell[19139, 726, 21, 0, 33, "Text"],
Cell[19163, 728, 106, 2, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19306, 735, 36, 0, 73, "Section"],
Cell[19345, 737, 46, 1, 30, "Input"],
Cell[19394, 740, 26, 0, 33, "Text"],
Cell[19423, 742, 324, 5, 110, "Input"],
Cell[19750, 749, 46, 0, 33, "Text"],
Cell[19799, 751, 105, 2, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[19953, 759, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[20003, 763, 34, 0, 73, "Section"],
Cell[20040, 765, 46, 1, 30, "Input"],
Cell[20089, 768, 26, 0, 33, "Text"],
Cell[20118, 770, 284, 4, 90, "Input"],
Cell[20405, 776, 29, 0, 33, "Text"],
Cell[20437, 778, 127, 2, 51, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

