(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     29524,       1008]*)
(*NotebookOutlinePosition[     30197,       1031]*)
(*  CellTagsIndexPosition[     30153,       1027]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["26. Whirls and vortices", "Title",
  InitializationCell->True],

Cell[BoxData[{
    \(<< Lautrup`\), "\n", 
    \(<< Graphics`\), "\n", 
    \(<< Calculus`VectorAnalysis`\), "\[IndentingNewLine]", 
    \(Off[NIntegrate::nintp]\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rankine and Lamb vortices", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Azimuthal velocity", "Section"],

Cell["Choice of units", "Text"],

Cell[BoxData[{
    \(\(\[CapitalOmega] = 1;\)\), "\[IndentingNewLine]", 
    \(\(c = 1;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1;\)\)}], "Input"],

Cell["Rankine vortex", "Text"],

Cell[BoxData[
    \(\(v\_R = 
        If[r \[LessEqual] \ 
            c, \[CapitalOmega]\ r, \(\[CapitalOmega]\ c\^2\)\/r];\)\)], \
"Input"],

Cell["Lamb vortex", "Text"],

Cell[BoxData[
    \(\(v\_L = \(\(\[CapitalOmega]\ c\^2\)\/r\) \((1 - 
              Exp[\(-\(r\^2\/c\^2\)\)])\);\)\)], "Input"],

Cell["Plot of both", "Text"],

Cell[BoxData[
    \(\(Plot[{v\_R, v\_L}, {r, 0, 3}, 
          AxesLabel -> {"\<r\>", \*"\"\<\!\(v\_\[Phi]\)\>\""}, 
          PlotStyle -> {{Dashing[{0.02, 0.02}]}, {}}, 
          PlotLabel -> "\<(a)\>"] // \[IndentingNewLine]Figure["\<whirls1\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pressure", "Section"],

Cell["Rankine pressure", "Text"],

Cell[BoxData[
    \(\(p\_R = \(-\[Rho]\_0\) 
          If[r \[LessEqual] 
              c, \(\[CapitalOmega]\^\(\(2\)\(\ \)\)\) 
                c\^2 - \(\(\(\[CapitalOmega]\^2\)\^\ \) r\^2\)\/2, \(\(\
\[CapitalOmega]\^2\) c\^4\)\/\(2  r\^2\)];\)\)], "Input"],

Cell["Standard function", "Text"],

Cell[BoxData[
    \(F[\[Xi]_] = 
      Integrate[\((\(1 - \[ExponentialE]\^\(-x\)\)\/x)\)\^2, {x, \[Xi], \
\[Infinity]}, Assumptions \[Rule] \[Xi] > 1]\)], "Input"],

Cell["Lamb pressure", "Text"],

Cell[BoxData[
    \(\(p\_L = \(-\(1\/2\)\) \(\[Rho]\_0\) \(\[CapitalOmega]\^2\) \(c\^2\) 
          F[r\^2\/c\^2];\)\)], "Input"],

Cell["Plot both", "Text"],

Cell[BoxData[
    \(\(Plot[{p\_R, p\_L}, {r, 0, 3}, 
          PlotStyle \[Rule] {Dashing[{0.02, 0.02}], Dashing[{}]}, 
          AxesLabel \[Rule] {"\<r\>", "\<p\>"}, 
          PlotLabel -> "\<(b)\>"] \
// \[IndentingNewLine]Figure["\<whirls1b\>", 
          ImageSize \[Rule] 250];\)\)], "Input"],

Cell["Value at \[Xi]=0", "Text"],

Cell[BoxData[
    \(Limit[F[\[Xi]], \[Xi] \[Rule] 0]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Stress", "Section"],

Cell["\<\
Rankine stress (must be multiplied with viscosity \[Eta])\
\>", "Text"],

Cell[BoxData[
    \(\(\[Sigma]\_R = 
        If[r < c, 0, \(-2\)\ \[CapitalOmega]  c\^2\/r\^2];\)\)], "Input"],

Cell["Lamb stress", "Text"],

Cell[BoxData[
    \(\(\[Sigma]\_L = \[PartialD]\_r v\_L - v\_L\/r;\)\)], "Input"],

Cell["Plot both", "Text"],

Cell[BoxData[
    \(\(Plot[{\[Sigma]\_R, \[Sigma]\_L}, {r, 0, 3}, 
        PlotRange \[Rule] All];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TheOseen-Lamb vortex ", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[CapitalOmega] = 1;\)\), "\[IndentingNewLine]", 
    \(\(c = 1;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 0.1;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1;\)\), "\[IndentingNewLine]", 
    \(\(\[Epsilon] = 10\^\(-3\);\)\), "\[IndentingNewLine]", 
    \(\(A = 100;\)\)}], "Input"],

Cell["Vortex shape", "Text"],

Cell[BoxData[
    \(\(v[r_?NumericQ, 
          t_?NumericQ] = \(\(\[CapitalOmega]\ c\^2\)\/r\) \((1 - 
              Exp[\(-\(r\^2\/\(c\^2 + 4  \[Nu]\ t\)\)\)])\);\)\)], "Input"],

Cell["Choice of times", "Text"],

Cell[BoxData[{
    \(\(ts = {0, 1, 2, 4, 8, 16};\)\), "\[IndentingNewLine]", 
    \(\(vs = Map[v[r, #] &, ts];\)\)}], "Input"],

Cell["Plot of vortex evolution", "Text"],

Cell[BoxData[
    \(\(Plot[vs // Evaluate, {r, 0, 3}, 
          AxesLabel -> {"\<r\>", \*"\"\<\!\(v\_\[Phi]\)\>\""}, 
          PlotRange \[Rule] {0, 0.7}, PlotLabel -> "\<(a)\>", 
          PlotStyle \[Rule] {Text["\<t=0\>", {1, 0.63}, {0, \(-1\)}], 
              Text["\<1\>", {1.2, 0.54}, {0, \(-1\)}], 
              Text["\<2\>", {1.5, 0.47}, {0, \(-1\)}], 
              Text["\<4\>", {1.7, 0.39}, {0, \(-1\)}], 
              Text["\<8\>", {2, 0.3}, {0, \(-1\)}], 
              Text["\<16\>", {2.5, 
                  0.22}, {0, \(-1\)}]}] \
// \[IndentingNewLine]Figure["\<whirls5\>", 
          ImageSize \[Rule] 250];\)\)], "Input"],

Cell["Lamb pressure function", "Text"],

Cell[BoxData[
    \(F[\[Xi]_] = 
      Integrate[\((\(1 - \[ExponentialE]\^\(-x\)\)\/x)\)\^2, {x, \[Xi], \
\[Infinity]}, Assumptions \[Rule] \[Xi] > 1]\)], "Input"],

Cell["Pressure", "Text"],

Cell[BoxData[
    \(\(p = \(-\(1\/2\)\) \(\[Rho]\_0\) \(\[CapitalOmega]\^2\) \(c\^4\/\(c\^2 \
+ 4  \[Nu]\ t\)\) F[r\^2\/\(c\^2 + 4  \[Nu]\ t\)];\)\)], "Input"],

Cell["Selected times", "Text"],

Cell[BoxData[
    \(\(ps = Map[p /. t \[Rule] # &, ts];\)\)], "Input"],

Cell["Plot all", "Text"],

Cell[BoxData[
    \(\(Plot[ps // Evaluate, {r, \[Epsilon], 3}, 
          AxesLabel -> {"\<r\>", "\<p\>"}, PlotRange \[Rule] {\(-0.75\), 0}, 
          PlotLabel -> "\<(b)\>", 
          PlotStyle \[Rule] {Text["\<t=0\>", {0.4, \(-0.63\)}, {1, \(-1\)}], 
              Text["\<1\>", {0.3, \(-0.47\)}, {1, \(-1\)}], 
              Text["\<2\>", {0.3, \(-0.37\)}, {1, \(-1\)}], 
              Text["\<4\>", {0.3, \(-0.26\)}, {1, \(-1\)}], 
              Text["\<8\>", {0.3, \(-0.16\)}, {1, \(-1\)}], 
              Text["\<16\>", {0.3, \(-0.09\)}, {1, \(-1\)}]}] \
// \[IndentingNewLine]Figure["\<whirls6\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Line vortices", "Title"],

Cell[CellGroupData[{

Cell["Opposite sense of rotation", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Complex velocity field", "Text"],

Cell[BoxData[
    \(w = \(-\ \(\[ImaginaryI]\/\(z - \[ImaginaryI]\ b\)\)\) + \ \
\[ImaginaryI]\/\(z + \[ImaginaryI]\ b\) // Simplify\)], "Input"],

Cell["Integral", "Text"],

Cell[BoxData[
    \(\[CapitalPsi]\  = \[Integral]\ w\ \[DifferentialD]z\)], "Input"],

Cell["Choice of distance", "Text"],

Cell[BoxData[
    \(\(b = 1;\)\)], "Input"],

Cell["Stream function (its contours are streamlines)", "Text"],

Cell[BoxData[
    \(\(\[Psi][x_?NumericQ, y_?NumericQ] = 
        Im[\[CapitalPsi] /. z \[Rule] x + \[ImaginaryI]\ y];\)\)], "Input"],

Cell[BoxData[
    \(\(ContourPlot[\[Psi][x, y], {x, \(-4\), 4}, {y, \(-4\), 4}, 
          AspectRatio \[Rule] 1, PlotPoints \[Rule] 100, Contours \[Rule] 21, 
          ContourShading \[Rule] False, PlotLabel -> "\<(a)\>", 
          FrameTicks \[Rule] 
            None] // \[IndentingNewLine]Figure["\<whirls13\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Same sense of rotation", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Complex velocity field", "Text"],

Cell[BoxData[
    \(w = \(-\ \(\[ImaginaryI]\/\(z - \[ImaginaryI]\ b\)\)\) - \ \
\[ImaginaryI]\/\(z + \[ImaginaryI]\ b\) // Simplify\)], "Input"],

Cell["Integral", "Text"],

Cell[BoxData[
    \(\[CapitalPsi]\  = \[Integral]\ w\ \[DifferentialD]z\)], "Input"],

Cell[BoxData[
    \(\(b = 1;\)\)], "Input"],

Cell["Stream function", "Text"],

Cell[BoxData[
    \(\(\[Psi][x_?NumericQ, y_?NumericQ] = 
        Im[\[CapitalPsi] /. z \[Rule] x + \[ImaginaryI]\ y];\)\)], "Input"],

Cell[BoxData[
    \(\(ContourPlot[\[Psi][x, y], {x, \(-4\), 4}, {y, \(-4\), 4}, 
          AspectRatio \[Rule] 1, PlotPoints \[Rule] 250, Contours \[Rule] 11, 
          ContourShading \[Rule] False, PlotLabel -> "\<(a)\>", 
          FrameTicks \[Rule] 
            None] // \[IndentingNewLine]Figure["\<whirls14\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["von Karmans vortex street", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[TextData[{
  "Complex velocity field for ",
  Cell[BoxData[
      \(TraditionalForm\`a = \[Pi]\/2\)]]
}], "Text"],

Cell[BoxData[
    \(\(w = \[ImaginaryI]\ Cot[z + \[ImaginaryI]\ b] + \[ImaginaryI]\ Tan[
              z - \[ImaginaryI]\ b];\)\)], "Input"],

Cell["Integral", "Text"],

Cell[BoxData[
    \(\[CapitalPsi] = \[Integral]\ w\ \[DifferentialD]z\)], "Input"],

Cell["Critical distance", "Text"],

Cell[BoxData[
    \(b = 0.5\ ArcSinh[1]\)], "Input"],

Cell["Stream function", "Text"],

Cell[BoxData[
    \(\(\[Psi][x_?NumericQ, y_?NumericQ] = 
        Im[\[CapitalPsi] /. z \[Rule] x + \[ImaginaryI]\ y];\)\)], "Input"],

Cell["Choice of interval", "Text"],

Cell[BoxData[{
    \(\(\[Lambda] = 3.8;\)\), "\n", 
    \(\(ContourPlot[\[Psi][x, y], {x, \[Pi]/4, 
            3  \[Pi] + \[Pi]/4}, {y, \(-\[Lambda]\)\ b, \[Lambda]\ b}, 
          AspectRatio \[Rule] \(2  \[Lambda]\ b\)\/\(3  \[Pi]\), 
          ContourShading \[Rule] False, ImageSize \[Rule] 800, 
          PlotPoints \[Rule] 200, Contours \[Rule] 19, 
          FrameTicks \[Rule] False, Frame \[Rule] False, 
          PlotRange \[Rule] {\(-\[Lambda]\), \[Lambda]}] \
// \[IndentingNewLine]Figure["\<whirls15\>"];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Magical value", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\[Lambda] = ArcSinh[1] // N\)], "Input"],

Cell[BoxData[
    \(\[Lambda]\/\[Pi]\)], "Input"],

Cell[BoxData[
    \(\(\[Lambda]\/\[Pi]\) \((1 - 
          0.5  \[Lambda]\ Tanh[\[Lambda]])\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Vortex with localized axial jet", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Generalized form of the field", "Text"],

Cell[BoxData[{
    \(\(v\_\[Phi] = 
        CC\/r + A\ r\^\(1 - 2  \[Alpha]\);\)\), "\[IndentingNewLine]", 
    \(\(v\_r = \(-2\) \[Alpha]\ \[Nu]\/r;\)\), "\[IndentingNewLine]", 
    \(\(v\_z = B\ r\^\(\(-2\) \[Alpha]\);\)\), "\[IndentingNewLine]", 
    \(\(p = \(-\(\(A\^2\ r\^\(2 - 4\ \[Alpha]\)\)\/\(2\ \((\(-1\) + 
                      2\ \[Alpha])\)\)\)\) - \(\(A\ CC\)\/\[Alpha]\) 
            r\^\(\(-2\) \[Alpha]\) - \(CC\^2 + 4\ \[Alpha]\^2\ \
\[Nu]\^2\)\/\(2  r\^2\);\)\)}], "Input"],

Cell["Cylindrical Navier-Stokes equations (must all vanish)", "Text"],

Cell[BoxData[
    \(\[PartialD]\_t v\_\[Phi] + \(v\_r\) \[PartialD]\_r v\_\[Phi] + \(\(v\_r\
\) v\_\[Phi]\)\/r + \(v\_z\) \[PartialD]\_z v\_\[Phi] - \[Nu] \
\((\[PartialD]\_r\^2 v\_\[Phi] + \(1\/r\) \[PartialD]\_r v\_\[Phi] - 
              v\_\[Phi]\/r\^2 + \[PartialD]\_z\^2 v\_\[Phi])\) // 
      Expand\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_t v\_r + \(v\_r\) \[PartialD]\_r v\_r - 
        v\_\[Phi]\^2\/r + \(v\_z\) \[PartialD]\_z v\_r - \[Nu] \((\[PartialD]\
\_r\^2 v\_r + \(1\/r\) \[PartialD]\_r v\_r - 
              v\_r\/r\^2 + \[PartialD]\_z\^2 v\_r)\) + \[PartialD]\_r p // 
      Simplify\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_t v\_z + \(v\_r\) \[PartialD]\_r v\_z + \(v\_z\) \
\[PartialD]\_z v\_z - \[Nu] \((\[PartialD]\_r\^2 v\_z + \(1\/r\) \
\[PartialD]\_r v\_z + \[PartialD]\_z\^2 v\_z)\) + \[PartialD]\_z p // 
      Expand\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_r v\_r + v\_r\/r + \[PartialD]\_z v\_z // 
      Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Vortex with extended axial jet", "Title"],

Cell[CellGroupData[{

Cell["Navier-Stokes equations", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Field ansatz", "Text"],

Cell[BoxData[
    \(\(u[
          r_] = \(-2\)\ \(a\^2\/c\^2\) \((1 - \
\[ExponentialE]\^\(\(-r\^2\)/a\^2\))\);\)\)], "Input"],

Cell[BoxData[{
    \(\(v\_\[Phi] = v[r]\/r;\)\), "\[IndentingNewLine]", 
    \(\(v\_r = \[Nu]\ u[r]\/r;\)\), "\[IndentingNewLine]", 
    \(\(v\_z = \(-\(z\/r\)\) \[PartialD]\_r\((r\ v\_r)\);\)\), "\
\[IndentingNewLine]", 
    \(\(p = 
        q[r]\  - 
          8\ \(\((a\^2 + c\^2)\)\/\(\(a\^2\) 
                  c\^4\)\) \[Nu]\^2\ z\^2\ \((1 + \ 
                r\^2\/a\^2)\) \[ExponentialE]\^\(\(-r\^2\)/a\^2\);\)\), "\
\[IndentingNewLine]", 
    \(\(v[
          r_] = \ \[Integral]Exp[\[Integral]\ \(u[
                        r]\/r\) \[DifferentialD]r]\ r\ \
\[DifferentialD]r;\)\), "\[IndentingNewLine]", 
    \(\(q[
          r_] = \(\[Nu]\^2\) \((\(u'\)[r]\/r - \(1\/2\) 
                  u[r]\^2\/r\^2)\) + \[Integral]\(v[r]\^2\/r\^3\) \
\[DifferentialD]r;\)\)}], "Input"],

Cell["Cylindrical Navier-Stokes equations (must all vanish)", "Text"],

Cell[BoxData[{
    \(\(zero1 = \[PartialD]\_t v\_\[Phi] + \(v\_r\) \[PartialD]\_r v\_\[Phi] \
+ \(\(v\_r\) v\_\[Phi]\)\/r + \(v\_z\) \[PartialD]\_z v\_\[Phi] - \[Nu] \((\
\[PartialD]\_r\^2 v\_\[Phi] + \(1\/r\) \[PartialD]\_r v\_\[Phi] - 
                v\_\[Phi]\/r\^2 + \[PartialD]\_z\^2 v\_\[Phi])\);\)\), "\
\[IndentingNewLine]", 
    \(\(zero2 = \[PartialD]\_t v\_r + \(v\_r\) \[PartialD]\_r v\_r - 
          v\_\[Phi]\^2\/r + \(v\_z\) \[PartialD]\_z v\_r - \[Nu] \((\
\[PartialD]\_r\^2 v\_r + \(1\/r\) \[PartialD]\_r v\_r - 
                v\_r\/r\^2 + \[PartialD]\_z\^2 v\_r)\) + \[PartialD]\_r 
            p;\)\), "\[IndentingNewLine]", 
    \(\(zero3 = \[PartialD]\_t v\_z + \(v\_r\) \[PartialD]\_r v\_z + \(v\_z\) \
\[PartialD]\_z v\_z - \[Nu] \((\[PartialD]\_r\^2 v\_z + \(1\/r\) \
\[PartialD]\_r v\_z + \[PartialD]\_z\^2 v\_z)\) + \[PartialD]\_z 
            p;\)\), "\[IndentingNewLine]", 
    \(\(zero4 = \[PartialD]\_r v\_r + 
          v\_r\/r + \[PartialD]\_z v\_z;\)\)}], "Input"],

Cell["Check if fields satisfy (those that don't are small)", "Text"],

Cell[BoxData[
    \(zero1\  // Expand\)], "Input"],

Cell[BoxData[
    \(zero2 // Simplify\)], "Input"],

Cell[BoxData[
    \(zero3a = zero3 // Simplify\)], "Input"],

Cell[BoxData[
    \(zero4 // Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Velocity field", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Choice of parameters", "Text"],

Cell[BoxData[{
    \(\(a = 1;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 0.01;\)\), "\[IndentingNewLine]", 
    \(\(\[Epsilon] = 10\^\(-3\);\)\), "\[IndentingNewLine]", 
    \(\(A = 200;\)\), "\[IndentingNewLine]", 
    \(\(CC = 1;\)\)}], "Input"],

Cell["Fields", "Text"],

Cell[BoxData[{
    \(\(v\_r = \(-\[Nu]\)\ \(\(\(2\)  \(\[Alpha]\)\(\ \)\)\/r\) \((1 - \
\[ExponentialE]\^\(\(-r\^2\)/a\^2\))\);\)\), "\[IndentingNewLine]", 
    \(\(v\_\[Phi] = v[r];\)\)}], "Input"],

Cell["Azimuthal Navier-Stokes", "Text"],

Cell[BoxData[
    \(zero1[
        r_, \[Alpha]_] = \(v\_r\/r\) \[PartialD]\_r\((r\ v\_\[Phi])\) - \[Nu]\
\ \[PartialD]\_r\((\(1\/r\) \[PartialD]\_r\((r\ v\_\[Phi])\))\)\)], "Input"],

Cell["Solve for azimuthal field", "Text"],

Cell[BoxData[
    \(sol[\[Alpha]_] := 
      NDSolve[{zero1[r, \[Alpha]] \[Equal] 0, 
            v[\[Epsilon]] \[Equal] \ \[Epsilon], \(v'\)[\[Epsilon]] \[Equal] 
              1}, v, {r, \[Epsilon], A}, AccuracyGoal \[Rule] 9, 
          PrecisionGoal \[Rule] 9] // First\)], "Input"],

Cell["Choice of \[Alpha]'s", "Text"],

Cell[BoxData[
    \(\(\[Alpha]s = {2, 4, 8, 16};\)\)], "Input"],

Cell["Corresponding normalized velocity fields", "Text"],

Cell[BoxData[
    \(\(vs = 
        Map[CC \(\(\ \)\(v[r]\)\)\/\(A\ v[A]\) /. 
              sol[#] &, \[Alpha]s];\)\)], "Input"],

Cell["Plot them all", "Text"],

Cell[BoxData[
    \(\(Plot[vs // Evaluate, {r, \[Epsilon], 1.5}, 
          AxesLabel \[Rule] {"\<r\>", \*"\"\<\!\(v\_\[Phi]\)\>\""}, 
          PlotRange \[Rule] {0, 2.8}, 
          PlotStyle \[Rule] {Text["\<\[Alpha]=2\>", {0.8, 
                  0.62}, {0, \(-1\)}], 
              Text["\<4\>", {0.53, 1.1}, {0, \(-1\)}], 
              Text["\<8\>", {0.35, 1.7}, {0, \(-1\)}], 
              Text["\<16\>", {0.27, 2.5}, {0, \(-1\)}]}, 
          PlotLabel \[Rule] "\<(a)\>"] \
// \[IndentingNewLine]Figure["\<whirls2\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pressure", "Section"],

Cell["Pressure derivative", "Text"],

Cell[BoxData[
    \(\(qp = \(-v\_r\) \[PartialD]\_r v\_r + 
            v\_\[Phi]\^2\/r + \[Nu]\ \[PartialD]\_r\((\(1\/r\) \[PartialD]\_r\
\((r\ v\_r)\))\) // Simplify;\)\)], "Input"],

Cell["Collection of pressure derivatives", "Text"],

Cell[BoxData[
    \(\(qps = 
        MapThread[\(qp /. \[Alpha] \[Rule] #1\) /. 
              v[r_] \[Rule] #2 &, {\[Alpha]s, vs}];\)\)], "Input"],

Cell["Show shape of pressure derivatives", "Text"],

Cell[BoxData[
    \(\(Plot[qps // Evaluate, {r, \[Epsilon], 3}];\)\)], "Input"],

Cell["Solve for normalized pressure", "Text"],

Cell[BoxData[
    \(\(qs = 
        Map[q[r] /. \(NDSolve[{\(q'\)[r] \[Equal] #, q[A] \[Equal] 0}, 
                  q, {r, \[Epsilon], A}]\)[\([1]\)] &, qps];\)\)], "Input"],

Cell["Plot pressures", "Text"],

Cell[BoxData[
    \(\(Plot[qs // Evaluate, {r, \[Epsilon], 1.5}, 
          AxesLabel \[Rule] {"\<r\>", "\<p\>"}, PlotLabel -> "\<(b)\>", 
          PlotStyle \[Rule] 
            Text["\<\[Alpha]=16\>", {0.1, \(-10\)}, {\(-1\), 
                1}]] // \[IndentingNewLine]Figure["\<whirls3\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advective spin-up of a vortex", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(q = 1;\)\), "\[IndentingNewLine]", 
    \(\(b = 8;\)\)}], "Input"],

Cell["Fields", "Text"],

Cell[BoxData[{
    \(\(u[r_] = \(-\(q\/r\)\);\)\), "\n", 
    \(\(\[Tau][r_] = r\^2\/\(2  q\);\)\)}], "Input"],

Cell["Circulation coefficient", "Text"],

Cell[BoxData[
    \(\(CC[r_] = If[b < r < 2\ b, r\/b - 0.6, 0];\)\)], "Input"],

Cell["Choice of times", "Text"],

Cell[BoxData[
    \(\(ts = {0, 25, 50, 75, 100, 125\ };\)\)], "Input"],

Cell["Circulation coefficients", "Text"],

Cell[BoxData[
    \(\(CCs = Map[CC[\@\(r\^2 + 2  q\ #\)] &, ts];\)\)], "Input"],

Cell["Plot of time evolution", "Text"],

Cell[BoxData[
    \(\(Plot[CCs // Evaluate, {r, 0, 2\ b + 1}, PlotRange \[Rule] {0, 1.6}, 
          AxesLabel \[Rule] {"\<r\>", \*"\"\<\!\(rv\_\[Phi]\)\>\""}, 
          PlotStyle \[Rule] {Text["\<t=0\>", {16, 1.5}], 
              Text["\<25\>", {14.2, 1.5}], Text["\<50\>", {12.4, 1.5}], 
              Text["\<75\>", {10.3, 1.5}], Text["\<100\>", {7.4, 1.5}], 
              Text["\<125\>", {2.4, 
                  1.5}]}] // \[IndentingNewLine]Figure["\<whirls4\>"];\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bathtub vortex ", "Title"],

Cell[CellGroupData[{

Cell["Differential equations ", "Section"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Treatment function", "Text"],

Cell[BoxData[
    RowBox[{"do", ":=", 
      RowBox[{\(# /. \[Xi] \[Rule] r\^2\/a\^2 &\), Cell[""]}]}]], "Input"],

Cell["Fields", "Text"],

Cell[BoxData[{
    \(\(w = \(-W\)\ f[\[Xi]] // do;\)\), "\[IndentingNewLine]", 
    \(\(h = L\ g[\[Xi]] // do;\)\), "\[IndentingNewLine]", 
    \(\(v\_\[Phi] = \(C\/r\) u[\[Xi]] // do;\)\), "\[IndentingNewLine]", 
    \(\(v\_r = \ \(-\(W\/2\)\)\ a\^2\/\(r\ h\)\ v[\[Xi]] // do;\)\)}], "Input"],

Cell["Cylindrical Navier-Stokes equations (must all vanish", "Text"],

Cell[BoxData[{
    \(\(zero1 = \(v\_r\/r\) \[PartialD]\_r\((r\ v\_\[Phi])\) - \[Nu]\ \
\[PartialD]\_r\((\(1\/r\) \[PartialD]\_r\((r\ v\_\[Phi])\))\) // 
          do;\)\), "\[IndentingNewLine]", 
    \(\(zero2 = \[PartialD]\_r h - v\_\[Phi]\^2\/\(\(g\_0\) r\) // 
          do;\)\), "\[IndentingNewLine]", 
    \(\(zero3 = 
        w - \(1\/r\) \[PartialD]\_r\((r\ \(v\_r\) h)\) // do;\)\)}], "Input"],

Cell["New treatment function", "Text"],

Cell[BoxData[
    \(do := \(\(# /. r \[Rule] a\ \@\[Xi]\) /. 
            W \[Rule] \[Alpha] \( \(4\)\(\ \)\(L\)\(\ \)\(\[Nu]\)\(\ \
\)\)\/a\^2\) /. C \[Rule] \ a\ \@\(2  g\_0\ L\ \[Beta]\) &\)], "Input"],

Cell["Final differential equations", "Text"],

Cell[BoxData[
    \(zero1a = \(\(-zero1\)\ a\^4/\((4\ C\ r\ \[Nu])\) // do\) // 
        Expand\)], "Input"],

Cell[BoxData[
    \(zero2a = \(zero2\ a\^2/\((2\ L\ r\ )\) // do\) // Expand\)], "Input"],

Cell[BoxData[
    \(zero3a = \(zero3/W // do\) // Expand\)], "Input"],

Cell[TextData[{
  "At this point one may choose a soft drain flow:",
  StyleBox[" f=Exp[-\[Xi]] ", "Input"],
  "and ",
  StyleBox[" v=1-f.", "Input"]
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve differential equations", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(a = 10\^\(-8\);\)\), "\[IndentingNewLine]", 
    \(\(c = 3;\)\), "\[IndentingNewLine]", 
    \(\(b = 10;\)\), "\[IndentingNewLine]", 
    \(\(ss = 0.1;\)\), "\[IndentingNewLine]", 
    \(\(mm = 0.9;\)\), "\[IndentingNewLine]", 
    \(\(kk = 10\^\(-12\);\)\), "\[IndentingNewLine]", 
    \(\(\[Beta]0 = 0.0217;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha]0 = 16;\)\)}], "Input"],

Cell["\<\
Solution of boundary value problem (see description of Lautrup`NDSolveBV)\
\>", "Text"],

Cell[BoxData[
    \(sol[\[Alpha]_, \[Beta]_] := \(NDSolveBV[{\(u'\)[x] \[Equal] 
              v[x], \(v'\)[
                x] \[Equal] \(-\[Alpha]\)\ \(\((1 - \
\[ExponentialE]\^\(-x\))\) v[x]\)\/\(x\ g[x]\), \(g'\)[x] \[Equal] \[Beta] 
                 u[x]\^2\/x\^2, u[a] \[Equal] a\ s, v[a] \[Equal] s, 
            g[a] \[Equal] m, u[b] \[Equal] 1, v[b] \[Equal] k, 
            g[b] \[Equal] 1}, {u, v, g}, {x, a, c, 
            b}, {{s, ss, 1.01  ss}, {m, mm, 1.01  mm}, {k, kk, 1.01  kk}}, 
          AccuracyGoal \[Rule] 10, 
          PrecisionGoal \[Rule] 10]\)[\([1]\)]\)], "Input"],

Cell["Plot solutions", "Text"],

Cell[BoxData[
    \(\(Plot[{u[x], g[x], g[a]} /. sol[\[Alpha]0, \[Beta]0] // Evaluate, {x, 
          a, 2}, PlotRange \[Rule] All];\)\)], "Input"],

Cell["Choice of alphas", "Text"],

Cell[BoxData[
    \(\(as = {2, 4, 8, 16};\)\)], "Input"],

Cell["Get corresponding solutions in one go", "Text"],

Cell[BoxData[
    \(ugs = Map[\(({u, g} /. sol[#, \[Beta]0])\) &, as]\)], "Input"],

Cell["Split them up", "Text"],

Cell[BoxData[{
    \(\(us = ugs[\([All, 1]\)];\)\), "\[IndentingNewLine]", 
    \(\(gs = ugs[\([All, 2]\)];\)\)}], "Input"],

Cell["Plot azimuthal velocity", "Text"],

Cell[BoxData[
    \(\(Plot[Map[#[r\^2]\/r &, us] // Evaluate, {r, \@a, 1.4}, 
          AxesLabel \[Rule] {"\<r/a\>", \*"\"\<\!\(v\_\[Phi]\) a/C\>\""}, 
          PlotLabel -> "\<(a)\>", PlotRange \[Rule] {0, 3.2}, 
          PlotStyle \[Rule] {Text["\<\[Alpha]=2\>", {0.8, 0.6}, {0, 1}], 
              Text["\<\[Alpha]=16\>", {0.3, 
                  2.9}, {\(-1\), \(-1\)}]}] \
// \[IndentingNewLine]Figure["\<whirls11a\>", 
          ImageSize \[Rule] 250];\)\)], "Input"],

Cell["Plot depression", "Text"],

Cell[BoxData[
    \(\(Plot[Map[#[r\^2] &, gs] // Evaluate, {r, \@a, 1}, 
          AxesLabel \[Rule] {"\<r/a\>", "\<h/L\>"}, PlotLabel -> "\<(b)\>", 
          PlotRange \[Rule] {0, 1.05}, 
          PlotStyle \[Rule] {Text["\<\[Alpha]=2\>", {0.2, 
                  0.98}, {0, \(-1\)}], 
              Text["\<\[Alpha]=16\>", {0.13, 
                  0.3}, {\(-1\), \(-1\)}]}] \
// \[IndentingNewLine]Figure["\<whirls11b\>", 
          ImageSize \[Rule] 250];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["26.1.1 Water vortex", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(c = 3.0\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = \(10\^\(-6\)\) Meter\^2/Second;\)\)}], "Input"],

Cell["Time for doubling radius", "Text"],

Cell[BoxData[
    \(t = \(3  c\^2\)\/\(4  \[Nu]\) // ToSI\)], "Input"],

Cell[BoxData[
    \(t // ToUnit[Minute]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.3.1 Piano wire", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(\[Nu] = 1.5*^-5\ Meter\^2/Second;\)\), "\[IndentingNewLine]", 
    \(\(d = 1\ Millimeter;\)\), "\[IndentingNewLine]", 
    \(\(U = 3\ Meter/Second;\)\), "\[IndentingNewLine]", 
    \(\(Sr = 0.2;\)\)}], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Rey = \(U\ d\)\/\[Nu] // ToSI\)], "Input"],

Cell["Frequency", "Text"],

Cell[BoxData[
    \(f = Sr\ U\/d // ToSI\)], "Input"],

Cell["Tacoma case", "Text"],

Cell[BoxData[{
    \(U = 42.0\ Mile/Hour // ToSI\), "\[IndentingNewLine]", 
    \(\(d = 4\ Meter;\)\)}], "Input"],

Cell[BoxData[
    \(f = Sr\ U\/d // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bathtub", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(L = 50\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(a = 2\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(CC = \ 20\ Centimeter\^2/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = \ \ \ \(10\^\(-6\)\) 
          Meter\^2/Second;\)\), "\[IndentingNewLine]", 
    \(\(Q = 1.0\ Liter/Second;\)\)}], "Input"],

Cell["Outflow velocity", "Text"],

Cell[BoxData[
    \(W = Q\/\(\[Pi]\ a\^2\) // ToSI\)], "Input"],

Cell["Height of central depression", "Text"],

Cell[BoxData[
    \(h\_0 = \@\(L\^2 - \(\(CC\^2\) W\)\/\(2 \( g\_0\) \[Nu]\)\) // 
        ToSI\)], "Input"],

Cell["Depth of depression", "Text"],

Cell[BoxData[
    \(L - h\_0 // ToSI\)], "Input"],

Cell["Size of vortex core", "Text"],

Cell[BoxData[
    \(c = \@\(\(2  \[Nu] \((L + h\_0)\)\)\/W\) // 
        ToUnit[Millimeter]\)], "Input"],

Cell["Rotation speed", "Text"],

Cell[BoxData[
    \(\[CapitalOmega] = CC\/c\^2 // ToSI\)], "Input"],

Cell[BoxData[
    \(\[CapitalOmega]\/\(2  \[Pi]\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["26.7 The Burgers vortex", "Section"],

Cell[CellGroupData[{

Cell["Derive the equations", "Subsection"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Cylindrical coordinates", "Text"],

Cell[BoxData[
    \(\(SetCoordinates[Cylindrical[r, \[Phi], z]];\)\)], "Input"],

Cell["Define field", "Text"],

Cell[BoxData[
    \(\(v = {v\_r[r, z], v\_\[Phi][r, z], v\_z[r, z]};\)\)], "Input"],

Cell["Steady-flow Navier-Stokes equations", "Text"],

Cell[BoxData[
    \(NS = \(v . Grad[v] + Grad[p[r, z]] - \[Nu]\ Laplacian[v] // Expand\) // 
        Collect[#, \[Nu]] &\)], "Input"],

Cell["Divergence conditions", "Text"],

Cell[BoxData[
    \(DC = Div[v] // Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Show that Burgers vortex satisfies the NS", "Subsection"],

Cell[BoxData[{
    \(\(v\_\[Phi][r_, 
          z_] = \(\(\[CapitalOmega]\ c\^2\)\/r\) \((1 - \
\[ExponentialE]\^\(\(-r\^2\)/c\^2\))\);\)\), "\[IndentingNewLine]", 
    \(\(v\_r[r_, z_] = \(-\(\(2  \[Nu]\)\/c\^2\)\) 
          r;\)\), "\[IndentingNewLine]", 
    \(\(v\_z[r_, z_] = \(\(4  \[Nu]\)\/c\^2\) z;\)\), "\[IndentingNewLine]", 
    \(\(p[r_, z_] = 
        p\_0 + \(1\/2\) \(\[CapitalOmega]\^2\) \(c\^2\) 
            F[r\^2\/c\^2] - \ \(\(2  \[Nu]\^2\)\/c\^4\) \((r\^2 + 
                4  z\^2)\);\)\), "\[IndentingNewLine]", 
    \(\(\(F'\)[
          x_] = \((\(1 - \[ExponentialE]\^\(-x\)\)\/x)\)\^2;\)\)}], "Input"],

Cell["Check that cylindrical Navier-Stokes equations are fulfilled", "Text"],

Cell[BoxData[
    \(NS // Expand\)], "Input"],

Cell[BoxData[
    \(DC // Expand\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.16 Depth of depression", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\[Integral]\_0\%\[Infinity]\(\((1 - \[ExponentialE]\^\(-s\^2\))\)\^2\/s\
\^3\) \[DifferentialD]s\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{1105, 903},
WindowMargins->{{49, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 68, 1, 95, "Title",
  InitializationCell->True],
Cell[1847, 56, 203, 5, 90, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[2087, 66, 42, 0, 95, "Title"],
Cell[2132, 68, 51, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2208, 73, 37, 0, 73, "Section"],
Cell[2248, 75, 31, 0, 33, "Text"],
Cell[2282, 77, 155, 3, 70, "Input"],
Cell[2440, 82, 30, 0, 33, "Text"],
Cell[2473, 84, 141, 4, 45, "Input"],
Cell[2617, 90, 27, 0, 33, "Text"],
Cell[2647, 92, 127, 2, 45, "Input"],
Cell[2777, 96, 28, 0, 33, "Text"],
Cell[2808, 98, 299, 5, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3144, 108, 27, 0, 73, "Section"],
Cell[3174, 110, 32, 0, 33, "Text"],
Cell[3209, 112, 259, 5, 46, "Input"],
Cell[3471, 119, 33, 0, 33, "Text"],
Cell[3507, 121, 164, 3, 44, "Input"],
Cell[3674, 126, 29, 0, 33, "Text"],
Cell[3706, 128, 129, 2, 45, "Input"],
Cell[3838, 132, 25, 0, 33, "Text"],
Cell[3866, 134, 299, 6, 50, "Input"],
Cell[4168, 142, 32, 0, 33, "Text"],
Cell[4203, 144, 65, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4305, 150, 25, 0, 73, "Section"],
Cell[4333, 152, 81, 2, 33, "Text"],
Cell[4417, 156, 110, 2, 45, "Input"],
Cell[4530, 160, 27, 0, 33, "Text"],
Cell[4560, 162, 81, 1, 40, "Input"],
Cell[4644, 165, 25, 0, 33, "Text"],
Cell[4672, 167, 113, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4834, 175, 38, 0, 95, "Title"],
Cell[4875, 177, 51, 1, 30, "Input"],
Cell[4929, 180, 26, 0, 33, "Text"],
Cell[4958, 182, 313, 6, 130, "Input"],
Cell[5274, 190, 28, 0, 33, "Text"],
Cell[5305, 192, 179, 3, 46, "Input"],
Cell[5487, 197, 31, 0, 33, "Text"],
Cell[5521, 199, 126, 2, 50, "Input"],
Cell[5650, 203, 40, 0, 33, "Text"],
Cell[5693, 205, 645, 12, 90, "Input"],
Cell[6341, 219, 38, 0, 33, "Text"],
Cell[6382, 221, 164, 3, 44, "Input"],
Cell[6549, 226, 24, 0, 33, "Text"],
Cell[6576, 228, 159, 2, 46, "Input"],
Cell[6738, 232, 30, 0, 33, "Text"],
Cell[6771, 234, 70, 1, 30, "Input"],
Cell[6844, 237, 24, 0, 33, "Text"],
Cell[6871, 239, 656, 11, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7564, 255, 30, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[7619, 259, 45, 0, 73, "Section"],
Cell[7667, 261, 46, 1, 30, "Input"],
Cell[7716, 264, 38, 0, 33, "Text"],
Cell[7757, 266, 145, 2, 44, "Input"],
Cell[7905, 270, 24, 0, 33, "Text"],
Cell[7932, 272, 84, 1, 41, "Input"],
Cell[8019, 275, 34, 0, 33, "Text"],
Cell[8056, 277, 43, 1, 30, "Input"],
Cell[8102, 280, 62, 0, 33, "Text"],
Cell[8167, 282, 133, 2, 30, "Input"],
Cell[8303, 286, 367, 6, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8707, 297, 41, 0, 73, "Section"],
Cell[8751, 299, 46, 1, 30, "Input"],
Cell[8800, 302, 38, 0, 33, "Text"],
Cell[8841, 304, 145, 2, 44, "Input"],
Cell[8989, 308, 24, 0, 33, "Text"],
Cell[9016, 310, 84, 1, 41, "Input"],
Cell[9103, 313, 43, 1, 30, "Input"],
Cell[9149, 316, 31, 0, 33, "Text"],
Cell[9183, 318, 133, 2, 30, "Input"],
Cell[9319, 322, 367, 6, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9723, 333, 44, 0, 73, "Section"],
Cell[9770, 335, 46, 1, 30, "Input"],
Cell[9819, 338, 118, 4, 36, "Text"],
Cell[9940, 344, 140, 2, 30, "Input"],
Cell[10083, 348, 24, 0, 33, "Text"],
Cell[10110, 350, 82, 1, 41, "Input"],
Cell[10195, 353, 33, 0, 33, "Text"],
Cell[10231, 355, 52, 1, 30, "Input"],
Cell[10286, 358, 31, 0, 33, "Text"],
Cell[10320, 360, 133, 2, 30, "Input"],
Cell[10456, 364, 34, 0, 33, "Text"],
Cell[10493, 366, 536, 9, 106, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11066, 380, 32, 0, 73, "Section"],
Cell[11101, 382, 46, 1, 30, "Input"],
Cell[11150, 385, 60, 1, 30, "Input"],
Cell[11213, 388, 49, 1, 42, "Input"],
Cell[11265, 391, 106, 2, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11420, 399, 48, 0, 95, "Title"],
Cell[11471, 401, 46, 1, 30, "Input"],
Cell[11520, 404, 45, 0, 33, "Text"],
Cell[11568, 406, 494, 8, 133, "Input"],
Cell[12065, 416, 69, 0, 33, "Text"],
Cell[12137, 418, 319, 5, 42, "Input"],
Cell[12459, 425, 302, 5, 45, "Input"],
Cell[12764, 432, 247, 4, 42, "Input"],
Cell[13014, 438, 101, 2, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13152, 445, 47, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[13224, 449, 42, 0, 73, "Section"],
Cell[13269, 451, 46, 1, 30, "Input"],
Cell[13318, 454, 28, 0, 33, "Text"],
Cell[13349, 456, 127, 3, 45, "Input"],
Cell[13479, 461, 787, 18, 217, "Input"],
Cell[14269, 481, 69, 0, 33, "Text"],
Cell[14341, 483, 1001, 16, 145, "Input"],
Cell[15345, 501, 68, 0, 33, "Text"],
Cell[15416, 503, 50, 1, 30, "Input"],
Cell[15469, 506, 50, 1, 30, "Input"],
Cell[15522, 509, 59, 1, 30, "Input"],
Cell[15584, 512, 48, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15669, 518, 33, 0, 73, "Section"],
Cell[15705, 520, 51, 1, 30, "Input"],
Cell[15759, 523, 36, 0, 33, "Text"],
Cell[15798, 525, 249, 5, 110, "Input"],
Cell[16050, 532, 22, 0, 33, "Text"],
Cell[16075, 534, 198, 3, 63, "Input"],
Cell[16276, 539, 39, 0, 33, "Text"],
Cell[16318, 541, 182, 3, 42, "Input"],
Cell[16503, 546, 41, 0, 33, "Text"],
Cell[16547, 548, 286, 5, 30, "Input"],
Cell[16836, 555, 36, 0, 33, "Text"],
Cell[16875, 557, 63, 1, 30, "Input"],
Cell[16941, 560, 56, 0, 33, "Text"],
Cell[17000, 562, 129, 3, 44, "Input"],
Cell[17132, 567, 29, 0, 33, "Text"],
Cell[17164, 569, 577, 11, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17778, 585, 27, 0, 73, "Section"],
Cell[17808, 587, 35, 0, 33, "Text"],
Cell[17846, 589, 183, 3, 45, "Input"],
Cell[18032, 594, 50, 0, 33, "Text"],
Cell[18085, 596, 147, 3, 30, "Input"],
Cell[18235, 601, 50, 0, 33, "Text"],
Cell[18288, 603, 79, 1, 30, "Input"],
Cell[18370, 606, 45, 0, 33, "Text"],
Cell[18418, 608, 175, 3, 30, "Input"],
Cell[18596, 613, 30, 0, 33, "Text"],
Cell[18629, 615, 344, 6, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[19022, 627, 46, 0, 95, "Title"],
Cell[19071, 629, 51, 1, 30, "Input"],
Cell[19125, 632, 26, 0, 33, "Text"],
Cell[19154, 634, 89, 2, 50, "Input"],
Cell[19246, 638, 22, 0, 33, "Text"],
Cell[19271, 640, 110, 2, 79, "Input"],
Cell[19384, 644, 39, 0, 33, "Text"],
Cell[19426, 646, 78, 1, 40, "Input"],
Cell[19507, 649, 31, 0, 33, "Text"],
Cell[19541, 651, 70, 1, 30, "Input"],
Cell[19614, 654, 40, 0, 33, "Text"],
Cell[19657, 656, 79, 1, 34, "Input"],
Cell[19739, 659, 38, 0, 33, "Text"],
Cell[19780, 661, 489, 8, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20306, 674, 32, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[20363, 678, 42, 0, 73, "Section"],
Cell[20408, 680, 51, 1, 30, "Input"],
Cell[20462, 683, 34, 0, 33, "Text"],
Cell[20499, 685, 113, 2, 45, "Input"],
Cell[20615, 689, 22, 0, 33, "Text"],
Cell[20640, 691, 293, 4, 121, "Input"],
Cell[20936, 697, 68, 0, 33, "Text"],
Cell[21007, 699, 401, 7, 116, "Input"],
Cell[21411, 708, 38, 0, 33, "Text"],
Cell[21452, 710, 204, 3, 42, "Input"],
Cell[21659, 715, 44, 0, 33, "Text"],
Cell[21706, 717, 108, 2, 30, "Input"],
Cell[21817, 721, 89, 1, 30, "Input"],
Cell[21909, 724, 69, 1, 30, "Input"],
Cell[21981, 727, 161, 5, 34, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22179, 737, 47, 0, 73, "Section"],
Cell[22229, 739, 46, 1, 30, "Input"],
Cell[22278, 742, 26, 0, 33, "Text"],
Cell[22307, 744, 401, 8, 170, "Input"],
Cell[22711, 754, 97, 2, 33, "Text"],
Cell[22811, 758, 596, 10, 91, "Input"],
Cell[23410, 770, 30, 0, 33, "Text"],
Cell[23443, 772, 147, 2, 30, "Input"],
Cell[23593, 776, 32, 0, 33, "Text"],
Cell[23628, 778, 56, 1, 30, "Input"],
Cell[23687, 781, 53, 0, 33, "Text"],
Cell[23743, 783, 82, 1, 30, "Input"],
Cell[23828, 786, 29, 0, 33, "Text"],
Cell[23860, 788, 123, 2, 50, "Input"],
Cell[23986, 792, 39, 0, 33, "Text"],
Cell[24028, 794, 476, 8, 87, "Input"],
Cell[24507, 804, 31, 0, 33, "Text"],
Cell[24541, 806, 476, 9, 74, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[25066, 821, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[25116, 825, 38, 0, 73, "Section"],
Cell[25157, 827, 46, 1, 30, "Input"],
Cell[25206, 830, 26, 0, 33, "Text"],
Cell[25235, 832, 136, 2, 50, "Input"],
Cell[25374, 836, 40, 0, 33, "Text"],
Cell[25417, 838, 70, 1, 46, "Input"],
Cell[25490, 841, 52, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25579, 847, 36, 0, 73, "Section"],
Cell[25618, 849, 46, 1, 30, "Input"],
Cell[25667, 852, 26, 0, 33, "Text"],
Cell[25696, 854, 233, 4, 90, "Input"],
Cell[25932, 860, 31, 0, 33, "Text"],
Cell[25966, 862, 62, 1, 42, "Input"],
Cell[26031, 865, 25, 0, 33, "Text"],
Cell[26059, 867, 53, 1, 42, "Input"],
Cell[26115, 870, 27, 0, 33, "Text"],
Cell[26145, 872, 113, 2, 50, "Input"],
Cell[26261, 876, 53, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26351, 882, 26, 0, 73, "Section"],
Cell[26380, 884, 46, 1, 30, "Input"],
Cell[26429, 887, 26, 0, 33, "Text"],
Cell[26458, 889, 408, 7, 130, "Input"],
Cell[26869, 898, 32, 0, 33, "Text"],
Cell[26904, 900, 63, 1, 43, "Input"],
Cell[26970, 903, 44, 0, 33, "Text"],
Cell[27017, 905, 108, 2, 50, "Input"],
Cell[27128, 909, 35, 0, 33, "Text"],
Cell[27166, 911, 49, 1, 30, "Input"],
Cell[27218, 914, 35, 0, 33, "Text"],
Cell[27256, 916, 104, 2, 51, "Input"],
Cell[27363, 920, 30, 0, 33, "Text"],
Cell[27396, 922, 67, 1, 42, "Input"],
Cell[27466, 925, 62, 1, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27577, 932, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[27627, 936, 42, 0, 73, "Section"],

Cell[CellGroupData[{
Cell[27694, 940, 42, 0, 38, "Subsection"],
Cell[27739, 942, 46, 1, 30, "Input"],
Cell[27788, 945, 39, 0, 33, "Text"],
Cell[27830, 947, 79, 1, 30, "Input"],
Cell[27912, 950, 28, 0, 33, "Text"],
Cell[27943, 952, 83, 1, 30, "Input"],
Cell[28029, 955, 51, 0, 33, "Text"],
Cell[28083, 957, 133, 2, 30, "Input"],
Cell[28219, 961, 37, 0, 33, "Text"],
Cell[28259, 963, 54, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28350, 969, 63, 0, 38, "Subsection"],
Cell[28416, 971, 631, 12, 186, "Input"],
Cell[29050, 985, 76, 0, 33, "Text"],
Cell[29129, 987, 45, 1, 30, "Input"],
Cell[29177, 990, 45, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[29271, 997, 44, 0, 73, "Section"],
Cell[29318, 999, 46, 1, 30, "Input"],
Cell[29367, 1002, 129, 2, 58, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

