(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     15958,        652]*)
(*NotebookOutlinePosition[     16630,        675]*)
(*  CellTagsIndexPosition[     16586,        671]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["29. Subsonic Flight", "Title"],

Cell[BoxData[{
    \(<< Lautrup`\), "\n", 
    \(<< Graphics`PlotField`\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Inside solution", "Title"],

Cell[BoxData[
    \(\(CleanGlobal[];\)\)], "Input"],

Cell["Basic structure of inside solution", "Text"],

Cell[BoxData[
    \(\(f = \(1\/x\) 
          Exp[\(-U\)\ \(\(\ \)\(y\^2 + \ \ z\^2\)\)\/\(4\ \[Nu]\ x\)];\)\)], \
"Input"],

Cell["Check that it satisfies the field equation", "Text"],

Cell[BoxData[
    \(0 == \((U\ \[PartialD]\_x 
                f - \[Nu] \((\[PartialD]\_\(y, y\)f + \[PartialD]\_\(z, z\)f)\
\))\) // Simplify\)], "Input"],

Cell["The three components of the velocity field", "Text"],

Cell[BoxData[{
    \(\(u\_x = \(A\_x\) f;\)\), "\[IndentingNewLine]", 
    \(\(u\_y = \(A\_y\) f;\)\), "\[IndentingNewLine]", 
    \(\(u\_z = \(A\_z\) f;\)\)}], "Input"],

Cell["Velocity potential", "Text"],

Cell[BoxData[
    \(\(\[CapitalPhi] = \(\(2  \[Nu]\)\/U\) \((\(-\(\(\(A\_y\) y + \(A\_z\) 
                        z\)\/\(y\^2 + z\^2\)\)\) + \ A\_x\/\(2  x\))\) \((1 - 
              Exp[\(-U\)\ \(\(\ \)\(y\^2 + \ \ z\^2\)\)\/\(4\ \[Nu]\ x\)])\);\
\)\)], "Input"],

Cell["Check that it satisfies the field equation", "Text"],

Cell[BoxData[
    \(0 == \[PartialD]\_x u\_x + \[PartialD]\_y u\_y + \[PartialD]\_z u\_z + \
\[PartialD]\_\(y, y\)\[CapitalPhi] + \[PartialD]\_\(z, z\)\[CapitalPhi] // 
      Simplify\)], "Input"],

Cell["Pressure", "Text"],

Cell[BoxData[
    \(p = \(\[Rho]\_0\) \((\(-U\) \[PartialD]\_x \[CapitalPhi] + \[Nu] \((\
\[PartialD]\_\(y, y\)\[CapitalPhi] + \[PartialD]\_\(z, z\)\[CapitalPhi])\))\) \
// Simplify\)], "Input"],

Cell[BoxData[
    \(\($Assumptions = U\/\(x\ \[Nu]\) > 0;\)\)], "Input"],

Cell[BoxData[
    \(drag = \(-\[Rho]\_0\) 
        U\ \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity]\((\[Integral]\_\(-\
\[Infinity]\)\%\[Infinity]\( 
                      u\_x\) \[DifferentialD]y)\) \[DifferentialD]z\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pressure and lift", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Pressure contribution", "Section"],

Cell["Velocity potential", "Text"],

Cell[BoxData[
    \(\(\[CapitalPhi] = \(1\/\(4  \[Pi]\ r\)\) 
          Cos[\[Phi]]\ Cot[\[Theta]\/2];\)\)], "Input"],

Cell["Pressure", "Text"],

Cell[BoxData[
    \(p = \(-\((Cos[\[Theta]] \[PartialD]\_r \[CapitalPhi] - 
              Sin[\[Theta]] \(1\/r\) \[PartialD]\_\[Theta] \[CapitalPhi])\)\) \
// Simplify\)], "Input"],

Cell["Contribution to lift", "Text"],

Cell[BoxData[
    \(\(-\(\[Integral]\_0\%\[Pi]\((\[Integral]\_0\%\(2  \[Pi]\)p\ \
\(r\^\(\(2\)\(\ \)\)\) 
                Cos[\[Phi]] \[DifferentialD]\[Phi])\)\ \(Sin[\[Theta]]\^2\) \
\[DifferentialD]\[Theta]\)\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["29.1.1 Boing 747-200", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(M = 374000.0\ Kilogram;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(T = 973000.0\ Newton;\)\), "\[IndentingNewLine]", 
    \(\(U = 290\ Kilometer/Hour;\)\)}], "Input"],

Cell["Horizontal acceleration", "Text"],

Cell[BoxData[
    \(g = T\/M // ToSI\)], "Input"],

Cell["Relative to standard gravity", "Text"],

Cell[BoxData[
    \(g\/g\_0 // ToSI\)], "Input"],

Cell["Takeoff length", "Text"],

Cell[BoxData[
    \(L = U\^2\/\(2  g\) // ToSI\)], "Input"],

Cell["Takeoff time", "Text"],

Cell[BoxData[
    \(t = U\/g // ToSI\)], "Input"],

Cell[CellGroupData[{

Cell["Example 29.3.1", "Subsection"],

Cell[BoxData[
    \(k = T\/\(M\ g\_0\) // ToSI\)], "Input"],

Cell[BoxData[
    \(\[Theta] = ArcSin[k] // ToSI\)], "Input"],

Cell[BoxData[
    \(\[Theta]\/Degree\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["29.3.2 Banked turn", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(U = 900\ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(\[Beta] = 15\ Degree;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Effective gravity for various bank angles", "Text"],

Cell[BoxData[
    \(Sec[60\ Degree] // N\)], "Input"],

Cell[BoxData[
    \(Sec[84.3\ Degree] // N\)], "Input"],

Cell[BoxData[
    \(Sec[\[Beta]] // N\)], "Input"],

Cell["Clean turn radius", "Text"],

Cell[BoxData[
    \(R = U\^2\/\(\(g\_0\) Tan[\[Beta]]\) // ToUnit[Kilometer]\)], "Input"],

Cell[BoxData[
    \(2  R\)], "Input"],

Cell["Time of full turn", "Text"],

Cell[BoxData[
    \(t = \(2  \[Pi]\ R\)\/U // ToUnit[Minute]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cessna 150", "Section"],

Cell[CellGroupData[{

Cell["Structure", "Subsection"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(L = 10.17\ \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(A = 14.8\ Meter\^2;\)\), "\[IndentingNewLine]", 
    \(\(M = 681\ Kilogram;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(U = 180. \ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 1.5*^-5\ Meter\^2/Second;\)\), "\[IndentingNewLine]", 
    \(\(c\_S = 343\ Meter/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1.2\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(LD = 8.4;\)\), "\[IndentingNewLine]", 
    \(\(fuel = 135\ Pound;\)\), "\[IndentingNewLine]", 
    \(\(power = 75\ Kilo\ Watt;\)\), "\[IndentingNewLine]", 
    \(\(range = 764\ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(stall = 42  Knot;\)\), "\[IndentingNewLine]", 
    \(\(glide = 8.4;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha]\_0 = \(-2\) Degree;\)\)}], "Input"],

Cell["Average chord length", "Text"],

Cell[BoxData[
    \(c = A\/L // ToSI\)], "Input"],

Cell["Aspect ratio", "Text"],

Cell[BoxData[
    \(L\/c // ToSI\)], "Input"],

Cell["Wing load", "Text"],

Cell[BoxData[
    RowBox[{"\[CapitalDelta]p", "=", 
      RowBox[{\(\(M\ g\_0\)\/A\), "//", 
        RowBox[{\(ToUnit[Pascal]\), Cell[""]}]}]}]], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]p // ToUnit[Atmosphere]\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Rey = \(U\ c\)\/\[Nu] // ToSI\)], "Input"],

Cell["Mach number", "Text"],

Cell[BoxData[
    \(Ma = U\/c\_S // ToSI\)], "Input"],

Cell["Velocity difference on wing surfaces", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]v = \(M\ g\_0\)\/\(\(\[Rho]\_0\) U\ A\) // 
        ToSI\)], "Input"],

Cell["Relative to speed", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]v\/U // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Lift coefficient", "Subsection"],

Cell[BoxData[
    \(C\_L = 2  \[CapitalDelta]v\/U // ToSI\)], "Input"],

Cell["Lift slope", "Text"],

Cell[BoxData[
    \(\[Lambda] = \(2  \[Pi]\)\/\(1 + 2\ c/L\) // ToSI\)], "Input"],

Cell["Relative angle of attack", "Text"],

Cell[BoxData[
    \(\[Alpha]\_rel = C\_L\/\[Lambda]\)], "Input"],

Cell[BoxData[
    \(\[Alpha]\_rel/Degree\)], "Input"],

Cell["Angle of attack", "Text"],

Cell[BoxData[
    \(\[Alpha] = \[Alpha]\_rel + \[Alpha]\_0\)], "Input"],

Cell[BoxData[
    \(\[Alpha]/Degree\)], "Input"],

Cell["Stall angle", "Text"],

Cell[BoxData[
    \(\(\[Alpha]\_stall = 16  Degree;\)\)], "Input"],

Cell["Stall velocity", "Text"],

Cell[BoxData[
    \(U\_stall = \(\@\(\[Alpha]\_rel\/\(\[Alpha]\_stall - \[Alpha]\_0\)\)\) 
        U\)], "Input"],

Cell["Compare with tabulated value", "Text"],

Cell[BoxData[
    \(stall // ToUnit[Kilometer/Hour]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Drag coefficient", "Subsection"],

Cell[BoxData[
    \(\[Delta]\_lam = 
      5\ \(Rey\^\(\(-1\)/2\)\) c // ToUnit[Millimeter]\)], "Input"],

Cell[BoxData[
    \(\[Delta]\_turb = 
      0.16\ \(Rey\^\(\(-1\)/7\)\) c // ToUnit[Millimeter]\)], "Input"],

Cell[BoxData[
    \(C\_Dlam = 2.65\ Rey\^\(\(-1\)/2\) // ToSI\)], "Input"],

Cell[BoxData[
    \(C\_Dturb = 0.063\ Rey\^\(\(-1\)/7\) // ToSI\)], "Input"],

Cell[BoxData[
    \(C\_Dind = \(c\/\(\(\[Pi]\)\(\ \)\(L\)\(\ \)\)\) C\_L\^2\)], "Input"],

Cell[BoxData[
    \(C\_D = C\_L\/LD\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Glide parameters", "Subsection"],

Cell[BoxData[
    \(\[Gamma] = ArcCot[LD] // ToSI\)], "Input"],

Cell[BoxData[
    \(\[Gamma]/Degree\)], "Input"],

Cell[BoxData[
    \(U\_glide = 60\ Knot // ToUnit[Kilometer/Hour]\)], "Input"],

Cell[BoxData[
    RowBox[{\(C\_Lglide\), "=", 
      RowBox[{
        FractionBox[
          RowBox[{
          "M", " ", \(g\_0\), " ", \(Cos[\[Gamma]]\), Cell[
            ""]}], \(\(1\/2\) \(\[Rho]\_0\) \(U\_glide\^2\) A\)], "//", 
        "ToSI"}]}]], "Input"],

Cell["Glide angle of attack", "Text"],

Cell[BoxData[
    \(\[Alpha]\_glide = \[Alpha]\_0 + C\_Lglide\/\[Lambda]\)], "Input"],

Cell[BoxData[
    \(\[Alpha]\_glide/Degree\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fuel consumption in cruise", "Subsection"],

Cell["Assumed lift to drag ratio", "Text"],

Cell[BoxData[
    \(\(LD1 = 5;\)\)], "Input"],

Cell["Thrust in level flight", "Text"],

Cell[BoxData[
    \(T = \(M\ g\_0\)\/LD1 // ToUnit[Newton]\)], "Input"],

Cell["Fuel consumption", "Text"],

Cell[BoxData[
    \(\(F = 16  Kilogram/Hour;\)\)], "Input"],

Cell["Specific fuel consumption", "Text"],

Cell[BoxData[
    \(f = \(F\ g\_0\)\/T // ToSI\)], "Input"],

Cell["Amount of fuel", "Text"],

Cell[BoxData[
    \(\(\[CapitalDelta]M = fuel;\)\)], "Input"],

Cell["Endurance", "Text"],

Cell[BoxData[
    \(t = \(1\/f\) LD1\ Log[M\/\(M - \[CapitalDelta]M\)] // ToSI\)], "Input"],

Cell[BoxData[
    \(t // ToUnit[Hour]\)], "Input"],

Cell["Range", "Text"],

Cell[BoxData[
    \(x = U\ t // ToUnit[Kilometer]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["29.4.1 Boeing 747-400", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(L = 64. \ Meter;\)\), "\[IndentingNewLine]", 
    \(\(A = 520. \ Meter\^2;\)\), "\[IndentingNewLine]", 
    \(\(U = 900\ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(\[Eta]\_0 = 1.8*^-5\ Pascal\ Second;\)\), "\[IndentingNewLine]", 
    \(\(M = 400000. \ Kilogram;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(h\_2 = 30. \ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(\[Rho]\_0 = 1.2\ Kilogram/Meter\^3;\)\), "\[IndentingNewLine]", 
    \(\(p\_0 = 1\ Atmosphere;\)\), "\[IndentingNewLine]", 
    \(\(T\_0 = 293\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(z = 10. \ Kilometer;\)\), "\[IndentingNewLine]", 
    \(\(\[Gamma] = 7\/5;\)\)}], "Input"],

Cell["Velocity", "Text"],

Cell[BoxData[
    \(U // ToSI\)], "Input"],

Cell["Pressure at cruising altitude", "Text"],

Cell[BoxData[
    \(p = \(p\_0\) \((1 - z\/h\_2)\)\^\(\[Gamma]/\((\[Gamma] - 1)\)\)\)], \
"Input"],

Cell["Density", "Text"],

Cell[BoxData[
    \(\[Rho] = \(\[Rho]\_0\) \((1 - z\/h\_2)\)\^\(1/\((\[Gamma] - 1)\)\) // 
        ToSI\)], "Input"],

Cell["Temperature", "Text"],

Cell[BoxData[
    \(T = \(T\_0\) \((1 - z\/h\_2)\)\)], "Input"],

Cell["Sound speed", "Text"],

Cell[BoxData[
    \(c\_S = \@\(\[Gamma]\ p\/\[Rho]\) // ToSI\)], "Input"],

Cell["Mach number", "Text"],

Cell[BoxData[
    \(Ma = U\/c\_S // ToSI\)], "Input"],

Cell["Glauert factor", "Text"],

Cell[BoxData[
    \(1\/\@\(1 - Ma\^2\)\)], "Input"],

Cell["Average chord length", "Text"],

Cell[BoxData[
    \(c = A\/L // ToSI\)], "Input"],

Cell["Aspect ratio", "Text"],

Cell[BoxData[
    \(L\/c // ToSI\)], "Input"],

Cell["Viscosity", "Text"],

Cell[BoxData[
    \(\[Eta] = \(\[Eta]\_0\) \@\(T\/T\_0\)\)], "Input"],

Cell["Kinematic viscosity", "Text"],

Cell[BoxData[
    \(\[Nu] = \[Eta]\/\[Rho] // ToSI\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Rey\_c = \(U\ c\)\/\[Nu] // ToSI\)], "Input"],

Cell["Pressure difference", "Text"],

Cell[BoxData[
    \(\[CapitalDelta]p = \(M\ g\_0\)\/A // ToUnit[Pascal]\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]p // ToUnit[Atmosphere]\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]p\/p // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Airfoil", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\(y = 2  K\ \(x\/c\) \((c - x)\);\)\)], "Input"],

Cell[BoxData[
    \(\($Assumptions = c > 0;\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(\[Integral]\_0\%c\), 
      RowBox[{
        FractionBox["y", 
          RowBox[{\((c - x)\), 
            SqrtBox[
              RowBox[{
              "x", \((c - x)\), Cell[
                ""]}]]}]], \(\[DifferentialD]x\)}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["29.1 Concorde takeof", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(M = 185000. \ Kilogram;\)\), "\[IndentingNewLine]", 
    \(\(T = 677000. \ Newton;\)\), "\[IndentingNewLine]", 
    \(\(U = 360. \ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\)}], "Input"],

Cell["Acceleration", "Text"],

Cell[BoxData[
    \(g = T\/M // ToSI\)], "Input"],

Cell["In units of standard gravity", "Text"],

Cell[BoxData[
    \(g\/g\_0\)], "Input"],

Cell["Takeoff time at constant acceleration", "Text"],

Cell[BoxData[
    \(t = U\/g // ToSI\)], "Input"],

Cell["Takeoff length", "Text"],

Cell[BoxData[
    \(L = \(1\/2\) g\ t\^2 // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["29. 4 Cessna takeoff", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(M = 681. \ Kilogram;\)\), "\[IndentingNewLine]", 
    \(\(U = 120. \ Kilometer/Hour;\)\), "\[IndentingNewLine]", 
    \(\(L = 225. \ Meter;\)\)}], "Input"],

Cell["Takeoff power", "Text"],

Cell[BoxData[
    \(P = \(M\ U\^3\)\/\(3  L\) // ToUnit[Kilo\ Watt]\)], "Input"],

Cell["Takeoff time", "Text"],

Cell[BoxData[
    \(\(M\ U\^2\)\/\(2  P\) // ToSI\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{717, 740},
WindowMargins->{{0, Automatic}, {Automatic, 23}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 36, 0, 95, "Title"],
Cell[1815, 55, 113, 3, 50, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[1965, 63, 32, 0, 95, "Title"],
Cell[2000, 65, 51, 1, 30, "Input"],
Cell[2054, 68, 50, 0, 33, "Text"],
Cell[2107, 70, 123, 3, 46, "Input"],
Cell[2233, 75, 58, 0, 33, "Text"],
Cell[2294, 77, 156, 3, 30, "Input"],
Cell[2453, 82, 58, 0, 33, "Text"],
Cell[2514, 84, 169, 3, 70, "Input"],
Cell[2686, 89, 34, 0, 33, "Text"],
Cell[2723, 91, 264, 4, 47, "Input"],
Cell[2990, 97, 58, 0, 33, "Text"],
Cell[3051, 99, 196, 3, 30, "Input"],
Cell[3250, 104, 24, 0, 33, "Text"],
Cell[3277, 106, 194, 3, 30, "Input"],
Cell[3474, 111, 72, 1, 43, "Input"],
Cell[3549, 114, 233, 5, 41, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3819, 124, 34, 0, 95, "Title"],
Cell[3856, 126, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[3927, 131, 40, 0, 73, "Section"],
Cell[3970, 133, 34, 0, 33, "Text"],
Cell[4007, 135, 117, 2, 43, "Input"],
Cell[4127, 139, 24, 0, 33, "Text"],
Cell[4154, 141, 180, 3, 42, "Input"],
Cell[4337, 146, 36, 0, 33, "Text"],
Cell[4376, 148, 225, 4, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4650, 158, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[4700, 162, 39, 0, 73, "Section"],
Cell[4742, 164, 46, 1, 30, "Input"],
Cell[4791, 167, 26, 0, 33, "Text"],
Cell[4820, 169, 253, 4, 90, "Input"],
Cell[5076, 175, 39, 0, 33, "Text"],
Cell[5118, 177, 49, 1, 42, "Input"],
Cell[5170, 180, 44, 0, 33, "Text"],
Cell[5217, 182, 48, 1, 42, "Input"],
Cell[5268, 185, 30, 0, 33, "Text"],
Cell[5301, 187, 59, 1, 47, "Input"],
Cell[5363, 190, 28, 0, 33, "Text"],
Cell[5394, 192, 49, 1, 44, "Input"],

Cell[CellGroupData[{
Cell[5468, 197, 36, 0, 38, "Subsection"],
Cell[5507, 199, 59, 1, 44, "Input"],
Cell[5569, 202, 61, 1, 30, "Input"],
Cell[5633, 205, 49, 1, 44, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5731, 212, 37, 0, 73, "Section"],
Cell[5771, 214, 46, 1, 30, "Input"],
Cell[5820, 217, 26, 0, 33, "Text"],
Cell[5849, 219, 192, 3, 70, "Input"],
Cell[6044, 224, 57, 0, 33, "Text"],
Cell[6104, 226, 53, 1, 30, "Input"],
Cell[6160, 229, 55, 1, 30, "Input"],
Cell[6218, 232, 50, 1, 30, "Input"],
Cell[6271, 235, 33, 0, 33, "Text"],
Cell[6307, 237, 89, 1, 47, "Input"],
Cell[6399, 240, 37, 1, 30, "Input"],
Cell[6439, 243, 33, 0, 33, "Text"],
Cell[6475, 245, 73, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6585, 251, 29, 0, 73, "Section"],

Cell[CellGroupData[{
Cell[6639, 255, 31, 0, 38, "Subsection"],
Cell[6673, 257, 46, 1, 30, "Input"],
Cell[6722, 260, 26, 0, 33, "Text"],
Cell[6751, 262, 908, 15, 310, "Input"],
Cell[7662, 279, 36, 0, 33, "Text"],
Cell[7701, 281, 49, 1, 42, "Input"],
Cell[7753, 284, 28, 0, 33, "Text"],
Cell[7784, 286, 45, 1, 42, "Input"],
Cell[7832, 289, 25, 0, 33, "Text"],
Cell[7860, 291, 154, 3, 42, "Input"],
Cell[8017, 296, 71, 1, 30, "Input"],
Cell[8091, 299, 31, 0, 33, "Text"],
Cell[8125, 301, 62, 1, 42, "Input"],
Cell[8190, 304, 27, 0, 33, "Text"],
Cell[8220, 306, 53, 1, 44, "Input"],
Cell[8276, 309, 52, 0, 33, "Text"],
Cell[8331, 311, 104, 2, 44, "Input"],
Cell[8438, 315, 33, 0, 33, "Text"],
Cell[8474, 317, 60, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8571, 323, 38, 0, 38, "Subsection"],
Cell[8612, 325, 70, 1, 42, "Input"],
Cell[8685, 328, 26, 0, 33, "Text"],
Cell[8714, 330, 81, 1, 43, "Input"],
Cell[8798, 333, 40, 0, 33, "Text"],
Cell[8841, 335, 64, 1, 42, "Input"],
Cell[8908, 338, 53, 1, 30, "Input"],
Cell[8964, 341, 31, 0, 33, "Text"],
Cell[8998, 343, 71, 1, 30, "Input"],
Cell[9072, 346, 48, 1, 30, "Input"],
Cell[9123, 349, 27, 0, 33, "Text"],
Cell[9153, 351, 66, 1, 30, "Input"],
Cell[9222, 354, 30, 0, 33, "Text"],
Cell[9255, 356, 113, 2, 51, "Input"],
Cell[9371, 360, 44, 0, 33, "Text"],
Cell[9418, 362, 64, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9519, 368, 38, 0, 38, "Subsection"],
Cell[9560, 370, 104, 2, 30, "Input"],
Cell[9667, 374, 108, 2, 30, "Input"],
Cell[9778, 378, 74, 1, 30, "Input"],
Cell[9855, 381, 76, 1, 30, "Input"],
Cell[9934, 384, 88, 1, 41, "Input"],
Cell[10025, 387, 48, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10110, 393, 38, 0, 38, "Subsection"],
Cell[10151, 395, 62, 1, 30, "Input"],
Cell[10216, 398, 48, 1, 30, "Input"],
Cell[10267, 401, 78, 1, 30, "Input"],
Cell[10348, 404, 264, 7, 60, "Input"],
Cell[10615, 413, 37, 0, 33, "Text"],
Cell[10655, 415, 85, 1, 43, "Input"],
Cell[10743, 418, 55, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10835, 424, 48, 0, 38, "Subsection"],
Cell[10886, 426, 42, 0, 33, "Text"],
Cell[10931, 428, 45, 1, 30, "Input"],
Cell[10979, 431, 38, 0, 33, "Text"],
Cell[11020, 433, 71, 1, 42, "Input"],
Cell[11094, 436, 32, 0, 33, "Text"],
Cell[11129, 438, 59, 1, 30, "Input"],
Cell[11191, 441, 41, 0, 33, "Text"],
Cell[11235, 443, 59, 1, 42, "Input"],
Cell[11297, 446, 30, 0, 33, "Text"],
Cell[11330, 448, 61, 1, 30, "Input"],
Cell[11394, 451, 25, 0, 33, "Text"],
Cell[11422, 453, 91, 1, 42, "Input"],
Cell[11516, 456, 50, 1, 30, "Input"],
Cell[11569, 459, 21, 0, 33, "Text"],
Cell[11593, 461, 62, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11704, 468, 40, 0, 73, "Section"],
Cell[11747, 470, 46, 1, 30, "Input"],
Cell[11796, 473, 26, 0, 33, "Text"],
Cell[11825, 475, 742, 12, 263, "Input"],
Cell[12570, 489, 24, 0, 33, "Text"],
Cell[12597, 491, 42, 1, 30, "Input"],
Cell[12642, 494, 45, 0, 33, "Text"],
Cell[12690, 496, 98, 2, 45, "Input"],
Cell[12791, 500, 23, 0, 33, "Text"],
Cell[12817, 502, 116, 2, 45, "Input"],
Cell[12936, 506, 27, 0, 33, "Text"],
Cell[12966, 508, 63, 1, 44, "Input"],
Cell[13032, 511, 27, 0, 33, "Text"],
Cell[13062, 513, 73, 1, 51, "Input"],
Cell[13138, 516, 27, 0, 33, "Text"],
Cell[13168, 518, 53, 1, 44, "Input"],
Cell[13224, 521, 30, 0, 33, "Text"],
Cell[13257, 523, 51, 1, 48, "Input"],
Cell[13311, 526, 36, 0, 33, "Text"],
Cell[13350, 528, 49, 1, 42, "Input"],
Cell[13402, 531, 28, 0, 33, "Text"],
Cell[13433, 533, 45, 1, 42, "Input"],
Cell[13481, 536, 25, 0, 33, "Text"],
Cell[13509, 538, 69, 1, 51, "Input"],
Cell[13581, 541, 35, 0, 33, "Text"],
Cell[13619, 543, 63, 1, 42, "Input"],
Cell[13685, 546, 31, 0, 33, "Text"],
Cell[13719, 548, 65, 1, 42, "Input"],
Cell[13787, 551, 35, 0, 33, "Text"],
Cell[13825, 553, 84, 1, 42, "Input"],
Cell[13912, 556, 71, 1, 30, "Input"],
Cell[13986, 559, 60, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14083, 565, 26, 0, 73, "Section"],
Cell[14112, 567, 46, 1, 30, "Input"],
Cell[14161, 570, 68, 1, 40, "Input"],
Cell[14232, 573, 58, 1, 30, "Input"],
Cell[14293, 576, 268, 8, 74, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[14610, 590, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[14660, 594, 39, 0, 73, "Section"],
Cell[14702, 596, 46, 1, 30, "Input"],
Cell[14751, 599, 26, 0, 33, "Text"],
Cell[14780, 601, 255, 4, 90, "Input"],
Cell[15038, 607, 28, 0, 33, "Text"],
Cell[15069, 609, 49, 1, 42, "Input"],
Cell[15121, 612, 44, 0, 33, "Text"],
Cell[15168, 614, 40, 1, 42, "Input"],
Cell[15211, 617, 53, 0, 33, "Text"],
Cell[15267, 619, 49, 1, 44, "Input"],
Cell[15319, 622, 30, 0, 33, "Text"],
Cell[15352, 624, 61, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15450, 630, 39, 0, 73, "Section"],
Cell[15492, 632, 46, 1, 30, "Input"],
Cell[15541, 635, 178, 3, 70, "Input"],
Cell[15722, 640, 29, 0, 33, "Text"],
Cell[15754, 642, 80, 1, 46, "Input"],
Cell[15837, 645, 28, 0, 33, "Text"],
Cell[15868, 647, 62, 1, 46, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

