(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     41862,       1475]*)
(*NotebookOutlinePosition[     42570,       1499]*)
(*  CellTagsIndexPosition[     42526,       1495]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["31. Convection", "Title",
  InitializationCell->True],

Cell[BoxData[{
    \(<< Lautrup`\), "\[IndentingNewLine]", 
    \(<< Graphics`Graphics`\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Vertical slot heated on one side", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[
    \(\(Plot[4  x \((1 - x\^2)\), {x, 0, 1}, 
          AxesLabel \[Rule] {"\<x/d\>", \*"\"\<\!\(v\_z\)/U\>\""}] \
// \[IndentingNewLine]Figure["\<convection1\>", 
          ImageSize \[Rule] 200];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Thermal boundary layer", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Self-similar form of the fields", "Section"],

Cell[BoxData[
    \(\(\[Lambda] = \((\(\[Alpha]\ \[CapitalTheta]\ g\_0\)\/\(\[Kappa]\ \[Nu]\
\ z\))\)\^\(1/4\);\)\)], "Input"],

Cell[BoxData[
    \(\(\[Nu] = Pr\ \[Kappa];\)\)], "Input"],

Cell[BoxData[
    \(\(do[it_] := it /. s \[Rule] \[Lambda]\ x;\)\)], "Input"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = \[CapitalTheta]\ F[s] // do;\)\)], "Input"],

Cell[BoxData[
    \(\(v\_z = \(\@\(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) z\)\) G[s] // 
          do;\)\)], "Input"],

Cell[BoxData[
    \(\(v\_x = \(\((\(\[Alpha]\ \[CapitalTheta]\ \[Kappa]\ \[Nu]\ \
g\_0\)\/z)\)\^\(1/4\)\) H[s] // do;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Reduce field equations", "Section"],

Cell[BoxData[
    \(do[it_] := \(it /. x -> s/\[Lambda] // PowerExpand\) // 
        Simplify\)], "Input"],

Cell["Mass conservation", "Text"],

Cell[BoxData[
    \(zero1 = \[PartialD]\_x v\_x + \[PartialD]\_z v\_z // do\)], "Input"],

Cell[BoxData[
    \(eq1 = 
      zero1\ 4 \@ z/\((\(\@\[Alpha]\) \(\@\[CapitalTheta]\) \@g\_0)\)\)], \
"Input"],

Cell["Heat equation", "Text"],

Cell[BoxData[
    \(zero2 = \(v\_x\) \[PartialD]\_x \[CapitalDelta]T + \(v\_z\) \[PartialD]\
\_z \[CapitalDelta]T - \[Kappa]\ \[PartialD]\_x\^2 \[CapitalDelta]T // 
        do\)], "Input"],

Cell[BoxData[
    \(eq2 = \(-zero2\)\ 4 \@ z/\((\(\@\[Alpha]\) \(\[CapitalTheta]\^\(3/
                      2\)\) \@g\_0)\) // Expand\)], "Input"],

Cell["Navier-Stokes equation", "Text"],

Cell[BoxData[
    \(zero3 = \(v\_x\) \[PartialD]\_x v\_z + \(v\_z\) \[PartialD]\_z v\_z - \
\[Nu]\ \[PartialD]\_x\^2 v\_z - \[Alpha]\ \[CapitalDelta]T\ g\_0 // 
        do\)], "Input"],

Cell[BoxData[
    \(eq3 = \(\(-zero3\)\ 4/\((\[Alpha]\ \[CapitalTheta]\ g\_0)\) // 
          PowerExpand\) // Simplify\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduce convenient new variable", "Section"],

Cell[BoxData[
    \(\(H[s_] = Z[s] + \(1\/4\) s\ G[s];\)\)], "Input"],

Cell[BoxData[
    \(eq1 = eq1/4 // Expand\)], "Input"],

Cell[BoxData[
    \(eq2 = eq2/4 // Expand\)], "Input"],

Cell[BoxData[
    \(eq3 = eq3/4 // Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Table of solutions from MatLab", "Section"],

Cell["\<\
Data consists of  the value of  Pr, F'(0), G'(0), and Z(\[Infinity]).\
\>", "Text"],

Cell[BoxData[
    \(\(data = \[IndentingNewLine]\ {{0.001, \(-0.104962\), 
            8.28203, \(-5.74115\)}, \[IndentingNewLine]{0.003, \(-0.136491\), 
            6.17725, \(-4.39941\)}, \[IndentingNewLine]{0.01, \(-0.180211\), 
            4.41732, \(-3.25159\)}, \[IndentingNewLine]{0.03, \(-0.228766\), 
            3.18877, \(-2.46129\)}, \[IndentingNewLine]{0.1, \(-0.2894\), 
            2.16068, \(-1.81787\)}, \[IndentingNewLine]{0.3, \(-0.346229\), 
            1.45881, \(-1.39801\)}, \[IndentingNewLine]{1, \(-0.401033\), 
            0.908189, \(-1.10941\)}, \[IndentingNewLine]{3, \(-0.438181\), 
            0.570458, \(-0.988542\)}, \[IndentingNewLine]{10, \(-0.46497\), 
            0.333375, \(-0.940077\)}, \[IndentingNewLine]{30, \(-0.480073\), 
            0.200111, \(-0.916737\)}, \[IndentingNewLine]{100, \(-0.490007\), 
            0.112562, \(-0.916613\)}, \[IndentingNewLine]{300, \(-0.495289\), 
            0.0659342, \(-0.913399\)}, \[IndentingNewLine]{1000, \
\(-0.498621\), 0.0364498, \(-0.897798\)}};\)\)], "Input"],

Cell["Slope of heat function at the plate", "Text"],

Cell[BoxData[
    \(heatslope = data[\([All, {1, 2}]\)]\)], "Input"],

Cell[BoxData[
    \(heatslope[\([All, 2]\)] = \(-heatslope[\([All, 2]\)]\)\)], "Input"],

Cell[BoxData[
    \(\(LogLogListPlot[heatslope, PlotJoined \[Rule] True, 
          AspectRatio \[Rule] 0.7, 
          PlotStyle \[Rule] Text["\<(b)\>", {\(-0.3\), \(-0.3\)}], 
          AxesLabel \[Rule] {"\<Pr\>", "\<-F'(0)\>"}] \
// \[IndentingNewLine]Figure["\<convection3\>", 
          ImageSize \[Rule] 300];\)\)], "Input"],

Cell["Velocity gradient at plate", "Text"],

Cell[BoxData[
    \(momslope = data[\([All, {1, 3}]\)]\)], "Input"],

Cell[BoxData[
    \(\(LogLogListPlot[momslope, PlotJoined \[Rule] True];\)\)], "Input"],

Cell["Asymptotic value of Z", "Text"],

Cell[BoxData[
    \(\(asymp = data[\([All, {1, 4}]\)];\)\)], "Input"],

Cell[BoxData[
    \(asymp[\([All, 2]\)] = \(-asymp[\([All, 2]\)]\)\)], "Input"],

Cell[BoxData[
    \(\(LogLogListPlot[asymp, PlotJoined \[Rule] True];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numeric integration", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters. Slopes obtained from MatLab calculation at Pr=1", "Text"],

Cell[BoxData[{
    \(\(\[Alpha] = \(-0.4010\);\)\), "\[IndentingNewLine]", 
    \(\(\[Beta] = 0.9082;\)\), "\[IndentingNewLine]", 
    \(\(Pr = 1;\)\), "\[IndentingNewLine]", 
    \(\(A = 20;\)\)}], "Input"],

Cell["Differential equations", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"eq1", "=", 
        RowBox[{\(\(3\ G[s]\)\/4\), "+", 
          RowBox[{
            SuperscriptBox["Z", "\[Prime]",
              MultilineFunction->None], "[", "s", "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"eq2", "=", 
        RowBox[{
          RowBox[{\(-Z[s]\), " ", 
            RowBox[{
              SuperscriptBox["F", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox["F", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}], \(\@Pr\)]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"eq3", "=", 
        RowBox[{\(F[s]\), "-", \(G[s]\^2\/2\), "-", 
          RowBox[{\(Z[s]\), " ", 
            RowBox[{
              SuperscriptBox["G", "\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}], "+", 
          RowBox[{\(\@Pr\), " ", 
            RowBox[{
              SuperscriptBox["G", "\[Prime]\[Prime]",
                MultilineFunction->None], "[", "s", "]"}]}]}]}], 
      ";"}]}], "Input"],

Cell["General solution", "Text"],

Cell[BoxData[
    \(\(sol[\[Alpha]_, \[Beta]_, 
          Pr_] := {F, G, Z} /. 
            NDSolve[{eq1 \[Equal] 0, eq2 \[Equal] 0, eq3 \[Equal] 0, 
                F[0] \[Equal] 1, G[0] \[Equal] 0, 
                Z[0] \[Equal] 
                  0, \(F'\)[0] \[Equal] \[Alpha], \(G'\)[
                    0] \[Equal] \[Beta]}, {F, G, Z}, {s, 0, A}] // 
          First;\)\)], "Input"],

Cell["Error function", "Text"],

Cell[BoxData[
    \(cost[\[Alpha]_, \[Beta]_, 
        Pr_] := \(\(sol[\[Alpha], \[Beta], Pr]\)[\([1]\)]\)[A]\^2 + \(\(sol[\
\[Alpha], \[Beta], Pr]\)[\([2]\)]\)[A]\^2\)], "Input"],

Cell[BoxData[
    \(cost[\[Alpha], \[Beta], Pr]\)], "Input"],

Cell["Actual solution", "Text"],

Cell[BoxData[
    \(\({FF, GG, ZZ} = sol[\[Alpha], \[Beta], Pr];\)\)], "Input"],

Cell[BoxData[
    \(HH[s_] := ZZ[s] + \(1\/4\) s\ GG[s]\)], "Input"],

Cell[BoxData[
    \(\(Plot[{FF[s], GG[s], HH[s]}, {s, 0, 7}, 
          PlotRange \[Rule] {\(-1.2\), 1}, 
          AxesLabel \[Rule] {"\<s\>", "\<\>"}, \[IndentingNewLine]PlotStyle \
\[Rule] {{Text["\<F\>", {0.5, 0.9}], Text["\<G\>", {2.5, 0.42}], 
                Text["\<H\>", {2.5, \(-0.4\)}], Text["\<(a)\>", {3, 0.9}]}}, 
          AspectRatio \[Rule] 
            0.7] // \[IndentingNewLine]Figure["\<convection2\>", 
          ImageSize \[Rule] 300];\)\)], "Input"],

Cell[BoxData[
    \(HH[A]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rayleigh-Benard for  two free boundaries", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Stability region", "Section"],

Cell[TextData[{
  "In non-dimensional form the Rayleigh number is (with ",
  Cell[BoxData[
      \(TraditionalForm\`kd = \(k\_x\) d\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`k\_z = \[Pi]/d\)]],
  ")"
}], "Text"],

Cell[BoxData[
    \(\(Ra[kd_] = \((\[Pi]\^2 + kd\^2)\)\^3\/kd\^2;\)\)], "Input"],

Cell[BoxData[
    \(\(Plot[Ra[kd], {kd, 0.1, 5}, PlotRange \[Rule] {0, 2000}, 
          AxesLabel \[Rule] {\*"\"\<\!\(k\_x\)d\>\"", "\<Ra\>"}, 
          PlotStyle \[Rule] {Text["\<unstable\>", {2.3, 1500}], 
              Text["\<stable\>", {2.3, 
                  300}]}] // \[IndentingNewLine]Figure["\<convection4\>", 
          ImageSize \[Rule] 200];\)\)], "Input"],

Cell["Critical Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra\_c = \(27\/4\) \[Pi]\^4 // N\)], "Input"],

Cell[TextData[{
  "Minimum ",
  Cell[BoxData[
      \(TraditionalForm\`\(k\_x\) d\)]]
}], "Text"],

Cell[BoxData[
    \(\[Pi]\/\@2 // N\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Temperature field", "Section"],

Cell[BoxData[
    \(k\_x = \[Pi]\/\(d \@ 2\); \ k\_z = \[Pi]\/d;\)], "Input"],

Cell["Temperature excess (apart from a constant factor)", "Text"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = Cos[\(k\_x\) x] Cos[\(k\_z\) z];\)\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        ContourPlot[\[CapitalDelta]T /. d \[Rule] 1, {x, 0, 
            5 \@ 2}, {z, \(-\(1\/2\)\), 1\/2}, PlotPoints \[Rule] 100, 
          AspectRatio \[Rule] \(1/5\)/\@2, FrameTicks \[Rule] None, 
          Contours \[Rule] 12, Frame \[Rule] False];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flow field", "Section"],

Cell["Stream function (apart from a constant factor)", "Text"],

Cell[BoxData[
    \(\(\[Psi] = Sin[\(k\_x\) x] Cos[k\_z\ z];\)\)], "Input"],

Cell[BoxData[
    \(\(figb = 
        ContourPlot[\[Psi] /. d \[Rule] 1, {x, 0, 5 \@ 2}, {z, \(-\(1\/2\)\), 
            1\/2}, PlotPoints \[Rule] 100, AspectRatio \[Rule] \(1/5\)/\@2, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shear", "Section"],

Cell["Fields with correct factors", "Text"],

Cell[BoxData[{
    \(\(\[CapitalDelta]T = \[CapitalTheta]\ Cos[\(k\_x\) x] 
          Cos[\(k\_z\) z];\)\), "\[IndentingNewLine]", 
    \(\ \(v\_x = \(\@2\) U\ Sin[\(k\_x\) x] 
          Sin[\(k\_z\) z];\)\), "\[IndentingNewLine]", 
    \(\(v\_z = U\ Cos[\(k\_x\) x] Cos[\(k\_z\) z];\)\)}], "Input"],

Cell["Shear stress", "Text"],

Cell[BoxData[
    \(\[Sigma]\_xz = \[Eta] \((\[PartialD]\_x v\_z + \[PartialD]\_z v\_x)\) // 
        Simplify\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Deformation", "Section"],

Cell["Pressure excess", "Text"],

Cell[BoxData[
    \(\(\[CapitalDelta]p = \(2\/\(3  \[Pi]\)\) \[Alpha]\ \[CapitalTheta]\ \(\
\[Rho]\_0\) g\_0\ d\ Cos[\(k\_x\) x] Sin[\(k\_z\) z];\)\)], "Input"],

Cell["Verify basic equation (31.40)", "Text"],

Cell[BoxData[
    \(\[PartialD]\_z\^2 \[CapitalDelta]p - \(\[Pi]\^2\/\(2 
                 d\^2\)\) \[CapitalDelta]p - \[Alpha]\ \(\[Rho]\_0\) \(g\_0\) \
\[PartialD]\_z \[CapitalDelta]T // Simplify\)], "Input"],

Cell["Velocity scale", "Text"],

Cell[BoxData[
    \(\(U = \(3  \[Pi]\^2\ \[Kappa]\ \[CapitalTheta]\)\/\(2\ G\ d\^2\);\)\)], \
"Input"],

Cell["Critical temperature gradient", "Text"],

Cell[BoxData[
    \(\(\[Eta] = \(\[Rho]\_0\) \[Nu];\)\)], "Input"],

Cell[BoxData[
    \(\(G = \(Ra\ \[Kappa]\ \[Nu]\)\/\(\[Alpha]\ \(g\_0\) d\^4\);\)\)], \
"Input"],

Cell["Critical Rayleigh number", "Text"],

Cell[BoxData[
    \(\(Ra = \(27\/4\) \[Pi]\^4;\)\)], "Input"],

Cell["Normal stress", "Text"],

Cell[BoxData[
    \(\[Sigma]\_zz = \(-\[CapitalDelta]p\) + 2  \[Eta]\ \[PartialD]\_z v\_z // 
        Simplify\)], "Input"],

Cell["Plot the shape", "Text"],

Cell[BoxData[
    \(\(area[x_, z_] = 
        With[{\[CapitalDelta]z = \[Epsilon]\ Cos[\(k\_x\) x]}, 
            If[z > \(-d\)/2 - \[CapitalDelta]z && z < 1/2 + \[CapitalDelta]z, 
              0.5, 1]] /. {d \[Rule] 1, \[Epsilon] \[Rule] 0.1};\)\)], "Input"],

Cell[BoxData[
    \(\(figc = 
        ContourPlot[area[x, z], {x, 0, 5 \@ 2}, {z, \(-0.7\), 0.7}, 
          PlotPoints \[Rule] 100, Contours \[Rule] 100, 
          ContourLines \[Rule] False, AspectRatio \[Rule] \(1/5\)/\@2, 
          Frame \[Rule] False, 
          ColorFunctionScaling \[Rule] False];\)\)], "Input"],

Cell["Plot the three panels", "Text"],

Cell[BoxData[{
    \(\(labela = Graphics[{Text["\<(a)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figla = Show[figa, labela];\)\)}], "Input"],

Cell[BoxData[{
    \(\(labelb = Graphics[{Text["\<(b)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figlb = Show[figb, labelb];\)\)}], "Input"],

Cell[BoxData[{
    \(\(labelc = Graphics[{Text["\<(c)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figlc = Show[figc, labelc];\)\)}], "Input"],

Cell[BoxData[
    \(\(Show[
          GraphicsArray[{{figla}, {figlb}, {figlc}}]] \
// \[IndentingNewLine]Figure["\<convection5\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rayleigh-Benard for  two solid boundaries (even)", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["In this analysis the coefficients are explicitly verified.", "Text"],

Cell[CellGroupData[{

Cell["Determinant of boundary conditions of (31.58)", "Section"],

Cell[BoxData[
    \(\(f[z_] = {Cos[\(\[Mu]\_0\) z], 
          Cosh[\(\[Mu]\_1\) z] Cos[\(\[Mu]\_2\) z], 
          Sinh[\(\[Mu]\_1\) z] Sin[\(\[Mu]\_2\) z]};\)\)], "Input"],

Cell[BoxData[
    \(\(m = {f[z], \(f''\)[z], \(f'''\)[z] - \(k\_x\^2\) \(f'\)[z]} /. 
          z \[Rule] 1/2;\)\)], "Input"],

Cell[BoxData[
    \(det = \(\(Det[m] // TrigExpand\) // Simplify\) // 
        Collect[#, {Cos[\[Mu]\_2], Cosh[\ \[Mu]\_1], Sinh[\ \[Mu]\_1], 
              Sin[\[Mu]\_2], Sin[\(\(\ \)\(\[Mu]\_0\)\)\/2], 
              Cos[\(\(\ \)\(\[Mu]\_0\)\)\/2]}] &\)], "Input"],

Cell[BoxData[
    \(KS0C1 = 
      Coefficient[det, \ Sin[\[Mu]\_0\/2] Cosh[\[Mu]\_1]] // 
        Simplify\)], "Input"],

Cell[BoxData[
    \(KS0C2 = 
      Coefficient[det, Sin[\[Mu]\_0\/2]\ Cos[\[Mu]\_2]] // 
        Simplify\)], "Input"],

Cell[BoxData[
    \(KC0S1 = 
      Coefficient[det, Cos[\[Mu]\_0\/2]\ Sinh[\[Mu]\_1]] // 
        Simplify\)], "Input"],

Cell[BoxData[
    \(KC0S2 = 
      Coefficient[det, Cos[\[Mu]\_0\/2]\ Sin[\[Mu]\_2]] // 
        Simplify\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Roots of (31.55)", "Section"],

Cell[BoxData[{
    \(\(\[Mu]\_0 = \[Mu]\ \[Xi] \@\( 1 - \[Xi]\^4\);\)\), \
"\[IndentingNewLine]", 
    \(\(\[Mu]\_1 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(1 + 2  \[Xi]\^4 + 2 \@\( 1 + \
\[Xi]\^4 + \[Xi]\^8\)\);\)\), "\[IndentingNewLine]", 
    \(\(\[Mu]\_2 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(\(-1\) - 2  \[Xi]\^4 + 2 \@\( 1 \
+ \[Xi]\^4 + \[Xi]\^8\)\);\)\)}], "Input"],

Cell[BoxData[{
    \(\(k\_x = \[Mu]\ \[Xi]\^3;\)\), "\[IndentingNewLine]", 
    \(\(Ra = \[Mu]\^4;\)\)}], "Input"],

Cell["Verify solutions", "Text"],

Cell[BoxData[
    \(\(zero = \((\[Lambda]\^2 - k\_x\^2)\)\^3 + Ra\ k\_x\^2;\)\)], "Input"],

Cell[BoxData[
    \(zero /. \[Lambda] \[Rule] \[ImaginaryI]\ \[Mu]\_0 // Expand\)], "Input"],

Cell[BoxData[
    \(zero /. \[Lambda] \[Rule] \[Mu]\_1 + \[ImaginaryI]\ \[Mu]\_2 // 
      Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Verify special forms of coefficients", "Section"],

Cell[BoxData[
    \(KS0C1 == 1\/4\ \(\@3\) \(\[Xi]\^4\) \(\[Mu]\^4\) \[Mu]\_0 // 
      Simplify[#, \[Xi] > 0] &\)], "Input"],

Cell[BoxData[
    \(KS0C2 \[Equal] KS0C1\)], "Input"],

Cell[BoxData[
    \(KC0S1 \[Equal] \(\[Xi]\^2\) \(\[Mu]\^2\) \(\[Mu]\_2\) \((\[Mu]\_1\^2 + \
\(\@3\) \(\[Mu]\_1\) \[Mu]\_2)\) // Simplify[#, \[Xi] > 0] &\)], "Input"],

Cell[BoxData[
    \(KC0S2 \[Equal] \(-\[Xi]\^2\) \(\[Mu]\^2\) \(\[Mu]\_1\) \((\[Mu]\_2\^2 - \
\(\@3\) \(\[Mu]\_1\) \[Mu]\_2)\) // Simplify[#, \[Xi] > 0] &\)], "Input"],

Cell["Common factor in the last two", "Text"],

Cell[BoxData[
    \(\(\[Mu]\_1\) \[Mu]\_2 // Simplify[#, \[Xi] > 0] &\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical values", "Section"],

Cell["Coefficient determinant", "Text"],

Cell[BoxData[
    \(\(det1[\[Mu]_?NumericQ, \[Xi]_?NumericQ] = \(\[Mu]\_0\) 
            Sin[\[Mu]\_0/2] \((Cosh[\[Mu]\_1] + Cos[\[Mu]\_2])\) + 
          Cos[\[Mu]\_0/
                2] \((\((\[Mu]\_1 + \(\@3\) \[Mu]\_2)\) 
                  Sinh[\[Mu]\_1] - \((\[Mu]\_2 - \(\@3\) \[Mu]\_1)\) 
                  Sin[\[Mu]\_2])\);\)\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        Plot[det1[\[Mu], 0.75`], {\[Mu], 0, 30}, 
            PlotRange \[Rule] {\(-75\), 75}, 
            AxesLabel \[Rule] {"\<\[Mu]\>", "\<\>"}, 
            PlotLabel \[Rule] \ "\<(a)\>"] \
// \[IndentingNewLine]Figure["\<convection6\>", 
            ImageSize \[Rule] 300];\)\)], "Input"],

Cell["Solution for vanishing determinant", "Text"],

Cell[BoxData[
    \(r[\[Xi]_?NumericQ] := \[Mu] /. 
        FindRoot[det1[\[Mu], \[Xi]] \[Equal] 0, {\[Mu], 9}]\)], "Input"],

Cell[BoxData[
    \(\(figb = 
        Plot[r[\[Xi]], {\[Xi], 0.5, 0.95}, PlotRange \[Rule] {6, 9}, 
            AxesLabel \[Rule] {"\<\[Xi]\>", "\<\[Mu]\>"}, 
            PlotStyle \[Rule] {Text["\<unstable\>", {0.8, 7}], 
                Text["\<stable\>", {0.8, 6.2}]}, 
            PlotLabel \[Rule] \ "\<(b)\>"] \
// \[IndentingNewLine]Figure["\<convection7\>", 
            ImageSize \[Rule] 300];\)\)], "Input"],

Cell["The minimal value", "Text"],

Cell[BoxData[
    \(root = FindMinimum[r[\[Xi]], {\[Xi], 0.85, 0.86}]\)], "Input"],

Cell["Critical Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra\_c = root[\([1]\)]\^4\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical solution", "Section"],

Cell["Reconstruct the critical parameters ", "Text"],

Cell[BoxData[
    \(sol = {\[Mu] \[Rule] 
          root[\([1]\)], \(root[\([2]\)]\)[\([1]\)]}\)], "Input"],

Cell[BoxData[
    \({\[Mu]\_0, \[Mu]\_1, \[Mu]\_2} /. sol\)], "Input"],

Cell["Critical boundary value matrix ", "Text"],

Cell[BoxData[
    \(\(m1 = m /. sol;\)\)], "Input"],

Cell[BoxData[
    \(m1 // MatrixForm\)], "Input"],

Cell["Verify rank 2", "Text"],

Cell[BoxData[
    \(m1 // MatrixRank\)], "Input"],

Cell["Reduce matrix", "Text"],

Cell[BoxData[
    \(m2 = RowReduce[m1, Tolerance \[Rule] 10\^\(-6\)]\)], "Input"],

Cell[BoxData[
    \(m2 // MatrixForm\)], "Input"],

Cell["Pick out coefficients", "Text"],

Cell[BoxData[
    \(A = \(-m2[\([All, 3]\)]\)\)], "Input"],

Cell["Insert special value", "Text"],

Cell[BoxData[
    \(\(A[\([3]\)] = 1;\)\)], "Input"],

Cell[BoxData[
    \(A\)], "Input"],

Cell["Normalized", "Text"],

Cell[BoxData[
    \(A/A[\([1]\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flow field", "Section"],

Cell["Critical function", "Text"],

Cell[BoxData[
    \(\(F[z_] = \((f[z] /. sol)\) . A;\)\)], "Input"],

Cell["Critical wave number", "Text"],

Cell[BoxData[
    \(k\_x = k\_x /. sol\)], "Input"],

Cell["Aspect ratio", "Text"],

Cell[BoxData[
    \(aspect = \ \[Pi]\/k\_x\)], "Input"],

Cell["Temperature field (apart from constant facto)", "Text"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = F[z]\ Cos[\(k\_x\) x];\)\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        ContourPlot[\[CapitalDelta]T, {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"],

Cell["Velocity field (apart from constant factor)", "Text"],

Cell[BoxData[
    \(\(v\_z = \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\) Cos[\(k\_x\) x] // 
          Simplify;\)\)], "Input"],

Cell[BoxData[
    \(\(v\_x = \ \(1\/k\_x\) 
          Sin[\(k\_x\) 
              x] \((\(F'''\)[z] - \(k\_x\^2\) \(F'\)[z])\);\)\)], "Input"],

Cell["Stream function (apart from constant factor)", "Text"],

Cell[BoxData[
    \(\(\[Psi] = 
        Sin[\(k\_x\) 
              x] \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\);\)\)], "Input"],

Cell[BoxData[
    \(\(figb = 
        ContourPlot[\[Psi], {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"],

Cell["Construct final figure", "Text"],

Cell[BoxData[{
    \(\(labela = Graphics[{Text["\<(a)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figla = Show[figa, labela];\)\)}], "Input"],

Cell[BoxData[{
    \(\(labelb = Graphics[{Text["\<(b)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figlb = Show[figb, labelb];\)\)}], "Input"],

Cell[BoxData[
    \(\(Show[
          GraphicsArray[{{figla}, {figlb}}]] \
// \[IndentingNewLine]Figure["\<convection8\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rayleigh-Benard for  two solid boundaries (odd)", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["This analysis is for completeness only.", "Text"],

Cell[CellGroupData[{

Cell["Roots of (31.55)", "Section"],

Cell[BoxData[{
    \(\(\[Mu]\_0 = \[Mu]\ \[Xi] \@\( 1 - \[Xi]\^4\);\)\), \
"\[IndentingNewLine]", 
    \(\(\[Mu]\_1 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(1 + 2  \[Xi]\^4 + 2 \@\( 1 + \
\[Xi]\^4 + \[Xi]\^8\)\);\)\), "\[IndentingNewLine]", 
    \(\(\[Mu]\_2 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(\(-1\) - 2  \[Xi]\^4 + 2 \@\( 1 \
+ \[Xi]\^4 + \[Xi]\^8\)\);\)\)}], "Input"],

Cell[BoxData[{
    \(\(k\_x = \[Mu]\ \[Xi]\^3;\)\), "\[IndentingNewLine]", 
    \(\(Ra = \[Mu]\^4;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Determinant of boundary conditions for  (31.66)", "Section"],

Cell[BoxData[
    \(\(f[z_] = {Sin[\(\[Mu]\_0\) z], 
          Cosh[\(\[Mu]\_1\) z] Sin[\(\[Mu]\_2\) z], 
          Sinh[\(\[Mu]\_1\) z] Cos[\(\[Mu]\_2\) z]};\)\)], "Input"],

Cell[BoxData[
    \(\(m = {f[z], \(f''\)[
              z], \(f'''\)[z] - \(\[Mu]\^2\) \(\[Xi]\^6\) \(f'\)[z]} /. 
          z \[Rule] 1/2;\)\)], "Input"],

Cell[BoxData[
    \(\(det[\[Mu]_, \[Xi]_] = Det[m]\/\[Mu]\^6;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical values", "Section"],

Cell[BoxData[
    \(\(Plot[det[\[Mu], 0.75], {\[Mu], 0, 20}, 
        PlotRange \[Rule] {\(-10\), 1}, 
        AxesLabel \[Rule] {"\<\[Mu]\>", "\<\>"}];\)\)], "Input"],

Cell[BoxData[
    \(r[\[Xi]_?NumericQ] := \[Mu] /. 
        FindRoot[det[\[Mu], \[Xi]] \[Equal] 0, {\[Mu], 12}]\)], "Input"],

Cell[BoxData[
    \(\(Plot[r[\[Xi]], {\[Xi], 0.7, 0.85}];\)\)], "Input"],

Cell["Determine critical parameters", "Text"],

Cell[BoxData[
    \(root = FindMinimum[r[\[Xi]], {\[Xi], 0.7, 0.8}]\)], "Input"],

Cell["Critical Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra\_c = root[\([1]\)]\^4\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical solution", "Section"],

Cell[BoxData[
    \(sol = {\[Mu] \[Rule] 
          root[\([1]\)], \(root[\([2]\)]\)[\([1]\)]}\)], "Input"],

Cell["Critical root values", "Text"],

Cell[BoxData[
    \({\[Mu]\_0, \[Mu]\_1, \[Mu]\_2} /. sol\)], "Input"],

Cell["Critical boundary matrix", "Text"],

Cell[BoxData[
    \(\(m1 = m /. sol;\)\)], "Input"],

Cell[BoxData[
    \(m1 // MatrixForm\)], "Input"],

Cell["Verify rank", "Text"],

Cell[BoxData[
    \(m1 // MatrixRank\)], "Input"],

Cell["Reduce matrix", "Text"],

Cell[BoxData[
    \(m2 = RowReduce[m1, Tolerance \[Rule] 10\^\(-6\)]\)], "Input"],

Cell[BoxData[
    \(m2 // MatrixForm\)], "Input"],

Cell["Determine normalized coefficients", "Text"],

Cell[BoxData[
    \(A = \(-m2[\([All, 3]\)]\)\)], "Input"],

Cell[BoxData[
    \(\(A[\([3]\)] = 1;\)\)], "Input"],

Cell[BoxData[
    \(A\)], "Input"],

Cell[BoxData[
    \(A/A[\([1]\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flow field", "Section"],

Cell[BoxData[
    \(\(F[z_] = \((f[z] /. sol)\) . A;\)\)], "Input"],

Cell[BoxData[
    \(k\_x = k\_x /. sol\)], "Input"],

Cell[BoxData[
    \(aspect = \ \[Pi]\/k\_x\)], "Input"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = F[z]\ Cos[\(k\_x\) x];\)\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        ContourPlot[\[CapitalDelta]T, {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    \(\(v\_z = \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\) Cos[\(k\_x\) x] // 
          Simplify;\)\)], "Input"],

Cell[BoxData[
    \(\(v\_x = \ \(1\/k\_x\) 
          Sin[\(k\_x\) 
              x] \((\(F'''\)[z] - \(k\_x\^2\) \(F'\)[z])\);\)\)], "Input"],

Cell[BoxData[
    \(\(\[Psi] = 
        Sin[\(k\_x\) 
              x] \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\);\)\)], "Input"],

Cell[BoxData[
    \(\(figb = 
        ContourPlot[\[Psi], {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rayleigh-Benard for one solid and one free boundaries", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["\<\
This analysis follows the same pattern as the preceding cases, but is \
slightly more complicated.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Roots of (31.55)", "Section"],

Cell[BoxData[{
    \(\(\[Mu]\_0 = \[Mu]\ \[Xi] \@\( 1 - \[Xi]\^4\);\)\), \
"\[IndentingNewLine]", 
    \(\(\[Mu]\_1 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(1 + 2  \[Xi]\^4 + 2 \@\( 1 + \
\[Xi]\^4 + \[Xi]\^8\)\);\)\), "\[IndentingNewLine]", 
    \(\(\[Mu]\_2 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(\(-1\) - 2  \[Xi]\^4 + 2 \@\( 1 \
+ \[Xi]\^4 + \[Xi]\^8\)\);\)\)}], "Input"],

Cell[BoxData[{
    \(\(k\_x = \[Mu]\ \[Xi]\^3;\)\), "\[IndentingNewLine]", 
    \(\(Ra = \[Mu]\^4;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Determinant of boundary conditions of (31.64)", "Section"],

Cell[BoxData[{
    \(\(f[z_] = {Cos[\(\[Mu]\_0\) z], 
          Cosh[\(\[Mu]\_1\) z] Cos[\(\[Mu]\_2\) z], 
          Sinh[\(\[Mu]\_1\) z] Sin[\(\[Mu]\_2\) z], Sin[\(\[Mu]\_0\) z], 
          Cosh[\(\[Mu]\_1\) z] Sin[\(\[Mu]\_2\) z], 
          Sinh[\(\[Mu]\_1\) z] 
            Cos[\(\[Mu]\_2\) z]};\)\), "\[IndentingNewLine]", 
    \(\(mt = {f[z], \(f''\)[z], \(f''''\)[z]} /. 
          z \[Rule] 1/2;\)\), "\[IndentingNewLine]", 
    \(\(mb = {f[z], \(f''\)[
              z], \(f'''\)[z] - \(\[Mu]\^2\) \(\[Xi]\^6\) \(f'\)[z]} /. 
          z \[Rule] \(-1\)/2;\)\)}], "Input"],

Cell[BoxData[
    \(\(m = mt \[Union] mb;\)\)], "Input"],

Cell[BoxData[
    \(\(det[\[Mu]_, \[Xi]_] = Det[m]\/\[Mu]\^6;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical values", "Section"],

Cell[BoxData[
    \(\(figa = 
        Plot[det[\[Mu], 0.77], {\[Mu], 0, 10}, 
          PlotRange -> {\(-3\)\ 10\^5, 5\ 10\^5}];\)\)], "Input"],

Cell[BoxData[
    \(r[\[Xi]_?NumericQ] := \[Mu] /. 
        FindRoot[det[\[Mu], \[Xi]] \[Equal] 0, {\[Mu], 8}]\)], "Input"],

Cell[BoxData[
    \(\(figb = Plot[r[\[Xi]], {\[Xi], 0.6, 0.9}];\)\)], "Input"],

Cell[BoxData[
    \(root = FindMinimum[r[\[Xi]], {\[Xi], 0.85, 0.86}]\)], "Input"],

Cell[BoxData[
    \(Ra\_c = root[\([1]\)]\^4\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical solution", "Section"],

Cell[BoxData[
    \(sol = {\[Mu] \[Rule] 
          root[\([1]\)], \(root[\([2]\)]\)[\([1]\)]}\)], "Input"],

Cell[BoxData[
    \({\[Mu]\_0, \[Mu]\_1, \[Mu]\_2} /. sol\)], "Input"],

Cell[BoxData[
    \(\(m1 = m /. sol;\)\)], "Input"],

Cell[BoxData[
    \(m1 // MatrixForm\)], "Input"],

Cell[BoxData[
    \(m1 // MatrixRank\)], "Input"],

Cell[BoxData[
    \(m2 = RowReduce[m1, Tolerance \[Rule] 10\^\(-6\)]\)], "Input"],

Cell[BoxData[
    \(m2 // MatrixForm\)], "Input"],

Cell[BoxData[
    \(A = \(-m2[\([All, 6]\)]\)\)], "Input"],

Cell[BoxData[
    \(\(A[\([6]\)] = 1;\)\)], "Input"],

Cell[BoxData[
    \(A\)], "Input"],

Cell[BoxData[
    \(A /= A[\([1]\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flow field", "Section"],

Cell[BoxData[
    \(\(F[z_] = \((f[z] /. sol)\) . A;\)\)], "Input"],

Cell[BoxData[
    \(k\_x = k\_x /. sol\)], "Input"],

Cell[BoxData[
    \(aspect = \ \[Pi]\/k\_x /. sol\)], "Input"],

Cell[BoxData[
    \(\(\[CapitalDelta]T = \ F[z]\ Cos[\(k\_x\) x];\)\)], "Input"],

Cell[BoxData[
    \(\(figa = 
        ContourPlot[\[CapitalDelta]T, {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    \(\(v\_z = \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\) 
          Cos[\(k\_x\) x];\)\)], "Input"],

Cell[BoxData[
    \(\(v\_x = \ \(-\(1\/k\_x\)\) 
          Sin[\(k\_x\) 
              x] \((\(-\(F'''\)[z]\) + \(k\_x\^2\) \(F'\)[z])\);\)\)], "Input"],

Cell[BoxData[
    \(\(\[Psi] = 
        Sin[\(k\_x\) 
              x] \((\(-\(F''\)[z]\) + \(k\_x\^2\) F[z])\);\)\)], "Input"],

Cell[BoxData[
    \(\(figb = 
        ContourPlot[\[Psi], {x, 0, 5\ aspect}, {z, \(-0.5\), 0.5}, 
          PlotPoints \[Rule] 50, AspectRatio \[Rule] \(1/5\)/aspect, 
          FrameTicks \[Rule] None, Contours \[Rule] 12, 
          Frame \[Rule] False];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Deformation ", "Section"],

Cell[BoxData[
    \(\(\[CapitalDelta]p = \ \ \(-\(1\/k\_x\^2\)\) \((\(F'''''\)[z] - 
              2 \( k\_x\^2\) \(F'''\)[z] + \(k\_x\^4\) \(F'\)[z])\) 
          Cos[\(k\_x\) x];\)\)], "Input"],

Cell[BoxData[
    \(\[PartialD]\_z\^2 \[CapitalDelta]p - \(k\_x\^2\) \[CapitalDelta]p - \ 
          Ra\_c\ \[PartialD]\_z \[CapitalDelta]T /. sol // 
      Simplify\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]\[Sigma]\_zz = \(-\[CapitalDelta]p\) + 
            2\ \[PartialD]\_z v\_z /. sol // Simplify\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]zu = \(-\[CapitalDelta]\[Sigma]\_zz\) /. 
        z \[Rule] 1/2\)], "Input"],

Cell[BoxData[
    \(\(area[x_, z_] = 
        With[{\[CapitalDelta]z = \[Epsilon]\ Cos[\(k\_x\) x]}, 
            If[z > \(-1\)/2 && z < 1/2 + \[CapitalDelta]z, 0.5, 
              1]] /. {\[Epsilon] \[Rule] 0.1};\)\)], "Input"],

Cell[BoxData[
    \(\(figc = 
        ContourPlot[area[x, z], {x, 0, 5  aspect}, {z, \(-0.7\), 0.7}, 
          PlotPoints \[Rule] 100, Contours \[Rule] 100, 
          ContourLines \[Rule] False, AspectRatio \[Rule] \(1/5\)/aspect, 
          Frame \[Rule] False, 
          ColorFunctionScaling \[Rule] False];\)\)], "Input"],

Cell[BoxData[{
    \(\(labela = Graphics[{Text["\<(a)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figla = Show[figa, labela];\)\)}], "Input"],

Cell[BoxData[{
    \(\(labelb = Graphics[{Text["\<(b)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figlb = Show[figb, labelb];\)\)}], "Input"],

Cell[BoxData[{
    \(\(labelc = Graphics[{Text["\<(c)\>", {\(-0.15\), 0}]}];\)\), "\n", 
    \(\(figlc = Show[figc, labelc];\)\)}], "Input"],

Cell[BoxData[
    \(\(Show[
          GraphicsArray[{{figla}, {figlb}, {figlc}}]] \
// \[IndentingNewLine]Figure["\<convection9\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Title"],

Cell[CellGroupData[{

Cell["31.1.1 and 31.1.2 Vertical slot with water", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(d = 1  Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(\[CapitalTheta] = 10\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 
        2.5\ \(10\^\(-4\)\) Kelvin\^\(-1\);\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 
        8.64\ \(10\^\(-7\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Kappa] = 1.4\ \(10\^\(-7\)\) Meter\^2\/Second;\)\)}], "Input"],

Cell["Terminal velocity", "Text"],

Cell[BoxData[
    \(U = \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) d\^2\)\/\(24\ \[Nu]\) // 
        ToUnit[Centimeter/Second]\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Reynolds = \(U\ d\)\/\[Nu] // ToSI\)], "Input"],

Cell["Viscous entrance length", "Text"],

Cell[BoxData[
    \(viscouslength = 0.02*Reynolds\ d // ToSI\)], "Input"],

Cell["Rayleigh number", "Text"],

Cell[BoxData[
    \(Rayleigh = \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) d\^3\)\/\(\[Nu]\ \
\[Kappa]\) // ToSI\)], "Input"],

Cell["Heat entrance lenth", "Text"],

Cell[BoxData[
    \(heatlength = \(Rayleigh\/96\) d // ToSI\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["31.1.1 and 31.1.2 Vertical slot with air", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[" Parameters", "Text"],

Cell[BoxData[{
    \(\(d = 1  Centimeter;\)\), "\n", 
    \(\(\[CapitalTheta] = 10\ Kelvin;\)\), "\n", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\n", 
    \(\(\[Alpha] = 3\ \(10\^\(-3\)\) Kelvin\^\(-1\);\)\), "\n", 
    \(\(\[Nu] = 1.57\ \(10\^\(-5\)\) Meter\^2\/Second;\)\), "\n", 
    \(\(\[Kappa] = 2\ \(10\^\(-5\)\) Meter\^2\/Second;\)\)}], "Input"],

Cell["Terminal velocity", "Text"],

Cell[BoxData[
    \(U = \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) d\^2\)\/\(24\ \[Nu]\) // 
        ToUnit[Centimeter/Second]\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Reynolds = \(U\ d\)\/\[Nu] // ToSI\)], "Input"],

Cell["Viscous entrance length", "Text"],

Cell[BoxData[
    \(viscouslength = 0.02*Reynolds\ d // ToUnit[Centimeter]\)], "Input"],

Cell["Rayleigh number", "Text"],

Cell[BoxData[
    \(Rayleigh = \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) d\^3\)\/\(\[Nu]\ \
\[Kappa]\) // ToSI\)], "Input"],

Cell["Heat entrance lenth", "Text"],

Cell[BoxData[
    \(heatlength = \(Rayleigh\/96\) d // ToUnit[Centimeter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["31.1.3 Heat radiator in air", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(Fp = \(-0.388\);\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = \(\(1\)\(\ \)\)\/293\ Kelvin\^\(-1\);\)\), "\
\[IndentingNewLine]", 
    \(\(\[CapitalTheta] = 45\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(h = 70\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(L = 1\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(k = 0.025\ Watt\/\(Kelvin\ Meter\);\)\), "\[IndentingNewLine]", 
    \(\(\[Kappa] = 
        2\ \(10\^\(-5\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 1.57\ \(10\^\(-5\)\) Meter\^2\/Second;\)\)}], "Input"],

Cell["Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra\  = \ \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) h\^3\)\/\(\[Kappa]\ \
\[Nu]\) // ToSI\)], "Input"],

Cell["Heat loss", "Text"],

Cell[BoxData[
    \(dotQ = \(-\(4\/3\)\) Fp\ \ Ra\^\(1/4\)\ k\ \[CapitalTheta]\ L // 
        ToUnit[Watt]\)], "Input"],

Cell[BoxData[
    \(2\ dotQ\)], "Input"],

Cell["Maximal velocity", "Text"],

Cell[BoxData[
    \(U = 
      0.5\ \@\(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) h\) // 
        ToUnit[Centimeter]\)], "Input"],

Cell["Boundary layer thickness", "Text"],

Cell[BoxData[
    \(\[Delta] = 
      3 \((\(\[Alpha]\ \[CapitalTheta]\ g\_0\)\/\(\[Kappa]\ \[Nu]\ h\))\)\^\(\
\(-1\)/4\) // ToUnit[Centimeter]\)], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData[
    \(Re\_\[Delta] = \ \(U\ \[Delta]\)\/\[Nu] // ToSI\)], "Input"],

Cell["Horizontal inflow", "Text"],

Cell[BoxData[
    \(\(d = 5  Centimeter;\)\)], "Input"],

Cell[BoxData[
    \(v\_x = \((\(\[Alpha]\ \[CapitalTheta]\ \[Kappa]\ \[Nu]\ g\_0\)\/\(\(\ \
\)\(d\)\))\)\^\(1/4\) // ToUnit[Centimeter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Heat flow from human in air", "Section"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(Fp = \(-0.388\);\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = \(\(1\)\(\ \)\)\/293\ Kelvin\^\(-1\);\)\), "\
\[IndentingNewLine]", 
    \(\(\[CapitalTheta] = 10\ Kelvin;\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(A = 2\ Meter\^2;\)\), "\[IndentingNewLine]", 
    \(\(h = 2\ Meter;\)\), "\[IndentingNewLine]", 
    \(\(L = A\/h\ ;\)\), "\[IndentingNewLine]", 
    \(\(k = 0.025\ Watt\/\(Kelvin\ Meter\);\)\), "\[IndentingNewLine]", 
    \(\(\[Kappa] = 
        2\ \(10\^\(-5\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 1.57\ \(10\^\(-5\)\) Meter\^2\/Second;\)\)}], "Input"],

Cell["Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra\  = \ \(\[Alpha]\ \[CapitalTheta]\ \(g\_0\) h\^3\)\/\(\[Kappa]\ \
\[Nu]\) // ToSI\)], "Input"],

Cell["Heat loss", "Text"],

Cell[BoxData[
    \(dotQ = \(-\(4\/3\)\) Fp\ \ Ra\^\(1/4\)\ k\ \[CapitalTheta]\ L // 
        ToUnit[Watt]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["31.2.1 Pot of water", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(G = 300\ Kelvin/Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 
        2.5\ \(10\^\(-4\)\) Kelvin\^\(-1\);\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Nu] = 
        8.64\ \(10\^\(-7\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(\[Kappa] = 
        1.4\ \(10\^\(-7\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(Ra\_c = 1080;\)\)}], "Input"],

Cell["Rayleigh number", "Text"],

Cell[BoxData[
    \(Ra = \(\(\ \)\(\(g\_0\) \[Alpha]\ G\ h\^4\)\)\/\(\[Kappa]\ \[Nu]\) // 
        ToSI\)], "Input"],

Cell["Critical diameter", "Text"],

Cell[BoxData[
    \(d = \((Ra\_c\ \(\[Kappa]\ \[Nu]\)\/\(\(g\_0\) \[Alpha]\ G\))\)\^\(1/4\) \
// ToUnit[Millimeter]\)], "Input"],

Cell["Terminal speed", "Text"],

Cell[BoxData[
    \(U = \(2\/45\) \(\(g\_0\) \[Alpha]\ G\ d\^3\)\/\[Kappa] // 
        ToUnit[Centimeter]\)], "Input"],

Cell["Porridge parameters", "Text"],

Cell[BoxData[
    \(\(\[Nu] = 1\ Meter\^2/Second;\)\)], "Input"],

Cell["Critical diameter", "Text"],

Cell[BoxData[
    \(d = \((Ra\_c\ \(\[Kappa]\ \[Nu]\)\/\(\(g\_0\) \[Alpha]\ G\))\)\^\(1/4\) \
// ToUnit[Centimeter]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["31.2.2 Terminal speed", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell["Parameters", "Text"],

Cell[BoxData[{
    \(\(d = 1\ Centimeter;\)\), "\[IndentingNewLine]", 
    \(\(G = 300\ Kelvin/Meter;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha] = 
        2.5\ \(10\^\(-4\)\) Kelvin\^\(-1\);\)\), "\[IndentingNewLine]", 
    \(\(g\_0 = AccelerationDueToGravity;\)\), "\[IndentingNewLine]", 
    \(\(\[Kappa] = 
        1.4\ \(10\^\(-7\)\) Meter\^2\/Second;\)\), "\[IndentingNewLine]", 
    \(\(Ra\_c = 1080;\)\)}], "Input"],

Cell["Terminal speed", "Text"],

Cell[BoxData[
    \(U = \(2\/45\) \(\(g\_0\) \[Alpha]\ G\ d\^3\)\/\[Kappa] // 
        ToUnit[Centimeter]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Title"],

Cell[CellGroupData[{

Cell["31.4 Approximative Rayleigh-Benard", "Section"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[BoxData[{
    \(\(\[Mu]\_0 = \[Mu]\ \[Xi] \@\( 1 - \[Xi]\^4\);\)\), \
"\[IndentingNewLine]", 
    \(\(\[Mu]\_1 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(1 + 2  \[Xi]\^4 + 2 \@\( 1 + \
\[Xi]\^4 + \[Xi]\^8\)\);\)\), "\[IndentingNewLine]", 
    \(\(\[Mu]\_2 = \(1\/2\) \[Mu]\ \[Xi]\ \@\(\(-1\) - 2  \[Xi]\^4 + 2 \@\( 1 \
+ \[Xi]\^4 + \[Xi]\^8\)\);\)\)}], "Input"],

Cell[BoxData[
    \(\[Mu][\[Xi]_] = 
      2 \( \[Mu]\/\[Mu]\_0\) \((\[Pi] - 
              ArcTan[\(\[Mu]\_1 + \(\@3\) \[Mu]\_2\)\/\[Mu]\_0])\) // 
        Simplify\)], "Input"],

Cell[BoxData[
    \(\(Plot[\[Mu][\[Xi]], {\[Xi], 0.5, 0.95}, 
        PlotRange \[Rule] {6, 8}];\)\)], "Input"],

Cell[BoxData[
    \(root = FindMinimum[\[Mu][\[Xi]], {\[Xi], 0.85, 0.86}]\)], "Input"],

Cell[BoxData[
    \(Ra\_c = root[\([1]\)]\^4\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{1051, 740},
WindowMargins->{{78, Automatic}, {Automatic, 38}},
StyleDefinitions -> "Default.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 59, 1, 95, "Title",
  InitializationCell->True],
Cell[1838, 56, 129, 3, 50, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[2004, 64, 49, 0, 95, "Title"],
Cell[2056, 66, 46, 1, 30, "Input"],
Cell[2105, 69, 226, 4, 51, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2368, 78, 39, 0, 95, "Title"],
Cell[2410, 80, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2481, 85, 50, 0, 73, "Section"],
Cell[2534, 87, 126, 2, 44, "Input"],
Cell[2663, 91, 58, 1, 30, "Input"],
Cell[2724, 94, 77, 1, 30, "Input"],
Cell[2804, 97, 84, 1, 30, "Input"],
Cell[2891, 100, 114, 2, 32, "Input"],
Cell[3008, 104, 132, 2, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3177, 111, 41, 0, 73, "Section"],
Cell[3221, 113, 106, 2, 30, "Input"],
Cell[3330, 117, 33, 0, 33, "Text"],
Cell[3366, 119, 88, 1, 30, "Input"],
Cell[3457, 122, 111, 3, 35, "Input"],
Cell[3571, 127, 29, 0, 33, "Text"],
Cell[3603, 129, 188, 3, 30, "Input"],
Cell[3794, 134, 147, 2, 35, "Input"],
Cell[3944, 138, 38, 0, 33, "Text"],
Cell[3985, 140, 184, 3, 30, "Input"],
Cell[4172, 145, 132, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4341, 152, 52, 0, 73, "Section"],
Cell[4396, 154, 69, 1, 42, "Input"],
Cell[4468, 157, 54, 1, 30, "Input"],
Cell[4525, 160, 54, 1, 30, "Input"],
Cell[4582, 163, 54, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4673, 169, 49, 0, 73, "Section"],
Cell[4725, 171, 93, 2, 33, "Text"],
Cell[4821, 175, 1050, 14, 290, "Input"],
Cell[5874, 191, 51, 0, 33, "Text"],
Cell[5928, 193, 68, 1, 30, "Input"],
Cell[5999, 196, 87, 1, 30, "Input"],
Cell[6089, 199, 331, 6, 70, "Input"],
Cell[6423, 207, 42, 0, 33, "Text"],
Cell[6468, 209, 67, 1, 30, "Input"],
Cell[6538, 212, 87, 1, 30, "Input"],
Cell[6628, 215, 37, 0, 33, "Text"],
Cell[6668, 217, 69, 1, 30, "Input"],
Cell[6740, 220, 79, 1, 30, "Input"],
Cell[6822, 223, 84, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6943, 229, 38, 0, 73, "Section"],
Cell[6984, 231, 46, 1, 30, "Input"],
Cell[7033, 234, 75, 0, 33, "Text"],
Cell[7111, 236, 207, 4, 90, "Input"],
Cell[7321, 242, 38, 0, 33, "Text"],
Cell[7362, 244, 1154, 31, 121, "Input"],
Cell[8519, 277, 32, 0, 33, "Text"],
Cell[8554, 279, 389, 8, 50, "Input"],
Cell[8946, 289, 30, 0, 33, "Text"],
Cell[8979, 291, 179, 3, 30, "Input"],
Cell[9161, 296, 60, 1, 30, "Input"],
Cell[9224, 299, 31, 0, 33, "Text"],
Cell[9258, 301, 79, 1, 30, "Input"],
Cell[9340, 304, 68, 1, 42, "Input"],
Cell[9411, 307, 473, 8, 70, "Input"],
Cell[9887, 317, 38, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9974, 324, 57, 0, 95, "Title"],
Cell[10034, 326, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[10105, 331, 35, 0, 73, "Section"],
Cell[10143, 333, 226, 8, 33, "Text"],
Cell[10372, 343, 80, 1, 46, "Input"],
Cell[10455, 346, 373, 6, 70, "Input"],
Cell[10831, 354, 40, 0, 33, "Text"],
Cell[10874, 356, 64, 1, 42, "Input"],
Cell[10941, 359, 97, 4, 33, "Text"],
Cell[11041, 365, 48, 1, 46, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11126, 371, 36, 0, 73, "Section"],
Cell[11165, 373, 77, 1, 46, "Input"],
Cell[11245, 376, 65, 0, 33, "Text"],
Cell[11313, 378, 88, 1, 30, "Input"],
Cell[11404, 381, 299, 5, 64, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11740, 391, 29, 0, 73, "Section"],
Cell[11772, 393, 62, 0, 33, "Text"],
Cell[11837, 395, 75, 1, 30, "Input"],
Cell[11915, 398, 289, 5, 64, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12241, 408, 24, 0, 73, "Section"],
Cell[12268, 410, 43, 0, 33, "Text"],
Cell[12314, 412, 299, 5, 72, "Input"],
Cell[12616, 419, 28, 0, 33, "Text"],
Cell[12647, 421, 123, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12807, 428, 30, 0, 73, "Section"],
Cell[12840, 430, 31, 0, 33, "Text"],
Cell[12874, 432, 160, 2, 43, "Input"],
Cell[13037, 436, 45, 0, 33, "Text"],
Cell[13085, 438, 210, 3, 46, "Input"],
Cell[13298, 443, 30, 0, 33, "Text"],
Cell[13331, 445, 102, 2, 46, "Input"],
Cell[13436, 449, 45, 0, 33, "Text"],
Cell[13484, 451, 66, 1, 30, "Input"],
Cell[13553, 454, 96, 2, 44, "Input"],
Cell[13652, 458, 40, 0, 33, "Text"],
Cell[13695, 460, 61, 1, 42, "Input"],
Cell[13759, 463, 29, 0, 33, "Text"],
Cell[13791, 465, 123, 2, 30, "Input"],
Cell[13917, 469, 30, 0, 33, "Text"],
Cell[13950, 471, 260, 4, 30, "Input"],
Cell[14213, 477, 321, 6, 59, "Input"],
Cell[14537, 485, 37, 0, 33, "Text"],
Cell[14577, 487, 140, 2, 50, "Input"],
Cell[14720, 491, 140, 2, 50, "Input"],
Cell[14863, 495, 140, 2, 50, "Input"],
Cell[15006, 499, 147, 3, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15202, 508, 65, 0, 95, "Title"],
Cell[15270, 510, 46, 1, 30, "Input"],
Cell[15319, 513, 74, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[15418, 517, 64, 0, 73, "Section"],
Cell[15485, 519, 173, 3, 30, "Input"],
Cell[15661, 524, 125, 2, 31, "Input"],
Cell[15789, 528, 266, 4, 40, "Input"],
Cell[16058, 534, 120, 3, 40, "Input"],
Cell[16181, 539, 118, 3, 40, "Input"],
Cell[16302, 544, 119, 3, 40, "Input"],
Cell[16424, 549, 118, 3, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16579, 557, 35, 0, 73, "Section"],
Cell[16617, 559, 352, 6, 108, "Input"],
Cell[16972, 567, 114, 2, 50, "Input"],
Cell[17089, 571, 32, 0, 33, "Text"],
Cell[17124, 573, 90, 1, 32, "Input"],
Cell[17217, 576, 92, 1, 30, "Input"],
Cell[17312, 579, 110, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17459, 586, 55, 0, 73, "Section"],
Cell[17517, 588, 125, 2, 42, "Input"],
Cell[17645, 592, 53, 1, 30, "Input"],
Cell[17701, 595, 166, 2, 33, "Input"],
Cell[17870, 599, 167, 2, 33, "Input"],
Cell[18040, 603, 45, 0, 33, "Text"],
Cell[18088, 605, 82, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18207, 611, 34, 0, 73, "Section"],
Cell[18244, 613, 39, 0, 33, "Text"],
Cell[18286, 615, 346, 6, 33, "Input"],
Cell[18635, 623, 324, 7, 50, "Input"],
Cell[18962, 632, 50, 0, 33, "Text"],
Cell[19015, 634, 124, 2, 30, "Input"],
Cell[19142, 638, 417, 8, 70, "Input"],
Cell[19562, 648, 33, 0, 33, "Text"],
Cell[19598, 650, 82, 1, 30, "Input"],
Cell[19683, 653, 40, 0, 33, "Text"],
Cell[19726, 655, 57, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19820, 661, 36, 0, 73, "Section"],
Cell[19859, 663, 52, 0, 33, "Text"],
Cell[19914, 665, 107, 2, 30, "Input"],
Cell[20024, 669, 70, 1, 30, "Input"],
Cell[20097, 672, 47, 0, 33, "Text"],
Cell[20147, 674, 51, 1, 30, "Input"],
Cell[20201, 677, 49, 1, 30, "Input"],
Cell[20253, 680, 29, 0, 33, "Text"],
Cell[20285, 682, 49, 1, 30, "Input"],
Cell[20337, 685, 29, 0, 33, "Text"],
Cell[20369, 687, 81, 1, 31, "Input"],
Cell[20453, 690, 49, 1, 30, "Input"],
Cell[20505, 693, 37, 0, 33, "Text"],
Cell[20545, 695, 58, 1, 30, "Input"],
Cell[20606, 698, 36, 0, 33, "Text"],
Cell[20645, 700, 52, 1, 30, "Input"],
Cell[20700, 703, 34, 1, 30, "Input"],
Cell[20737, 706, 26, 0, 33, "Text"],
Cell[20766, 708, 45, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20848, 714, 29, 0, 73, "Section"],
Cell[20880, 716, 33, 0, 33, "Text"],
Cell[20916, 718, 67, 1, 30, "Input"],
Cell[20986, 721, 36, 0, 33, "Text"],
Cell[21025, 723, 51, 1, 30, "Input"],
Cell[21079, 726, 28, 0, 33, "Text"],
Cell[21110, 728, 55, 1, 42, "Input"],
Cell[21168, 731, 61, 0, 33, "Text"],
Cell[21232, 733, 78, 1, 30, "Input"],
Cell[21313, 736, 281, 5, 50, "Input"],
Cell[21597, 743, 59, 0, 33, "Text"],
Cell[21659, 745, 124, 2, 31, "Input"],
Cell[21786, 749, 142, 3, 44, "Input"],
Cell[21931, 754, 60, 0, 33, "Text"],
Cell[21994, 756, 127, 3, 31, "Input"],
Cell[22124, 761, 271, 5, 50, "Input"],
Cell[22398, 768, 38, 0, 33, "Text"],
Cell[22439, 770, 140, 2, 50, "Input"],
Cell[22582, 774, 140, 2, 50, "Input"],
Cell[22725, 778, 138, 3, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22912, 787, 64, 0, 95, "Title"],
Cell[22979, 789, 46, 1, 30, "Input"],
Cell[23028, 792, 55, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[23108, 796, 35, 0, 73, "Section"],
Cell[23146, 798, 352, 6, 108, "Input"],
Cell[23501, 806, 114, 2, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23652, 813, 66, 0, 73, "Section"],
Cell[23721, 815, 173, 3, 30, "Input"],
Cell[23897, 820, 154, 3, 31, "Input"],
Cell[24054, 825, 76, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24167, 831, 34, 0, 73, "Section"],
Cell[24204, 833, 167, 3, 30, "Input"],
Cell[24374, 838, 124, 2, 30, "Input"],
Cell[24501, 842, 72, 1, 30, "Input"],
Cell[24576, 845, 45, 0, 33, "Text"],
Cell[24624, 847, 80, 1, 30, "Input"],
Cell[24707, 850, 40, 0, 33, "Text"],
Cell[24750, 852, 57, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24844, 858, 36, 0, 73, "Section"],
Cell[24883, 860, 107, 2, 30, "Input"],
Cell[24993, 864, 36, 0, 33, "Text"],
Cell[25032, 866, 70, 1, 30, "Input"],
Cell[25105, 869, 40, 0, 33, "Text"],
Cell[25148, 871, 51, 1, 30, "Input"],
Cell[25202, 874, 49, 1, 30, "Input"],
Cell[25254, 877, 27, 0, 33, "Text"],
Cell[25284, 879, 49, 1, 30, "Input"],
Cell[25336, 882, 29, 0, 33, "Text"],
Cell[25368, 884, 81, 1, 31, "Input"],
Cell[25452, 887, 49, 1, 30, "Input"],
Cell[25504, 890, 49, 0, 33, "Text"],
Cell[25556, 892, 58, 1, 30, "Input"],
Cell[25617, 895, 52, 1, 30, "Input"],
Cell[25672, 898, 34, 1, 30, "Input"],
Cell[25709, 901, 45, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25791, 907, 29, 0, 73, "Section"],
Cell[25823, 909, 67, 1, 30, "Input"],
Cell[25893, 912, 51, 1, 30, "Input"],
Cell[25947, 915, 55, 1, 42, "Input"],
Cell[26005, 918, 78, 1, 30, "Input"],
Cell[26086, 921, 281, 5, 50, "Input"],
Cell[26370, 928, 124, 2, 31, "Input"],
Cell[26497, 932, 142, 3, 44, "Input"],
Cell[26642, 937, 127, 3, 31, "Input"],
Cell[26772, 942, 271, 5, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27092, 953, 70, 0, 95, "Title"],
Cell[27165, 955, 46, 1, 30, "Input"],
Cell[27214, 958, 122, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[27361, 965, 35, 0, 73, "Section"],
Cell[27399, 967, 352, 6, 108, "Input"],
Cell[27754, 975, 114, 2, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27905, 982, 64, 0, 73, "Section"],
Cell[27972, 984, 580, 11, 71, "Input"],
Cell[28555, 997, 56, 1, 30, "Input"],
Cell[28614, 1000, 76, 1, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28727, 1006, 34, 0, 73, "Section"],
Cell[28764, 1008, 143, 3, 31, "Input"],
Cell[28910, 1013, 123, 2, 30, "Input"],
Cell[29036, 1017, 78, 1, 30, "Input"],
Cell[29117, 1020, 82, 1, 30, "Input"],
Cell[29202, 1023, 57, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29296, 1029, 36, 0, 73, "Section"],
Cell[29335, 1031, 107, 2, 30, "Input"],
Cell[29445, 1035, 70, 1, 30, "Input"],
Cell[29518, 1038, 51, 1, 30, "Input"],
Cell[29572, 1041, 49, 1, 30, "Input"],
Cell[29624, 1044, 49, 1, 30, "Input"],
Cell[29676, 1047, 81, 1, 31, "Input"],
Cell[29760, 1050, 49, 1, 30, "Input"],
Cell[29812, 1053, 58, 1, 30, "Input"],
Cell[29873, 1056, 52, 1, 30, "Input"],
Cell[29928, 1059, 34, 1, 30, "Input"],
Cell[29965, 1062, 48, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30050, 1068, 29, 0, 73, "Section"],
Cell[30082, 1070, 67, 1, 30, "Input"],
Cell[30152, 1073, 51, 1, 30, "Input"],
Cell[30206, 1076, 62, 1, 42, "Input"],
Cell[30271, 1079, 80, 1, 30, "Input"],
Cell[30354, 1082, 281, 5, 50, "Input"],
Cell[30638, 1089, 112, 2, 31, "Input"],
Cell[30753, 1093, 152, 3, 44, "Input"],
Cell[30908, 1098, 127, 3, 31, "Input"],
Cell[31038, 1103, 271, 5, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31346, 1113, 31, 0, 73, "Section"],
Cell[31380, 1115, 195, 3, 44, "Input"],
Cell[31578, 1120, 178, 3, 30, "Input"],
Cell[31759, 1125, 141, 2, 30, "Input"],
Cell[31903, 1129, 111, 2, 30, "Input"],
Cell[32017, 1133, 228, 4, 30, "Input"],
Cell[32248, 1139, 327, 6, 50, "Input"],
Cell[32578, 1147, 140, 2, 50, "Input"],
Cell[32721, 1151, 140, 2, 50, "Input"],
Cell[32864, 1155, 140, 2, 50, "Input"],
Cell[33007, 1159, 147, 3, 50, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[33203, 1168, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[33253, 1172, 61, 0, 73, "Section"],
Cell[33317, 1174, 46, 1, 30, "Input"],
Cell[33366, 1177, 26, 0, 33, "Text"],
Cell[33395, 1179, 466, 8, 163, "Input"],
Cell[33864, 1189, 33, 0, 33, "Text"],
Cell[33900, 1191, 134, 2, 46, "Input"],
Cell[34037, 1195, 31, 0, 33, "Text"],
Cell[34071, 1197, 67, 1, 42, "Input"],
Cell[34141, 1200, 39, 0, 33, "Text"],
Cell[34183, 1202, 73, 1, 30, "Input"],
Cell[34259, 1205, 31, 0, 33, "Text"],
Cell[34293, 1207, 119, 2, 46, "Input"],
Cell[34415, 1211, 35, 0, 33, "Text"],
Cell[34453, 1213, 72, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34562, 1219, 59, 0, 73, "Section"],
Cell[34624, 1221, 46, 1, 30, "Input"],
Cell[34673, 1224, 27, 0, 33, "Text"],
Cell[34703, 1226, 359, 6, 163, "Input"],
Cell[35065, 1234, 33, 0, 33, "Text"],
Cell[35101, 1236, 134, 2, 46, "Input"],
Cell[35238, 1240, 31, 0, 33, "Text"],
Cell[35272, 1242, 67, 1, 42, "Input"],
Cell[35342, 1245, 39, 0, 33, "Text"],
Cell[35384, 1247, 87, 1, 30, "Input"],
Cell[35474, 1250, 31, 0, 33, "Text"],
Cell[35508, 1252, 119, 2, 46, "Input"],
Cell[35630, 1256, 35, 0, 33, "Text"],
Cell[35668, 1258, 86, 1, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35791, 1264, 46, 0, 73, "Section"],
Cell[35840, 1266, 46, 1, 30, "Input"],
Cell[35889, 1269, 26, 0, 33, "Text"],
Cell[35918, 1271, 638, 11, 252, "Input"],
Cell[36559, 1284, 31, 0, 33, "Text"],
Cell[36593, 1286, 117, 2, 46, "Input"],
Cell[36713, 1290, 25, 0, 33, "Text"],
Cell[36741, 1292, 119, 2, 42, "Input"],
Cell[36863, 1296, 40, 1, 30, "Input"],
Cell[36906, 1299, 32, 0, 33, "Text"],
Cell[36941, 1301, 123, 3, 32, "Input"],
Cell[37067, 1306, 40, 0, 33, "Text"],
Cell[37110, 1308, 156, 3, 44, "Input"],
Cell[37269, 1313, 31, 0, 33, "Text"],
Cell[37303, 1315, 80, 1, 42, "Input"],
Cell[37386, 1318, 33, 0, 33, "Text"],
Cell[37422, 1320, 55, 1, 30, "Input"],
Cell[37480, 1323, 148, 2, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37665, 1330, 46, 0, 73, "Section"],
Cell[37714, 1332, 26, 0, 33, "Text"],
Cell[37743, 1334, 684, 12, 286, "Input"],
Cell[38430, 1348, 31, 0, 33, "Text"],
Cell[38464, 1350, 117, 2, 46, "Input"],
Cell[38584, 1354, 25, 0, 33, "Text"],
Cell[38612, 1356, 119, 2, 42, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38768, 1363, 38, 0, 73, "Section"],
Cell[38809, 1365, 46, 1, 30, "Input"],
Cell[38858, 1368, 26, 0, 33, "Text"],
Cell[38887, 1370, 463, 9, 164, "Input"],
Cell[39353, 1381, 31, 0, 33, "Text"],
Cell[39387, 1383, 116, 2, 46, "Input"],
Cell[39506, 1387, 33, 0, 33, "Text"],
Cell[39542, 1389, 128, 2, 45, "Input"],
Cell[39673, 1393, 30, 0, 33, "Text"],
Cell[39706, 1395, 118, 2, 45, "Input"],
Cell[39827, 1399, 35, 0, 33, "Text"],
Cell[39865, 1401, 64, 1, 30, "Input"],
Cell[39932, 1404, 33, 0, 33, "Text"],
Cell[39968, 1406, 128, 2, 45, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40133, 1413, 40, 0, 73, "Section"],
Cell[40176, 1415, 46, 1, 30, "Input"],
Cell[40225, 1418, 26, 0, 33, "Text"],
Cell[40254, 1420, 426, 8, 147, "Input"],
Cell[40683, 1430, 30, 0, 33, "Text"],
Cell[40716, 1432, 118, 2, 45, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[40883, 1440, 25, 0, 95, "Title"],

Cell[CellGroupData[{
Cell[40933, 1444, 53, 0, 73, "Section"],
Cell[40989, 1446, 46, 1, 30, "Input"],
Cell[41038, 1449, 352, 6, 108, "Input"],
Cell[41393, 1457, 178, 4, 53, "Input"],
Cell[41574, 1463, 111, 2, 30, "Input"],
Cell[41688, 1467, 86, 1, 30, "Input"],
Cell[41777, 1470, 57, 1, 30, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

