(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     11696,        486]*)
(*NotebookOutlinePosition[     12392,        510]*)
(*  CellTagsIndexPosition[     12348,        506]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["B. Curvilinear coordinates", "Title"],

Cell[BoxData[{
    \(<< Lautrup`\), "\[IndentingNewLine]", 
    \(<< Calculus`VectorAnalysis`\)}], "Input",
  InitializationCell->True],

Cell[TextData[{
  "In this notebook it is shown how to use symbolic manipulation to handle \
orthogonal curvilinear coordinates by means of the package \
\"Lautrup`VectorAnalysis`\" which is an improved version of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " package \"Calculus`VectorAnalysis`\". "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cylindrical coordinates", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Define coordinate system", "Section"],

Cell["Set up coordinate system and variables", "Text"],

Cell[BoxData[
    \(SetCoordinates[Cylindrical[r, \[Phi], z]]\)], "Input"],

Cell["Check ranges", "Text"],

Cell[BoxData[
    \(CoordinateRanges[]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coordinate transformations", "Section"],

Cell["\<\
Substitutions of variables to and from the curvilinear coordinates\
\>", "Text"],

Cell[BoxData[
    \(toCurvilinear = 
      Thread[{x, y, z} -> CoordinatesToCartesian[{r, \[Phi], z}]]\)], "Input"],

Cell[BoxData[
    \(toCartesian = 
      Thread[{r, \[Phi], z} \[Rule] 
          CoordinatesFromCartesian[{x, y, z}]]\)], "Input"],

Cell[CellGroupData[{

Cell["Examples", "Subsection"],

Cell[BoxData[
    \(x\^2 + y\^2 + z\^2 /. toCurvilinear // TrigExpand\)], "Input"],

Cell[BoxData[
    \(\(r\^2\) Cos[2  \[Phi]] /. toCartesian\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basis vectors", "Section"],

Cell["\<\
The Jacobian matrix connects Cartesian and curvilinear coordinates\
\>", "Text"],

Cell[BoxData[
    \(jacobian = JacobianMatrix[]\)], "Input"],

Cell[BoxData[
    \(jacobian // MatrixForm\)], "Input"],

Cell["\<\
The scale factors used to normalize the columns of the Jacobian\
\>", "Text"],

Cell[BoxData[
    \(scales = ScaleFactors[]\)], "Input"],

Cell["Curvilinear basis in the Cartesian system", "Text"],

Cell[BoxData[
    \({e\_r, e\_\[Phi], e\_z} = Transpose[jacobian]\/scales\)], "Input"],

Cell["Cartesian basis in the curvilinear system", "Text"],

Cell[BoxData[
    \({e\_x, e\_y, e\_z} = Transpose[{e\_r, e\_\[Phi], e\_z}]\)], "Input"],

Cell[CellGroupData[{

Cell["Examples", "Subsection"],

Cell["Cartesian vector", "Text"],

Cell[BoxData[
    \(\(v = {v\_x, v\_y, v\_z};\)\)], "Input"],

Cell["Transform to curvilinear system", "Text"],

Cell[BoxData[
    \({v . e\_r, v . e\_\[Phi], v . e\_z}\)], "Input"],

Cell["Curvilinear vector", "Text"],

Cell[BoxData[
    \(\(v = {v\_r, v\_\[Phi], v\_z};\)\)], "Input"],

Cell["Transform to Cartesian system", "Text"],

Cell[BoxData[
    \({v . e\_x, v . e\_y, v . e\_z}\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gradient, divergence, and curl", "Section"],

Cell["Scalar field in curvilinear coordinates", "Text"],

Cell[BoxData[
    \(\(s = f[r, \[Phi], z];\)\)], "Input"],

Cell["Gradient of scalar field", "Text"],

Cell[BoxData[
    \(Grad[s]\)], "Input"],

Cell["Vector field in curvilinear coordinates", "Text"],

Cell[BoxData[
    \(\(v = {v\_r[r, \[Phi], z], v\_\[Phi][r, \[Phi], z], 
          v\_z[r, \[Phi], z]};\)\)], "Input"],

Cell["Divergence of vector field", "Text"],

Cell[BoxData[
    \(Div[v] // Expand\)], "Input"],

Cell["Curl of vector field", "Text"],

Cell[BoxData[
    \(Curl[v] // Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Laplacian", "Section"],

Cell["The Laplacian of scalar field ", "Text"],

Cell[BoxData[
    \(Laplacian[s] // Expand\)], "Input"],

Cell["Laplacian of vector field", "Text"],

Cell[BoxData[
    \(\(Laplacian[v] // Expand\) // MatrixForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tensor operations (not included in Calculus`VectorAnalysis`)", "Section"],

Cell["Gradient from the left of a vector", "Text"],

Cell[BoxData[
    \(Grad[v] // MatrixForm\)], "Input"],

Cell["\<\
Divergence of the gradient of a vector should equal the Laplacian\
\>", "Text"],

Cell[BoxData[
    \(\(Div[Grad[v]] // Expand\) // MatrixForm\)], "Input"],

Cell["One may also calculate the inertial acceleration", "Text"],

Cell[BoxData[
    \(\(v . Grad[v] // Expand\) // MatrixForm\)], "Input"],

Cell["General tensor field", "Text"],

Cell[BoxData[
    \(\(t = {{t\_rr[r, \[Phi], z], t\_r\[Phi][r, \[Phi], z], 
            t\_rz[r, \[Phi], z]}, {t\_\[Phi]r[r, \[Phi], z], 
            t\_\[Phi]\[Phi][r, \[Phi], z], 
            t\_\[Phi]z[r, \[Phi], z]}, {t\_zr[r, \[Phi], z], 
            t\_z\[Phi][r, \[Phi], z], t\_zz[r, \[Phi], z]}};\)\)], "Input"],

Cell[BoxData[
    \(t // MatrixForm\)], "Input"],

Cell["\<\
Divergence from left of tensor field (for example the stress tensor)\
\>", "Text"],

Cell[BoxData[
    \(\(Div[t] // Expand\) // MatrixForm\)], "Input"],

Cell[CellGroupData[{

Cell["Example", "Subsection"],

Cell["The stress tensor in a Newtonian incompressible fluid", "Text"],

Cell[BoxData[
    \(Grad[v] + Transpose[Grad[v]] // MatrixForm\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spherical coordinates", "Title"],

Cell[BoxData[
    \(CleanGlobal[]\)], "Input"],

Cell[CellGroupData[{

Cell["Define coordinate system", "Section"],

Cell["Set up coordinate system and variables", "Text"],

Cell[BoxData[
    \(SetCoordinates[Spherical[r, \[Theta], \[Phi]]]\)], "Input"],

Cell["Check ranges", "Text"],

Cell[BoxData[
    \(CoordinateRanges[]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coordinate transformations", "Section"],

Cell["\<\
Substitutions of variables to and from the curvilinear coordinates\
\>", "Text"],

Cell[BoxData[
    \(toCurvilinear = 
      Thread[{x, y, z} -> 
          CoordinatesToCartesian[{r, \[Theta], \[Phi]}]]\)], "Input"],

Cell[BoxData[
    \(toCartesian = 
      Thread[{r, \[Phi], z} \[Rule] 
          CoordinatesFromCartesian[{x, y, z}]]\)], "Input"],

Cell[CellGroupData[{

Cell["Examples", "Subsection"],

Cell[BoxData[
    \(x\^2 + y\^2 + z\^2 /. toCurvilinear // TrigExpand\)], "Input"],

Cell[BoxData[
    \(\(\(r\^2\) Cos[2  \[Phi]] /. toCartesian // TrigExpand\) // 
      Simplify\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basis vectors", "Section"],

Cell["\<\
The Jacobian matrix connects Cartesian and curvilinear coordinates\
\>", "Text"],

Cell[BoxData[
    \(jacobian = JacobianMatrix[]\)], "Input"],

Cell[BoxData[
    \(jacobian // MatrixForm\)], "Input"],

Cell["\<\
The scale factors used to normalize the columns of the Jacobian\
\>", "Text"],

Cell[BoxData[
    \(scales = ScaleFactors[]\)], "Input"],

Cell["Curvilinear basis in the Cartesian system", "Text"],

Cell[BoxData[
    \({e\_r, e\_\[Theta], e\_\[Phi]} = Transpose[jacobian]\/scales\)], "Input"],

Cell["Cartesian basis in the curvilinear system", "Text"],

Cell[BoxData[
    \({e\_x, e\_y, e\_z} = 
      Transpose[{e\_r, e\_\[Theta], e\_\[Phi]}]\)], "Input"],

Cell[CellGroupData[{

Cell["Examples", "Subsection"],

Cell["Cartesian vector", "Text"],

Cell[BoxData[
    \(\(v = {v\_x, v\_y, v\_z};\)\)], "Input"],

Cell["Transform to curvilinear system", "Text"],

Cell[BoxData[
    \({v . e\_r, v . e\_\[Theta], v . e\_\[Phi]}\)], "Input"],

Cell["Curvilinear vector", "Text"],

Cell[BoxData[
    \(\(v = {v\_r, v\_\[Theta], v\_\[Phi]};\)\)], "Input"],

Cell["Transform to Cartesian system", "Text"],

Cell[BoxData[
    \({v . e\_x, v . e\_y, v . e\_z}\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gradient, divergence, and curl", "Section"],

Cell["Scalar field in curvilinear coordinates", "Text"],

Cell[BoxData[
    \(\(s = f[r, \[Theta], \[Phi]];\)\)], "Input"],

Cell["Gradient of scalar field", "Text"],

Cell[BoxData[
    \(Grad[s]\)], "Input"],

Cell["Vector field in curvilinear coordinates", "Text"],

Cell[BoxData[
    \(\(v = {v\_r[r, \[Theta], \[Phi]], v\_\[Theta][r, \[Theta], \[Phi]], 
          v\_\[Phi][r, \[Theta], \[Phi]]};\)\)], "Input"],

Cell["Divergence of vector field", "Text"],

Cell[BoxData[
    \(Div[v] // Expand\)], "Input"],

Cell["Curl of vector field", "Text"],

Cell[BoxData[
    \(Curl[v] // Expand\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Laplacian", "Section"],

Cell["The Laplacian of scalar field ", "Text"],

Cell[BoxData[
    \(Laplacian[s] // Expand\)], "Input"],

Cell["Laplacian of vector field", "Text"],

Cell[BoxData[
    \(\(Laplacian[v] // Expand\) // MatrixForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tensor operations (not included in Calculus`VectorAnalysis`)", "Section"],

Cell["Gradient from the left of a vector", "Text"],

Cell[BoxData[
    \(Grad[v] // MatrixForm\)], "Input"],

Cell["\<\
Divergence of the gradient of a vector should equal the Laplacian\
\>", "Text"],

Cell[BoxData[
    \(\(Div[Grad[v]] // Expand\) // MatrixForm\)], "Input"],

Cell["One may also calculate the inertial acceleration", "Text"],

Cell[BoxData[
    \(\(v . Grad[v] // Expand\) // MatrixForm\)], "Input"],

Cell["General tensor field", "Text"],

Cell[BoxData[
    \(\(t = {{t\_rr[r, \[Theta], \[Phi]], t\_r\[Phi][r, \[Theta], \[Phi]], 
            t\_rz[r, \[Theta], \[Phi]]}, {t\_\[Phi]r[r, \[Theta], \[Phi]], 
            t\_\[Phi]\[Phi][r, \[Theta], \[Phi]], 
            t\_\[Phi]z[r, \[Theta], \[Phi]]}, {t\_zr[r, \[Theta], \[Phi]], 
            t\_z\[Phi][r, \[Theta], \[Phi]], 
            t\_zz[r, \[Theta], \[Phi]]}};\)\)], "Input"],

Cell[BoxData[
    \(t // MatrixForm\)], "Input"],

Cell["\<\
Divergence from left of tensor field (for example the stress tensor)\
\>", "Text"],

Cell[BoxData[
    \(\(Div[t] // Expand\) // MatrixForm\)], "Input"],

Cell[CellGroupData[{

Cell["Example", "Subsection"],

Cell["The stress tensor in a Newtonian incompressible fluid", "Text"],

Cell[BoxData[
    \(Grad[v] + Transpose[Grad[v]] // MatrixForm\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->None,
WindowSize->{1005, 740},
WindowMargins->{{133, Automatic}, {Automatic, 76}},
ShowSelection->True
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 43, 0, 95, "Title"],
Cell[1822, 55, 135, 3, 50, "Input",
  InitializationCell->True],
Cell[1960, 60, 328, 7, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2325, 72, 40, 0, 95, "Title"],
Cell[2368, 74, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2439, 79, 43, 0, 73, "Section"],
Cell[2485, 81, 54, 0, 33, "Text"],
Cell[2542, 83, 74, 1, 30, "Input"],
Cell[2619, 86, 28, 0, 33, "Text"],
Cell[2650, 88, 51, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2738, 94, 45, 0, 73, "Section"],
Cell[2786, 96, 90, 2, 33, "Text"],
Cell[2879, 100, 115, 2, 30, "Input"],
Cell[2997, 104, 131, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[3153, 111, 30, 0, 38, "Subsection"],
Cell[3186, 113, 82, 1, 30, "Input"],
Cell[3271, 116, 71, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3391, 123, 32, 0, 73, "Section"],
Cell[3426, 125, 90, 2, 33, "Text"],
Cell[3519, 129, 60, 1, 30, "Input"],
Cell[3582, 132, 55, 1, 30, "Input"],
Cell[3640, 135, 87, 2, 33, "Text"],
Cell[3730, 139, 56, 1, 30, "Input"],
Cell[3789, 142, 57, 0, 33, "Text"],
Cell[3849, 144, 86, 1, 42, "Input"],
Cell[3938, 147, 57, 0, 33, "Text"],
Cell[3998, 149, 88, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[4111, 154, 30, 0, 38, "Subsection"],
Cell[4144, 156, 32, 0, 33, "Text"],
Cell[4179, 158, 60, 1, 30, "Input"],
Cell[4242, 161, 47, 0, 33, "Text"],
Cell[4292, 163, 68, 1, 30, "Input"],
Cell[4363, 166, 34, 0, 33, "Text"],
Cell[4400, 168, 65, 1, 30, "Input"],
Cell[4468, 171, 45, 0, 33, "Text"],
Cell[4516, 173, 63, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4628, 180, 49, 0, 73, "Section"],
Cell[4680, 182, 55, 0, 33, "Text"],
Cell[4738, 184, 57, 1, 30, "Input"],
Cell[4798, 187, 40, 0, 33, "Text"],
Cell[4841, 189, 40, 1, 30, "Input"],
Cell[4884, 192, 55, 0, 33, "Text"],
Cell[4942, 194, 118, 2, 30, "Input"],
Cell[5063, 198, 42, 0, 33, "Text"],
Cell[5108, 200, 49, 1, 30, "Input"],
Cell[5160, 203, 36, 0, 33, "Text"],
Cell[5199, 205, 50, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5286, 211, 28, 0, 73, "Section"],
Cell[5317, 213, 46, 0, 33, "Text"],
Cell[5366, 215, 55, 1, 30, "Input"],
Cell[5424, 218, 41, 0, 33, "Text"],
Cell[5468, 220, 73, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5578, 226, 79, 0, 73, "Section"],
Cell[5660, 228, 50, 0, 33, "Text"],
Cell[5713, 230, 54, 1, 30, "Input"],
Cell[5770, 233, 89, 2, 33, "Text"],
Cell[5862, 237, 73, 1, 30, "Input"],
Cell[5938, 240, 64, 0, 33, "Text"],
Cell[6005, 242, 72, 1, 30, "Input"],
Cell[6080, 245, 36, 0, 33, "Text"],
Cell[6119, 247, 319, 5, 30, "Input"],
Cell[6441, 254, 48, 1, 30, "Input"],
Cell[6492, 257, 92, 2, 33, "Text"],
Cell[6587, 261, 67, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[6679, 266, 29, 0, 38, "Subsection"],
Cell[6711, 268, 69, 0, 33, "Text"],
Cell[6783, 270, 75, 1, 30, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6919, 278, 38, 0, 95, "Title"],
Cell[6960, 280, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[7031, 285, 43, 0, 73, "Section"],
Cell[7077, 287, 54, 0, 33, "Text"],
Cell[7134, 289, 79, 1, 30, "Input"],
Cell[7216, 292, 28, 0, 33, "Text"],
Cell[7247, 294, 51, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7335, 300, 45, 0, 73, "Section"],
Cell[7383, 302, 90, 2, 33, "Text"],
Cell[7476, 306, 133, 3, 30, "Input"],
Cell[7612, 311, 131, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[7768, 318, 30, 0, 38, "Subsection"],
Cell[7801, 320, 82, 1, 30, "Input"],
Cell[7886, 323, 108, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8043, 331, 32, 0, 73, "Section"],
Cell[8078, 333, 90, 2, 33, "Text"],
Cell[8171, 337, 60, 1, 30, "Input"],
Cell[8234, 340, 55, 1, 30, "Input"],
Cell[8292, 343, 87, 2, 33, "Text"],
Cell[8382, 347, 56, 1, 30, "Input"],
Cell[8441, 350, 57, 0, 33, "Text"],
Cell[8501, 352, 93, 1, 42, "Input"],
Cell[8597, 355, 57, 0, 33, "Text"],
Cell[8657, 357, 102, 2, 30, "Input"],

Cell[CellGroupData[{
Cell[8784, 363, 30, 0, 38, "Subsection"],
Cell[8817, 365, 32, 0, 33, "Text"],
Cell[8852, 367, 60, 1, 30, "Input"],
Cell[8915, 370, 47, 0, 33, "Text"],
Cell[8965, 372, 75, 1, 30, "Input"],
Cell[9043, 375, 34, 0, 33, "Text"],
Cell[9080, 377, 72, 1, 30, "Input"],
Cell[9155, 380, 45, 0, 33, "Text"],
Cell[9203, 382, 63, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9315, 389, 49, 0, 73, "Section"],
Cell[9367, 391, 55, 0, 33, "Text"],
Cell[9425, 393, 64, 1, 30, "Input"],
Cell[9492, 396, 40, 0, 33, "Text"],
Cell[9535, 398, 40, 1, 30, "Input"],
Cell[9578, 401, 55, 0, 33, "Text"],
Cell[9636, 403, 146, 2, 30, "Input"],
Cell[9785, 407, 42, 0, 33, "Text"],
Cell[9830, 409, 49, 1, 30, "Input"],
Cell[9882, 412, 36, 0, 33, "Text"],
Cell[9921, 414, 50, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10008, 420, 28, 0, 73, "Section"],
Cell[10039, 422, 46, 0, 33, "Text"],
Cell[10088, 424, 55, 1, 30, "Input"],
Cell[10146, 427, 41, 0, 33, "Text"],
Cell[10190, 429, 73, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10300, 435, 79, 0, 73, "Section"],
Cell[10382, 437, 50, 0, 33, "Text"],
Cell[10435, 439, 54, 1, 30, "Input"],
Cell[10492, 442, 89, 2, 33, "Text"],
Cell[10584, 446, 73, 1, 30, "Input"],
Cell[10660, 449, 64, 0, 33, "Text"],
Cell[10727, 451, 72, 1, 30, "Input"],
Cell[10802, 454, 36, 0, 33, "Text"],
Cell[10841, 456, 395, 6, 30, "Input"],
Cell[11239, 464, 48, 1, 30, "Input"],
Cell[11290, 467, 92, 2, 33, "Text"],
Cell[11385, 471, 67, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[11477, 476, 29, 0, 38, "Subsection"],
Cell[11509, 478, 69, 0, 33, "Text"],
Cell[11581, 480, 75, 1, 30, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

