(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     11131,        297]*)
(*NotebookOutlinePosition[     11883,        322]*)
(*  CellTagsIndexPosition[     11839,        318]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[
    \(\(\( (*\ 
      strangely\ the\ two\ following\ packages\ do\ not\ protect\ their\ \
symbols\ *) \)\(\[IndentingNewLine]\)\(\(Begin["\<Miscellaneous`SIUnits`\>"];\
\)\[IndentingNewLine]
    \(Protect[Evaluate[ToExpression[Names["\<`*\>"]]]];\)\n
    \(End[];\)\)\)\)], "Input",
  CellLabel->"In[56]:=",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Begin["\<Miscellaneous`Units`\>"];\)\), "\[IndentingNewLine]", 
    \(\(Protect[Evaluate[ToExpression[Names["\<`*\>"]]]];\)\), "\n", 
    \(\(End[];\)\)}], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Unit\ handling\ package\ *) \)\(\n\
\)\(\(BeginPackage["\<Lautrup`Units`\>", {"\<Miscellaneous`SIUnits`\>", \
"\<Miscellaneous`Units`\>"}];\)\[IndentingNewLine]\n (*\ 
      secure\ reload\ *) \[IndentingNewLine]
    \(Unprotect[
        Evaluate[ToExpression[Names["\<`*\>"]]]];\)\[IndentingNewLine]
    \(ClearAttributes[{NToCGS, NToMKS, NToSI, NToUnit, ToCGS, ToMKS, ToSI, 
          ToUnit, Unit, UnitQ}, 
        ReadProtected];\)\ \[IndentingNewLine]\[IndentingNewLine] (*\ 
      Usage\ of\ unit\ functions\ *) \n\[IndentingNewLine]
    \(UnitQ::usage = "\<UnitQ[s] is true if a symbol is a unit\>";\)\n
    \(Unit::usage = "\<Unit[q] is the unit of a quantity q\>";\)\n
    \(ToSI::usage = "\<ToSI[q] converts a quantity q to fundamental SI \
units\>";\)\[IndentingNewLine]
    \(ToMKS::usage = "\<ToMKS[q] converts a quantity q to MKS units\>";\)\n
    \(ToCGS::usage = "\<ToCGS[q] converts a quantity q to CGS units\>";\)\n
    \(ToUnit::usage = "\<ToUnit[u][q] converts a quantity q to unit u\>";\)\n
    \(NToSI::usage = "\<NToSI[q] is the value of a quantity q in SI units\>";\
\)\n
    \(NToMKS::usage = "\<NToMKS[q] is the value of a quantity q in MKS \
units\>";\)\n
    \(NToCGS::usage = "\<NToCGS[q] is the value of a quantity q in CGS \
units\>";\)\n
    \(NToUnit::usage = "\<NToUnit[q] is the value of a quantity q in units of \
u\>";\)\[IndentingNewLine]\[IndentingNewLine]
    \(Kilometer::usage = "\<Kilometer is a thousand meter\>";\)\
\[IndentingNewLine]
    \(Millimeter::usage = "\<Millimeter is a thousands of a meter\>";\)\
\[IndentingNewLine]
    \(Micrometer::usage = "\<Micrometer is a millionth of a meter\>";\)\
\[IndentingNewLine]
    \(Nanometer::usage = "\<Nanometer is a billionth of a meter\>";\)\n
    \(RPM::usage = "\<Rotations per minute\>";\)\[IndentingNewLine]
    \(Millisecond::usage = "\<Millisecond is a thousandth of a second\>";\)\
\[IndentingNewLine]
    \(Microsecond::usage = "\<Microsecond is a millionth of a second\>";\)\
\[IndentingNewLine]
    \(Nanosecond::usage = "\<Nanosecond is a billionth of a second\>";\)\
\[IndentingNewLine]
    \(SiderealDay::usage = "\<SiderealDay is a unit of time\>";\)\
\[IndentingNewLine]\[IndentingNewLine]
    \(Begin["\<`Private`\>"];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      create\ list\ of\ all\ known\ units\ *) \n
    \(special = {Second, Newton};\)\n
    \(extra = {Millimeter, Kilometer, Micrometer, Nanometer, RPM, 
          Millisecond, Microsecond, Nanosecond, 
          SiderealDay};\)\[IndentingNewLine]
    \(units = 
        ToExpression[
          Join[Names["\<Miscellaneous`Units`*\>"], 
            Names["\<Miscellaneous`SIUnits`*\>"], special, extra]];\)\n
    \(exceptions = {CGS, Convert, ConvertTemperature, MKS, SI};\)\n
    \(units = 
        Select[Union[
            units], \(! MemberQ[
                exceptions, #]\) &];\)\[IndentingNewLine]\[IndentingNewLine] \
(*\ Define\ logical\ function\ UnitQ\ to\ be\ true\ for\ any\ recognized\ \
unit\ name\ *) \[IndentingNewLine]
    \(Clear[UnitQ];\)\n
    \(Map[\((Evaluate[UnitQ[#]] = True)\)\  &, units];\)\n
    UnitQ[x_\ y_] := UnitQ[x] && UnitQ[y]\n
    UnitQ[Power[x_, _]] := UnitQ[x]\n
    \(UnitQ[_] = False;\)\[IndentingNewLine]\n (*\ 
      Isolate\ unit\ from\ expression\ *) \t\t\n
    Unit[x_?UnitQ] := x\n
    Unit[x_?NumericQ] := 1\n
    Unit[x_\ y_] := Unit[x] Unit[y]\n\[IndentingNewLine] (*\ 
      General\ conversion\ function\ *) \[IndentingNewLine]
    \($ToIntermediate = {Millimeter \[Rule] Milli\ Meter, 
          Kilometer \[Rule] Kilo\ Meter, Micrometer \[Rule] Micro\ Meter, 
          Nanometer \[Rule] Nano\ Meter, RPM \[Rule] 2  \[Pi]/Minute, 
          Millisecond \[Rule] Milli\ Second, 
          Microsecond \[Rule] Micro\ Second, Nanosecond \[Rule] Nano\ Second, 
          SiderealDay \[Rule] 
            Day*SiderealSecond/Second};\)\[IndentingNewLine]
    ToSI[q_] := \ 
      Simplify[PowerExpand[
          SI[q //. $ToIntermediate] //. 
            Miscellaneous`Units`Private`$ToFundamental]]\[IndentingNewLine]
    ToMKS[q_] := ToSI[q]\n
    ToCGS[q_] := PowerExpand[CGS[ToSI[q]]]\n
    \(ToUnit[u_?UnitQ]\)[q_] := 
      ToSI[q/u] u\ \[IndentingNewLine]\[IndentingNewLine] (*\ 
      Numeric\ conversion\ functions\ *) \[IndentingNewLine]
    NToSI[q_] := With[{r = ToSI[q]}, N[r]/Unit[r]]\[IndentingNewLine]
    NToMKS[q_] := NToSI[q]\n
    NToCGS[q_] := With[{r = ToCGS[q]}, N[r]/Unit[r]]\n
    \(NToUnit[u_]\)[q_] := NToSI[q/u]\n
    \(End[];\)\[IndentingNewLine]\[IndentingNewLine] (*\ protect\ all\ *) \n
    \(Protect[Evaluate[ToExpression[Names["\<`*\>"]]]];\)\[IndentingNewLine]
    \(SetAttributes[{NToCGS, NToMKS, NToSI, NToUnit, ToCGS, ToMKS, ToSI, 
          ToUnit, Unit, UnitQ}, ReadProtected];\)\ \[IndentingNewLine]
    \ \(EndPackage[];\)\)\)\)], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Documentation", "Title"],

Cell["\<\
This package is an addition to the normal Miscellaneous`Units` package. It \
introduces a number of new functions that are useful for dealing with units \
of physical quantities.\
\>", "Text",
  FontWeight->"Plain",
  FontVariations->{"CompatibilityType"->0}],

Cell["The basic functions that manipulate units are", "Text",
  CellFrame->{{0, 0}, {0, 0.5}}],

Cell["\<\
\t\t\tUnitQ[x]                          determines if x is a unit
\t\t\tUnit[x]                            extracts  the unit from a quantity x\
\
\>", "Text",
  CellFrame->True,
  Background->RGBColor[0.992203, 0.640635, 0.617197]],

Cell["This determines if meter is a unit", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(UnitQ[Meter]\)], "Input"],

Cell["and this shows that composites are units", "Text"],

Cell[BoxData[
    \(UnitQ[Meter/Second^2]\)], "Input"],

Cell["\<\
But there are things that are not units, for example a string \"Meter\"\
\>", "Text"],

Cell[BoxData[
    \(UnitQ["\<Meter\>"]\)], "Input"],

Cell["The unit may be isolated from a quantity ", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(Unit[2.45  Meter]\)], "Input"],

Cell[BoxData[
    \(Unit[23\ Meter/Second^2]\)], "Input"],

Cell["\<\
Conversion of quantities to special units is accomplished by the functions\
\>", "Text",
  CellFrame->{{0, 0}, {0, 0.5}}],

Cell[BoxData[
    RowBox[{"\t\t\t ", 
      RowBox[{\(ToSI[
            x]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ converts\ \
a\ quantity\ to\ SI\ units\), "\[IndentingNewLine]", "\t\t\t", GridBox[{
            {\(ToMKS[
                x]\), \(\(\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\)\(converts\ a\ quantity\ to\ MKS\ units\)\)}
            }], "\n", 
        "\t\t\t ", \(ToCGS[
            x]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ converts\ a\ \
quantity\ to\ CGS\ units\), "\[IndentingNewLine]", 
        "\t\t\t ", \(\(ToUnit[u]\)[
            x]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ converts\ a\ quantity\ x\ to\ \
unit\ u\)}]}]], "Text",
  CellFrame->True,
  Background->RGBColor[0.992203, 0.640635, 0.617197]],

Cell["This converts 64 miles per hour to meters per second", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(64\ Mile/Hour // ToSI\)], "Input"],

Cell["This converts  it to centimeters per second", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(% // ToCGS\)], "Input"],

Cell["\<\
It is also possible to convert it to lightyears per billion years\
\>", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(% // ToUnit[LightYear/\((Giga\ Year)\)]\)], "Input"],

Cell["\<\
Numeric conversions are always done on top of a unit conversion, because the \
value of a quantity is totally dependent on its unit\
\>", "Text",
  CellFrame->{{0, 0}, {0, 0.5}}],

Cell["\<\
\t\t\tNToSI[x]                           converts a quantity to SI units and \
drops the unit
\t\t\tNToMKS[x]                         converts a quantity to MKS units and \
drops the unit
\t\t\tNToCGS[x]                         converts a quantity to CGS units and \
drops the unit
\t\t\tNToUnit[u][x]                converts a quantity x to unit u and drops \
the unit\
\>", "Text",
  CellFrame->True,
  Background->RGBColor[0.992203, 0.640635, 0.617197]],

Cell["Convert 64 miles per hour to SI units and drop the unit", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(64\ Mile/Hour // NToSI\)], "Input"],

Cell["Likewise to CGS units (notice you can't use % reference)", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(64\ Mile/Hour // NToCGS\)], "Input"],

Cell["And to lightyears per billion years", "Text",
  CellDingbat->"\[FilledSmallSquare]"],

Cell[BoxData[
    \(64\ Mile/Hour // NToUnit[LightYear/\((Giga\ Year)\)]\)], "Input"]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
AutoGeneratedPackage->Automatic,
ScreenStyleEnvironment->"Working",
WindowSize->{808, 735},
WindowMargins->{{197, Automatic}, {38, Automatic}},
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 350, 8, 89, "Input",
  InitializationCell->True],
Cell[2107, 61, 239, 5, 69, "Input",
  InitializationCell->True],
Cell[2349, 68, 4891, 96, 1529, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[7265, 168, 30, 0, 105, "Title"],
Cell[7298, 170, 269, 6, 42, "Text"],
Cell[7570, 178, 94, 1, 33, "Text"],
Cell[7667, 181, 242, 6, 66, "Text"],
Cell[7912, 189, 89, 1, 25, "Text"],
Cell[8004, 192, 45, 1, 29, "Input"],
Cell[8052, 195, 56, 0, 25, "Text"],
Cell[8111, 197, 54, 1, 29, "Input"],
Cell[8168, 200, 95, 2, 25, "Text"],
Cell[8266, 204, 51, 1, 29, "Input"],
Cell[8320, 207, 96, 1, 25, "Text"],
Cell[8419, 210, 50, 1, 29, "Input"],
Cell[8472, 213, 57, 1, 29, "Input"],
Cell[8532, 216, 131, 3, 33, "Text"],
Cell[8666, 221, 744, 16, 98, "Text"],
Cell[9413, 239, 107, 1, 25, "Text"],
Cell[9523, 242, 54, 1, 29, "Input"],
Cell[9580, 245, 98, 1, 25, "Text"],
Cell[9681, 248, 43, 1, 29, "Input"],
Cell[9727, 251, 128, 3, 25, "Text"],
Cell[9858, 256, 72, 1, 29, "Input"],
Cell[9933, 259, 188, 4, 33, "Text"],
Cell[10124, 265, 466, 11, 116, "Text"],
Cell[10593, 278, 110, 1, 25, "Text"],
Cell[10706, 281, 55, 1, 29, "Input"],
Cell[10764, 284, 111, 1, 25, "Text"],
Cell[10878, 287, 56, 1, 29, "Input"],
Cell[10937, 290, 90, 1, 25, "Text"],
Cell[11030, 293, 85, 1, 29, "Input"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

